@@ -16,6 +16,7 @@
 
 """Heat API exception subclasses - maps API response errors to AWS Errors."""
 
+from oslo_utils import reflection
 import six
 import webob.exc
 
@@ -30,7 +31,7 @@ class HeatAPIException(webob.exc.HTTPError):
     Subclass webob HTTPError so we can correctly serialize the wsgi response
     into the http response body, using the format specified by the request.
     Note this should not be used directly, instead use the subclasses
-    defined below which map to AWS API errors
+    defined below which map to AWS API errors.
     """
 
     code = 400
@@ -189,7 +190,7 @@ class HeatOptInRequiredError(HeatAPIException):
 
 class HeatRequestExpiredError(HeatAPIException):
 
-    """Request expired or more than 15mins in the future.
+    """Request expired or more than 15 minutes in the future.
 
     Request is past expires date or the request date (either with 15 minute
     padding), or the request date occurs more than 15 minutes in the future.
@@ -273,6 +274,15 @@ class HeatActionInProgressError(HeatAPIException):
                    "in progress")
 
 
+class HeatRequestLimitExceeded(HeatAPIException):
+
+    """Payload size of the request exceeds maximum allowed size."""
+
+    code = 400
+    title = 'RequestLimitExceeded'
+    explanation = _("Payload exceeds maximum allowed size")
+
+
 def map_remote_error(ex):
     """Map rpc_common.RemoteError exceptions to HeatAPIException subclasses.
 
@@ -284,12 +294,10 @@ def map_remote_error(ex):
         'AttributeError',
         'ValueError',
         'InvalidTenant',
-        'StackNotFound',
+        'EntityNotFound',
         'ResourceActionNotSupported',
         'ResourceNotFound',
         'ResourceNotAvailable',
-        'ResourceTypeNotFound',
-        'PhysicalResourceNotFound',
         'WatchRuleNotFound',
         'StackValidationFailed',
         'InvalidSchemaError',
@@ -308,8 +316,9 @@ def map_remote_error(ex):
     denied_errors = ('Forbidden', 'NotAuthorized')
     already_exists_errors = ('StackExists')
     invalid_action_errors = ('ActionInProgress',)
+    request_limit_exceeded = ('RequestLimitExceeded')
 
-    ex_type = ex.__class__.__name__
+    ex_type = reflection.get_class_name(ex, fully_qualified=False)
 
     if ex_type.endswith('_Remote'):
         ex_type = ex_type[:-len('_Remote')]
@@ -322,6 +331,8 @@ def map_remote_error(ex):
         return AlreadyExistsError(detail=six.text_type(ex))
     elif ex_type in invalid_action_errors:
         return HeatActionInProgressError(detail=six.text_type(ex))
+    elif ex_type in request_limit_exceeded:
+        return HeatRequestLimitExceeded(detail=six.text_type(ex))
     else:
         # Map everything else to internal server error for now
         return HeatInternalFailureError(detail=six.text_type(ex))
