@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from debtcollector import removals
 from heat.api.middleware import fault
 from heat.api.middleware import ssl
 from heat.api.middleware import version_negotiation as vn
@@ -26,5 +27,7 @@ def faultwrap_filter(app, conf, **local_conf):
     return fault.FaultWrapper(app)
 
 
+@removals.remove(message='Use oslo_middleware.http_proxy_to_wsgi instead.',
+                 version='6.0.0', removal_version='8.0.0')
 def sslmiddleware_filter(app, conf, **local_conf):
     return ssl.SSLMiddleware(app)
