@@ -11,9 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Stack endpoint for Heat v1 REST API.
-"""
+"""Stack endpoint for Heat v1 REST API."""
 
 import contextlib
 from oslo_log import log as logging
@@ -39,8 +37,9 @@ LOG = logging.getLogger(__name__)
 
 
 class InstantiationData(object):
-    """
-    The data accompanying a PUT or POST request to create or update a stack.
+    """The data to create or update a stack.
+
+    The data accompanying a PUT or POST request.
     """
 
     PARAMS = (
@@ -50,6 +49,7 @@ class InstantiationData(object):
         PARAM_USER_PARAMS,
         PARAM_ENVIRONMENT,
         PARAM_FILES,
+        PARAM_ENVIRONMENT_FILES,
     ) = (
         'stack_name',
         'template',
@@ -57,11 +57,12 @@ class InstantiationData(object):
         'parameters',
         'environment',
         'files',
+        'environment_files',
     )
 
     def __init__(self, data, patch=False):
-        """
-        Initialise from the request object.
+        """Initialise from the request object.
+
         If called from the PATCH api, insert a flag for the engine code
         to distinguish.
         """
@@ -81,15 +82,14 @@ class InstantiationData(object):
             raise exc.HTTPBadRequest(msg)
 
     def stack_name(self):
-        """
-        Return the stack name.
-        """
+        """Return the stack name."""
         if self.PARAM_STACK_NAME not in self.data:
             raise exc.HTTPBadRequest(_("No stack name specified"))
         return self.data[self.PARAM_STACK_NAME]
 
     def template(self):
-        """
+        """Get template file contents.
+
         Get template file contents, either inline, from stack adopt data or
         from a URL, in JSON or YAML format.
         """
@@ -98,6 +98,8 @@ class InstantiationData(object):
             adopt_data = self.data[rpc_api.PARAM_ADOPT_STACK_DATA]
             try:
                 adopt_data = template_format.simple_parse(adopt_data)
+                template_format.validate_template_limit(
+                    six.text_type(adopt_data['template']))
                 return adopt_data['template']
             except (ValueError, KeyError) as ex:
                 err_reason = _('Invalid adopt data: %s') % ex
@@ -105,7 +107,10 @@ class InstantiationData(object):
         elif self.PARAM_TEMPLATE in self.data:
             template_data = self.data[self.PARAM_TEMPLATE]
             if isinstance(template_data, dict):
+                template_format.validate_template_limit(six.text_type(
+                    template_data))
                 return template_data
+
         elif self.PARAM_TEMPLATE_URL in self.data:
             url = self.data[self.PARAM_TEMPLATE_URL]
             LOG.debug('TemplateUrl %s' % url)
@@ -125,8 +130,8 @@ class InstantiationData(object):
             return template_format.parse(template_data)
 
     def environment(self):
-        """
-        Get the user-supplied environment for the stack in YAML format.
+        """Get the user-supplied environment for the stack in YAML format.
+
         If the user supplied Parameters then merge these into the
         environment global options.
         """
@@ -147,18 +152,19 @@ class InstantiationData(object):
     def files(self):
         return self.data.get(self.PARAM_FILES, {})
 
+    def environment_files(self):
+        return self.data.get(self.PARAM_ENVIRONMENT_FILES, None)
+
     def args(self):
-        """
-        Get any additional arguments supplied by the user.
-        """
+        """Get any additional arguments supplied by the user."""
         params = self.data.items()
         return dict((k, v) for k, v in params if k not in self.PARAMS)
 
 
 class StackController(object):
-    """
-    WSGI controller for stacks resource in Heat v1 API
-    Implements the API actions
+    """WSGI controller for stacks resource in Heat v1 API.
+
+    Implements the API actions.
     """
     # Define request scope (must match what is in policy.json)
     REQUEST_SCOPE = 'stacks'
@@ -194,29 +200,29 @@ class StackController(object):
         filter_whitelist = {
             # usage of keys in this list are not encouraged, please use
             # rpc_api.STACK_KEYS instead
-            'id': 'mixed',
-            'status': 'mixed',
-            'name': 'mixed',
-            'action': 'mixed',
-            'tenant': 'mixed',
-            'username': 'mixed',
-            'owner_id': 'mixed',
+            'id': util.PARAM_TYPE_MIXED,
+            'status': util.PARAM_TYPE_MIXED,
+            'name': util.PARAM_TYPE_MIXED,
+            'action': util.PARAM_TYPE_MIXED,
+            'tenant': util.PARAM_TYPE_MIXED,
+            'username': util.PARAM_TYPE_MIXED,
+            'owner_id': util.PARAM_TYPE_MIXED,
         }
         whitelist = {
-            'limit': 'single',
-            'marker': 'single',
-            'sort_dir': 'single',
-            'sort_keys': 'multi',
-            'show_deleted': 'single',
-            'show_nested': 'single',
-            'show_hidden': 'single',
-            'tags': 'single',
-            'tags_any': 'single',
-            'not_tags': 'single',
-            'not_tags_any': 'single',
+            'limit': util.PARAM_TYPE_SINGLE,
+            'marker': util.PARAM_TYPE_SINGLE,
+            'sort_dir': util.PARAM_TYPE_SINGLE,
+            'sort_keys': util.PARAM_TYPE_MULTI,
+            'show_deleted': util.PARAM_TYPE_SINGLE,
+            'show_nested': util.PARAM_TYPE_SINGLE,
+            'show_hidden': util.PARAM_TYPE_SINGLE,
+            'tags': util.PARAM_TYPE_SINGLE,
+            'tags_any': util.PARAM_TYPE_SINGLE,
+            'not_tags': util.PARAM_TYPE_SINGLE,
+            'not_tags_any': util.PARAM_TYPE_SINGLE,
         }
         params = util.get_allowed_params(req.params, whitelist)
-        stack_keys = dict.fromkeys(rpc_api.STACK_KEYS, 'mixed')
+        stack_keys = dict.fromkeys(rpc_api.STACK_KEYS, util.PARAM_TYPE_MIXED)
         unsupported = (
             rpc_api.STACK_ID,  # not user visible
             rpc_api.STACK_CAPABILITIES,  # not supported
@@ -313,7 +319,7 @@ class StackController(object):
                                                      not_tags=not_tags,
                                                      not_tags_any=not_tags_any)
             except AttributeError as ex:
-                LOG.warn(_LW("Old Engine Version: %s"), ex)
+                LOG.warning(_LW("Old Engine Version: %s"), ex)
 
         return stacks_view.collection(req, stacks=stacks, count=count,
                                       tenant_safe=tenant_safe)
@@ -324,9 +330,7 @@ class StackController(object):
 
     @util.policy_enforce
     def index(self, req):
-        """
-        Lists summary information for all stacks
-        """
+        """Lists summary information for all stacks."""
         global_tenant = False
         name = rpc_api.PARAM_GLOBAL_TENANT
         if name in req.params:
@@ -341,27 +345,26 @@ class StackController(object):
 
     @util.policy_enforce
     def detail(self, req):
-        """
-        Lists detailed information for all stacks
-        """
+        """Lists detailed information for all stacks."""
         stacks = self.rpc_client.list_stacks(req.context)
 
         return {'stacks': [stacks_view.format_stack(req, s) for s in stacks]}
 
     @util.policy_enforce
     def preview(self, req, body):
-        """
-        Preview the outcome of a template and its params
-        """
+        """Preview the outcome of a template and its params."""
 
         data = InstantiationData(body)
         args = self.prepare_args(data)
-        result = self.rpc_client.preview_stack(req.context,
-                                               data.stack_name(),
-                                               data.template(),
-                                               data.environment(),
-                                               data.files(),
-                                               args)
+        result = self.rpc_client.preview_stack(
+            req.context,
+            data.stack_name(),
+            data.template(),
+            data.environment(),
+            data.files(),
+            args,
+            environment_files=data.environment_files()
+        )
 
         formatted_stack = stacks_view.format_stack(req, result)
         return {'stack': formatted_stack}
@@ -378,18 +381,18 @@ class StackController(object):
 
     @util.policy_enforce
     def create(self, req, body):
-        """
-        Create a new stack
-        """
+        """Create a new stack."""
         data = InstantiationData(body)
 
         args = self.prepare_args(data)
-        result = self.rpc_client.create_stack(req.context,
-                                              data.stack_name(),
-                                              data.template(),
-                                              data.environment(),
-                                              data.files(),
-                                              args)
+        result = self.rpc_client.create_stack(
+            req.context,
+            data.stack_name(),
+            data.template(),
+            data.environment(),
+            data.files(),
+            args,
+            environment_files=data.environment_files())
 
         formatted_stack = stacks_view.format_stack(
             req,
@@ -399,9 +402,7 @@ class StackController(object):
 
     @util.policy_enforce
     def lookup(self, req, stack_name, path='', body=None):
-        """
-        Redirect to the canonical URL for a stack
-        """
+        """Redirect to the canonical URL for a stack."""
         try:
             identity = dict(identifier.HeatIdentifier.from_arn(stack_name))
         except ValueError:
@@ -420,12 +421,17 @@ class StackController(object):
 
     @util.identified_stack
     def show(self, req, identity):
-        """
-        Gets detailed information for a stack
-        """
+        """Gets detailed information for a stack."""
+        params = req.params
 
+        p_name = rpc_api.RESOLVE_OUTPUTS
+        if rpc_api.RESOLVE_OUTPUTS in params:
+            resolve_outputs = self._extract_bool_param(
+                p_name, params[p_name])
+        else:
+            resolve_outputs = True
         stack_list = self.rpc_client.show_stack(req.context,
-                                                identity)
+                                                identity, resolve_outputs)
 
         if not stack_list:
             raise exc.HTTPInternalServerError()
@@ -436,9 +442,7 @@ class StackController(object):
 
     @util.identified_stack
     def template(self, req, identity):
-        """
-        Get the template body for an existing stack
-        """
+        """Get the template body for an existing stack."""
 
         templ = self.rpc_client.get_template(req.context,
                                              identity)
@@ -451,54 +455,67 @@ class StackController(object):
 
     @util.identified_stack
     def update(self, req, identity, body):
-        """
-        Update an existing stack with a new template and/or parameters
-        """
+        """Update an existing stack with a new template and/or parameters."""
         data = InstantiationData(body)
 
         args = self.prepare_args(data)
-        self.rpc_client.update_stack(req.context,
-                                     identity,
-                                     data.template(),
-                                     data.environment(),
-                                     data.files(),
-                                     args)
+        self.rpc_client.update_stack(
+            req.context,
+            identity,
+            data.template(),
+            data.environment(),
+            data.files(),
+            args,
+            environment_files=data.environment_files())
 
         raise exc.HTTPAccepted()
 
     @util.identified_stack
     def update_patch(self, req, identity, body):
-        """
+        """Update an existing stack with a new template.
+
         Update an existing stack with a new template by patching the parameters
         Add the flag patch to the args so the engine code can distinguish
         """
         data = InstantiationData(body, patch=True)
 
         args = self.prepare_args(data)
-        self.rpc_client.update_stack(req.context,
-                                     identity,
-                                     data.template(),
-                                     data.environment(),
-                                     data.files(),
-                                     args)
+        self.rpc_client.update_stack(
+            req.context,
+            identity,
+            data.template(),
+            data.environment(),
+            data.files(),
+            args,
+            environment_files=data.environment_files())
 
         raise exc.HTTPAccepted()
 
+    def _param_show_nested(self, req):
+        whitelist = {'show_nested': 'single'}
+        params = util.get_allowed_params(req.params, whitelist)
+
+        p_name = 'show_nested'
+        if p_name in params:
+            return self._extract_bool_param(p_name, params[p_name])
+
     @util.identified_stack
     def preview_update(self, req, identity, body):
-        """
-        Preview an update to an existing stack with a new template/parameters
-        """
+        """Preview update for existing stack with a new template/parameters."""
         data = InstantiationData(body)
 
         args = self.prepare_args(data)
+        show_nested = self._param_show_nested(req)
+        if show_nested is not None:
+            args[rpc_api.PARAM_SHOW_NESTED] = show_nested
         changes = self.rpc_client.preview_update_stack(
             req.context,
             identity,
             data.template(),
             data.environment(),
             data.files(),
-            args)
+            args,
+            environment_files=data.environment_files())
 
         return {'resource_changes': changes}
 
@@ -508,21 +525,23 @@ class StackController(object):
         data = InstantiationData(body, patch=True)
 
         args = self.prepare_args(data)
+        show_nested = self._param_show_nested(req)
+        if show_nested is not None:
+            args['show_nested'] = show_nested
         changes = self.rpc_client.preview_update_stack(
             req.context,
             identity,
             data.template(),
             data.environment(),
             data.files(),
-            args)
+            args,
+            environment_files=data.environment_files())
 
         return {'resource_changes': changes}
 
     @util.identified_stack
     def delete(self, req, identity):
-        """
-        Delete the specified stack
-        """
+        """Delete the specified stack."""
 
         res = self.rpc_client.delete_stack(req.context,
                                            identity,
@@ -535,23 +554,33 @@ class StackController(object):
 
     @util.identified_stack
     def abandon(self, req, identity):
-        """
+        """Abandons specified stack.
+
         Abandons specified stack by deleting the stack and it's resources
         from the database, but underlying resources will not be deleted.
         """
         return self.rpc_client.abandon_stack(req.context,
                                              identity)
 
+    @util.identified_stack
+    def export(self, req, identity):
+        """Export specified stack.
+
+        Return stack data in JSON format.
+        """
+        return self.rpc_client.export_stack(req.context, identity)
+
     @util.policy_enforce
     def validate_template(self, req, body):
-        """
-        Implements the ValidateTemplate API action
-        Validates the specified template
+        """Implements the ValidateTemplate API action.
+
+        Validates the specified template.
         """
 
         data = InstantiationData(body)
 
-        whitelist = {'show_nested': 'single'}
+        whitelist = {'show_nested': util.PARAM_TYPE_SINGLE,
+                     'ignore_errors': util.PARAM_TYPE_SINGLE}
         params = util.get_allowed_params(req.params, whitelist)
 
         show_nested = False
@@ -560,11 +589,19 @@ class StackController(object):
             params[p_name] = self._extract_bool_param(p_name, params[p_name])
             show_nested = params[p_name]
 
-        result = self.rpc_client.validate_template(req.context,
-                                                   data.template(),
-                                                   data.environment(),
-                                                   files=data.files(),
-                                                   show_nested=show_nested)
+        if rpc_api.PARAM_IGNORE_ERRORS in params:
+            ignorable_errors = params[rpc_api.PARAM_IGNORE_ERRORS].split(',')
+        else:
+            ignorable_errors = None
+
+        result = self.rpc_client.validate_template(
+            req.context,
+            data.template(),
+            data.environment(),
+            files=data.files(),
+            environment_files=data.environment_files(),
+            show_nested=show_nested,
+            ignorable_errors=ignorable_errors)
 
         if 'Error' in result:
             raise exc.HTTPBadRequest(result['Error'])
@@ -573,9 +610,7 @@ class StackController(object):
 
     @util.policy_enforce
     def list_resource_types(self, req):
-        """
-        Returns a list of valid resource types that may be used in a template.
-        """
+        """Returns a resource types list which may be used in template."""
         support_status = req.params.get('support_status')
         type_name = req.params.get('name')
         version = req.params.get('version')
@@ -588,9 +623,7 @@ class StackController(object):
 
     @util.policy_enforce
     def list_template_versions(self, req):
-        """
-        Returns a list of available template versions
-        """
+        """Returns a list of available template versions."""
         return {
             'template_versions':
             self.rpc_client.list_template_versions(req.context)
@@ -598,9 +631,7 @@ class StackController(object):
 
     @util.policy_enforce
     def list_template_functions(self, req, template_version):
-        """
-        Returns a list of available functions in a given template
-        """
+        """Returns a list of available functions in a given template."""
         return {
             'template_functions':
             self.rpc_client.list_template_functions(req.context,
@@ -609,16 +640,12 @@ class StackController(object):
 
     @util.policy_enforce
     def resource_schema(self, req, type_name):
-        """
-        Returns the schema of the given resource type.
-        """
+        """Returns the schema of the given resource type."""
         return self.rpc_client.resource_schema(req.context, type_name)
 
     @util.policy_enforce
     def generate_template(self, req, type_name):
-        """
-        Generates a template based on the specified type.
-        """
+        """Generates a template based on the specified type."""
         template_type = 'cfn'
         if rpc_api.TEMPLATE_TYPE in req.params:
             try:
@@ -660,13 +687,29 @@ class StackController(object):
         self.rpc_client.stack_restore(req.context, identity, snapshot_id)
         raise exc.HTTPAccepted()
 
+    @util.identified_stack
+    def list_outputs(self, req, identity):
+        return {
+            'outputs': self.rpc_client.list_outputs(
+                req.context, identity)
+        }
+
+    @util.identified_stack
+    def show_output(self, req, identity, output_key):
+        return {'output': self.rpc_client.show_output(req.context,
+                                                      identity,
+                                                      output_key)}
+
 
 class StackSerializer(serializers.JSONResponseSerializer):
     """Handles serialization of specific controller method responses."""
 
     def _populate_response_header(self, response, location, status):
         response.status = status
-        response.headers['Location'] = location.encode('utf-8')
+        if six.PY2:
+            response.headers['Location'] = location.encode('utf-8')
+        else:
+            response.headers['Location'] = location
         response.headers['Content-Type'] = 'application/json'
         return response
 
@@ -679,9 +722,7 @@ class StackSerializer(serializers.JSONResponseSerializer):
 
 
 def create_resource(options):
-    """
-    Stacks resource factory method.
-    """
+    """Stacks resource factory method."""
     deserializer = wsgi.JSONRequestDeserializer()
     serializer = StackSerializer()
     return wsgi.Resource(StackController(options), deserializer, serializer)
