@@ -11,15 +11,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Routines for configuring Heat
-"""
-import logging as sys_logging
+"""Routines for configuring Heat."""
 import os
 
 from eventlet.green import socket
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_middleware import cors
+from osprofiler import opts as profiler
 
 from heat.common import exception
 from heat.common.i18n import _
@@ -41,8 +40,10 @@ service_opts = [
                default=60,
                help=_('Seconds between running periodic tasks.')),
     cfg.StrOpt('heat_metadata_server_url',
-               default="",
-               help=_('URL of the Heat metadata server.')),
+               help=_('URL of the Heat metadata server. '
+                      'NOTE: Setting this is only needed if you require '
+                      'instances to use a different endpoint than in the '
+                      'keystone catalog')),
     cfg.StrOpt('heat_waitcondition_server_url',
                help=_('URL of the Heat waitcondition server.')),
     cfg.StrOpt('heat_watch_server_url',
@@ -126,6 +127,16 @@ engine_opts = [
                help=_('Number of times to retry to bring a '
                       'resource to a non-error state. Set to 0 to disable '
                       'retries.')),
+    cfg.IntOpt('client_retry_limit',
+               default=2,
+               help=_('Number of times to retry when a client encounters an '
+                      'expected intermittent error. Set to 0 to disable '
+                      'retries.')),
+    cfg.IntOpt('max_interface_check_attempts',
+               min=1,
+               default=10,
+               help=_('Number of times to check whether an interface has '
+                      'been attached or detached.')),
     cfg.IntOpt('event_purge_batch_size',
                default=10,
                help=_("Controls how many events will be pruned whenever a "
@@ -162,7 +173,12 @@ engine_opts = [
                 default=False,
                 help=_('Enables engine with convergence architecture. All '
                        'stacks with this option will be created using '
-                       'convergence engine .')),
+                       'convergence engine.')),
+    cfg.BoolOpt('observe_on_update',
+                default=False,
+                help=_('On update, enables heat to collect existing resource '
+                       'properties from reality and converge to '
+                       'updated template.')),
     cfg.StrOpt('default_software_config_transport',
                choices=['POLL_SERVER_CFN',
                         'POLL_SERVER_HEAT',
@@ -240,14 +256,6 @@ rpc_opts = [
                       'It is not necessarily a hostname, FQDN, '
                       'or IP address.'))]
 
-profiler_group = cfg.OptGroup('profiler')
-profiler_opts = [
-    cfg.BoolOpt("profiler_enabled", default=False,
-                help=_('If False fully disable profiling feature.')),
-    cfg.BoolOpt("trace_sqlalchemy", default=False,
-                help=_("If False do not trace SQL requests."))
-]
-
 auth_password_group = cfg.OptGroup('auth_password')
 auth_password_opts = [
     cfg.BoolOpt('multi_cloud',
@@ -329,8 +337,8 @@ def startup_sanity_check():
             not cfg.CONF.stack_user_domain_name):
         # FIXME(shardy): Legacy fallback for folks using old heat.conf
         # files which lack domain configuration
-        LOG.warn(_LW('stack_user_domain_id or stack_user_domain_name not '
-                     'set in heat.conf falling back to using default'))
+        LOG.warning(_LW('stack_user_domain_id or stack_user_domain_name not '
+                        'set in heat.conf falling back to using default'))
     else:
         domain_admin_user = cfg.CONF.stack_domain_admin
         domain_admin_password = cfg.CONF.stack_domain_admin_password
@@ -342,7 +350,7 @@ def startup_sanity_check():
                                     '"stack_domain_admin_password"'))
     auth_key_len = len(cfg.CONF.auth_encryption_key)
     if auth_key_len in (16, 24):
-        LOG.warn(
+        LOG.warning(
             _LW('Please update auth_encryption_key to be 32 characters.'))
     elif auth_key_len != 32:
         raise exception.Error(_('heat.conf misconfigured, auth_encryption_key '
@@ -356,7 +364,7 @@ def list_opts():
     yield paste_deploy_group.name, paste_deploy_opts
     yield auth_password_group.name, auth_password_opts
     yield revision_group.name, revision_opts
-    yield profiler_group.name, profiler_opts
+    yield profiler.list_opts()[0]
     yield 'clients', default_clients_opts
 
     for client in ('nova', 'swift', 'neutron', 'cinder',
@@ -374,25 +382,25 @@ def list_opts():
 cfg.CONF.register_group(paste_deploy_group)
 cfg.CONF.register_group(auth_password_group)
 cfg.CONF.register_group(revision_group)
-cfg.CONF.register_group(profiler_group)
+profiler.set_defaults(cfg.CONF)
 
 for group, opts in list_opts():
     cfg.CONF.register_opts(opts, group=group)
 
 
 def _get_deployment_flavor():
-    """
-    Retrieve the paste_deploy.flavor config item, formatted appropriately
-    for appending to the application name.
+    """Retrieves the paste_deploy.flavor config item.
+
+    Item formatted appropriately for appending to the application name.
     """
     flavor = cfg.CONF.paste_deploy.flavor
     return '' if not flavor else ('-' + flavor)
 
 
 def _get_deployment_config_file():
-    """
-    Retrieve the deployment_config_file config item, formatted as an
-    absolute pathname.
+    """Retrieves the deployment_config_file config item.
+
+    Item formatted as an absolute pathname.
     """
     config_path = cfg.CONF.find_file(
         cfg.CONF.paste_deploy['api_paste_config'])
@@ -403,8 +411,7 @@ def _get_deployment_config_file():
 
 
 def load_paste_app(app_name=None):
-    """
-    Builds and returns a WSGI app from a paste config file.
+    """Builds and returns a WSGI app from a paste config file.
 
     We assume the last config file specified in the supplied ConfigOpts
     object is the paste config file.
@@ -423,7 +430,8 @@ def load_paste_app(app_name=None):
 
     conf_file = _get_deployment_config_file()
     if conf_file is None:
-        raise RuntimeError(_("Unable to locate config file"))
+        raise RuntimeError(_("Unable to locate config file [%s]") %
+                           cfg.CONF.paste_deploy['api_paste_config'])
 
     try:
         app = wsgi.paste_deploy_app(conf_file, app_name, cfg.CONF)
@@ -431,7 +439,7 @@ def load_paste_app(app_name=None):
         # Log the options used when starting if we're in debug mode...
         if cfg.CONF.debug:
             cfg.CONF.log_opt_values(logging.getLogger(app_name),
-                                    sys_logging.DEBUG)
+                                    logging.DEBUG)
 
         return app
     except (LookupError, ImportError) as e:
@@ -457,3 +465,27 @@ def get_client_option(client, option):
     # look for the option in the generic [clients] section
     cfg.CONF.import_opt(option, 'heat.common.config', group='clients')
     return getattr(cfg.CONF.clients, option)
+
+
+def set_config_defaults():
+    """This method updates all configuration default values."""
+    # CORS Defaults
+    # TODO(krotscheck): Update with https://review.openstack.org/#/c/285368/
+    cfg.set_defaults(cors.CORS_OPTS,
+                     allow_headers=['X-Auth-Token',
+                                    'X-Identity-Status',
+                                    'X-Roles',
+                                    'X-Service-Catalog',
+                                    'X-User-Id',
+                                    'X-Tenant-Id',
+                                    'X-OpenStack-Request-ID'],
+                     expose_headers=['X-Auth-Token',
+                                     'X-Subject-Token',
+                                     'X-Service-Token',
+                                     'X-OpenStack-Request-ID'],
+                     allow_methods=['GET',
+                                    'PUT',
+                                    'POST',
+                                    'DELETE',
+                                    'PATCH']
+                     )
