@@ -47,7 +47,6 @@ cfg.CONF.register_opts(keystone_opts)
 
 
 class KeystoneClientV3(object):
-
     """Wrap keystone client so we can encapsulate logic used in resources.
 
     Note this is intended to be initialized from a resource on a per-session
@@ -101,7 +100,7 @@ class KeystoneClientV3(object):
     def stack_domain(self):
         """Domain scope data.
 
-        This is only used for checking for scoping data, not using the value
+        This is only used for checking for scoping data, not using the value.
         """
         return self._stack_domain_id or self.stack_domain_name
 
@@ -236,8 +235,8 @@ class KeystoneClientV3(object):
 
     def _get_username(self, username):
         if(len(username) > 64):
-            LOG.warn(_LW("Truncating the username %s to the last 64 "
-                         "characters."), username)
+            LOG.warning(_LW("Truncating the username %s to the last 64 "
+                            "characters."), username)
         # get the last 64 characters of the username
         return username[-64:]
 
@@ -550,7 +549,6 @@ class KeystoneClientV3(object):
 
 
 class KeystoneClient(object):
-
     """Keystone Auth Client.
 
     Delay choosing the backend client module until the client's class
