@@ -170,27 +170,41 @@ def translate_filters(params):
     return params
 
 
-def format_stack_outputs(stack, outputs):
+def format_stack_outputs(stack, outputs, resolve_value=False):
     """Return a representation of the given output template.
 
     Return a representation of the given output template for the given stack
     that matches the API output expectations.
     """
-    def format_stack_output(k):
-        output = {
-            rpc_api.OUTPUT_DESCRIPTION: outputs[k].get('Description',
-                                                       'No description given'),
-            rpc_api.OUTPUT_KEY: k,
-            rpc_api.OUTPUT_VALUE: stack.output(k)
-        }
-        if outputs[k].get('error_msg'):
-            output.update({rpc_api.OUTPUT_ERROR: outputs[k].get('error_msg')})
-        return output
-
-    return [format_stack_output(key) for key in outputs]
+    return [format_stack_output(stack, outputs,
+                                key, resolve_value=resolve_value)
+            for key in outputs]
 
 
-def format_stack(stack, preview=False):
+def format_stack_output(stack, outputs, k, resolve_value=True):
+    result = {
+        rpc_api.OUTPUT_KEY: k,
+        rpc_api.OUTPUT_DESCRIPTION: outputs[k].get('Description',
+                                                   'No description given'),
+    }
+
+    if resolve_value:
+        try:
+            value = stack.output(k)
+        except Exception as ex:
+            # We don't need error raising, just adding output_error to
+            # resulting dict.
+            value = None
+            result.update({rpc_api.OUTPUT_ERROR: six.text_type(ex)})
+        finally:
+            result.update({rpc_api.OUTPUT_VALUE: value})
+
+    if outputs[k].get('error_msg'):
+        result.update({rpc_api.OUTPUT_ERROR: outputs[k].get('error_msg')})
+    return result
+
+
+def format_stack(stack, preview=False, resolve_outputs=True):
     """Return a representation of the given stack.
 
     Return a representation of the given stack that matches the API output
@@ -225,9 +239,10 @@ def format_stack(stack, preview=False):
         info.update(update_info)
 
     # allow users to view the outputs of stacks
-    if stack.action != stack.DELETE and stack.status != stack.IN_PROGRESS:
+    if stack.action != stack.DELETE and resolve_outputs:
         info[rpc_api.STACK_OUTPUTS] = format_stack_outputs(stack,
-                                                           stack.outputs)
+                                                           stack.outputs,
+                                                           resolve_value=True)
 
     return info
 
@@ -310,11 +325,11 @@ def format_stack_resource(resource, detail=True, with_props=False,
         res[rpc_api.RES_DESCRIPTION] = resource.t.description
         res[rpc_api.RES_METADATA] = resource.metadata_get()
         if with_attr is not False:
-            res[rpc_api.RES_SCHEMA_ATTRIBUTES] = format_resource_attributes(
+            res[rpc_api.RES_ATTRIBUTES] = format_resource_attributes(
                 resource, with_attr)
 
     if with_props:
-        res[rpc_api.RES_SCHEMA_PROPERTIES] = format_resource_properties(
+        res[rpc_api.RES_PROPERTIES] = format_resource_properties(
             resource)
 
     return res
@@ -366,6 +381,10 @@ def format_notification_body(stack):
     result = {
         rpc_api.NOTIFY_TENANT_ID: stack.context.tenant_id,
         rpc_api.NOTIFY_USER_ID: stack.context.user,
+        # deprecated: please use rpc_api.NOTIFY_USERID for user id or
+        # rpc_api.NOTIFY_USERNAME for user name.
+        rpc_api.NOTIFY_USERID: stack.context.user_id,
+        rpc_api.NOTIFY_USERNAME: stack.context.user,
         rpc_api.NOTIFY_STACK_ID: stack.id,
         rpc_api.NOTIFY_STACK_NAME: stack.name,
         rpc_api.NOTIFY_STATE: state,
