@@ -58,7 +58,6 @@ class GetAZs(function.Function):
 
     def result(self):
         # TODO(therve): Implement region scoping
-        # region = function.resolve(self.args)
 
         if self.stack is None:
             return ['nova']
@@ -191,7 +190,7 @@ class GetAtt(function.Function):
 
         attr = function.resolve(self._attribute)
         from heat.engine import resource
-        if (type(res).FnGetAtt == resource.Resource.FnGetAtt and
+        if (type(res).get_attribute == resource.Resource.get_attribute and
                 attr not in six.iterkeys(res.attributes_schema)):
             raise exception.InvalidTemplateAttribute(
                 resource=self._resource_name, key=attr)
@@ -238,11 +237,6 @@ class Select(function.Function):
     def result(self):
         index = function.resolve(self._lookup)
 
-        try:
-            index = int(index)
-        except (ValueError, TypeError):
-            pass
-
         strings = function.resolve(self._strings)
 
         if strings == '':
@@ -266,6 +260,11 @@ class Select(function.Function):
                                 self.fn_name)
             return strings.get(index, '')
 
+        try:
+            index = int(index)
+        except (ValueError, TypeError):
+            pass
+
         if (isinstance(strings, collections.Sequence) and
                 not isinstance(strings, six.string_types)):
             if not isinstance(index, six.integer_types):
@@ -394,9 +393,8 @@ class Replace(function.Function):
 
         "<value_1> <value_2>"
 
-    This is implemented using python str.replace on each key. Longer keys are
-    substituted before shorter ones, but the order in which replacements are
-    performed is otherwise undefined.
+    This is implemented using python str.replace on each key. The order in
+    which replacements are performed is undefined.
     """
 
     def __init__(self, stack, fn_name, args):
@@ -456,9 +454,6 @@ class Replace(function.Function):
 
             return string.replace(placeholder, six.text_type(value))
 
-        mapping = collections.OrderedDict(sorted(mapping.items(),
-                                                 key=lambda t: len(t[0]),
-                                                 reverse=True))
         return six.moves.reduce(replace, six.iteritems(mapping), template)
 
 
