@@ -11,11 +11,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_config import cfg
+import six
+
 from heatclient import client as hc
 from heatclient import exc
 
 from heat.engine.clients import client_plugin
 
+CLIENT_NAME = 'heat'
+
 
 class HeatClientPlugin(client_plugin.ClientPlugin):
 
@@ -30,14 +35,14 @@ class HeatClientPlugin(client_plugin.ClientPlugin):
             'token': self.auth_token,
             'username': None,
             'password': None,
-            'ca_file': self._get_client_option('heat', 'ca_file'),
-            'cert_file': self._get_client_option('heat', 'cert_file'),
-            'key_file': self._get_client_option('heat', 'key_file'),
-            'insecure': self._get_client_option('heat', 'insecure')
+            'ca_file': self._get_client_option(CLIENT_NAME, 'ca_file'),
+            'cert_file': self._get_client_option(CLIENT_NAME, 'cert_file'),
+            'key_file': self._get_client_option(CLIENT_NAME, 'key_file'),
+            'insecure': self._get_client_option(CLIENT_NAME, 'insecure')
         }
 
         endpoint = self.get_heat_url()
-        if self._get_client_option('heat', 'url'):
+        if self._get_client_option(CLIENT_NAME, 'url'):
             # assume that the heat API URL is manually configured because
             # it is not in the keystone catalog, so include the credentials
             # for the standalone auth_password middleware
@@ -57,19 +62,47 @@ class HeatClientPlugin(client_plugin.ClientPlugin):
         return isinstance(ex, exc.HTTPConflict)
 
     def get_heat_url(self):
-        heat_url = self._get_client_option('heat', 'url')
+        heat_url = self._get_client_option(CLIENT_NAME, 'url')
         if heat_url:
             tenant_id = self.context.tenant_id
             heat_url = heat_url % {'tenant_id': tenant_id}
         else:
-            endpoint_type = self._get_client_option('heat', 'endpoint_type')
+            endpoint_type = self._get_client_option(CLIENT_NAME,
+                                                    'endpoint_type')
             heat_url = self.url_for(service_type=self.ORCHESTRATION,
                                     endpoint_type=endpoint_type)
         return heat_url
 
     def get_heat_cfn_url(self):
-        endpoint_type = self._get_client_option('heat',
+        endpoint_type = self._get_client_option(CLIENT_NAME,
                                                 'endpoint_type')
         heat_cfn_url = self.url_for(service_type=self.CLOUDFORMATION,
                                     endpoint_type=endpoint_type)
         return heat_cfn_url
+
+    def get_cfn_metadata_server_url(self):
+        # Historically, we've required heat_metadata_server_url set in
+        # heat.conf, which simply points to the heat-api-cfn endpoint in
+        # most cases, so fall back to looking in the catalog when not set
+        config_url = cfg.CONF.heat_metadata_server_url
+        if config_url is None:
+            config_url = self.get_heat_cfn_url()
+        # Backwards compatibility, previous heat_metadata_server_url
+        # values didn't have to include the version path suffix
+        # Also, we always added a trailing "/" in nova/server.py,
+        # which looks not required by os-collect-config, but maintain
+        # to avoid any risk other folks have scripts which expect it.
+        if '/v1' not in config_url:
+            config_url += '/v1'
+        if config_url and config_url[-1] != "/":
+            config_url += '/'
+        return config_url
+
+    def get_watch_server_url(self):
+        cfn_url = self.get_heat_cfn_url()
+        url_parts = cfn_url.split(':')
+        port_and_version = url_parts[-1].split('/')
+        port_and_version[0] = (
+            six.text_type(cfg.CONF.heat_api_cloudwatch.bind_port))
+        url_parts[-1] = '/'.join(port_and_version)
+        return ':'.join(url_parts)
