@@ -18,6 +18,8 @@ from heat.common import heat_keystoneclient as hkc
 from heat.engine.clients import client_plugin
 from heat.engine import constraints
 
+CLIENT_NAME = 'keystone'
+
 
 class KeystoneClientPlugin(client_plugin.ClientPlugin):
 
@@ -50,6 +52,8 @@ class KeystoneClientPlugin(client_plugin.ClientPlugin):
         raise exception.EntityNotFound(entity='KeystoneRole', name=role)
 
     def get_project_id(self, project):
+        if project is None:
+            return None
         try:
             project_obj = self.client().client.projects.get(project)
             return project_obj.id
@@ -63,6 +67,8 @@ class KeystoneClientPlugin(client_plugin.ClientPlugin):
                                        name=project)
 
     def get_domain_id(self, domain):
+        if domain is None:
+            return None
         try:
             domain_obj = self.client().client.domains.get(domain)
             return domain_obj.id
@@ -75,6 +81,8 @@ class KeystoneClientPlugin(client_plugin.ClientPlugin):
         raise exception.EntityNotFound(entity='KeystoneDomain', name=domain)
 
     def get_group_id(self, group):
+        if group is None:
+            return None
         try:
             group_obj = self.client().client.groups.get(group)
             return group_obj.id
@@ -87,6 +95,8 @@ class KeystoneClientPlugin(client_plugin.ClientPlugin):
         raise exception.EntityNotFound(entity='KeystoneGroup', name=group)
 
     def get_service_id(self, service):
+        if service is None:
+            return None
         try:
             service_obj = self.client().client.services.get(service)
             return service_obj.id
@@ -102,6 +112,8 @@ class KeystoneClientPlugin(client_plugin.ClientPlugin):
                                                name=service)
 
     def get_user_id(self, user):
+        if user is None:
+            return None
         try:
             user_obj = self.client().client.users.get(user)
             return user_obj.id
@@ -113,51 +125,70 @@ class KeystoneClientPlugin(client_plugin.ClientPlugin):
 
         raise exception.EntityNotFound(entity='KeystoneUser', name=user)
 
-
-class KeystoneRoleConstraint(constraints.BaseCustomConstraint):
-
-    expected_exceptions = (exception.EntityNotFound,)
-
-    def validate_with_client(self, client, role):
-        client.client_plugin('keystone').get_role_id(role)
+    def get_region_id(self, region):
+        try:
+            region_obj = self.client().client.regions.get(region)
+            return region_obj.id
+        except exceptions.NotFound:
+            raise exception.EntityNotFound(entity='KeystoneRegion',
+                                           name=region)
 
 
-class KeystoneDomainConstraint(constraints.BaseCustomConstraint):
+class KeystoneBaseConstraint(constraints.BaseCustomConstraint):
 
-    expected_exceptions = (exception.EntityNotFound,)
+    resource_client_name = CLIENT_NAME
+    entity = None
 
-    def validate_with_client(self, client, domain):
-        client.client_plugin('keystone').get_domain_id(domain)
+    def validate_with_client(self, client, resource_id):
+        # when user specify empty value in template, do not get the
+        # responding resource from backend, otherwise an error will happen
+        if resource_id == '':
+            raise exception.EntityNotFound(entity=self.entity,
+                                           name=resource_id)
+
+        super(KeystoneBaseConstraint, self).validate_with_client(client,
+                                                                 resource_id)
 
 
-class KeystoneProjectConstraint(constraints.BaseCustomConstraint):
+class KeystoneRoleConstraint(KeystoneBaseConstraint):
 
-    expected_exceptions = (exception.EntityNotFound,)
-
-    def validate_with_client(self, client, project):
-        client.client_plugin('keystone').get_project_id(project)
+    resource_getter_name = 'get_role_id'
+    entity = 'KeystoneRole'
 
 
-class KeystoneGroupConstraint(constraints.BaseCustomConstraint):
+class KeystoneDomainConstraint(KeystoneBaseConstraint):
 
-    expected_exceptions = (exception.EntityNotFound,)
-
-    def validate_with_client(self, client, group):
-        client.client_plugin('keystone').get_group_id(group)
+    resource_getter_name = 'get_domain_id'
+    entity = 'KeystoneDomain'
 
 
-class KeystoneServiceConstraint(constraints.BaseCustomConstraint):
+class KeystoneProjectConstraint(KeystoneBaseConstraint):
+
+    resource_getter_name = 'get_project_id'
+    entity = 'KeystoneProject'
+
+
+class KeystoneGroupConstraint(KeystoneBaseConstraint):
+
+    resource_getter_name = 'get_group_id'
+    entity = 'KeystoneGroup'
+
+
+class KeystoneServiceConstraint(KeystoneBaseConstraint):
 
     expected_exceptions = (exception.EntityNotFound,
                            exception.KeystoneServiceNameConflict,)
-
-    def validate_with_client(self, client, service):
-        client.client_plugin('keystone').get_service_id(service)
+    resource_getter_name = 'get_service_id'
+    entity = 'KeystoneService'
 
 
-class KeystoneUserConstraint(constraints.BaseCustomConstraint):
+class KeystoneUserConstraint(KeystoneBaseConstraint):
 
-    expected_exceptions = (exception.EntityNotFound,)
+    resource_getter_name = 'get_user_id'
+    entity = 'KeystoneUser'
 
-    def validate_with_client(self, client, user):
-        client.client_plugin('keystone').get_user_id(user)
+
+class KeystoneRegionConstraint(KeystoneBaseConstraint):
+
+    resource_getter_name = 'get_region_id'
+    entity = 'KeystoneRegion'
