@@ -15,7 +15,6 @@ import collections
 import email
 from email.mime import multipart
 from email.mime import text
-import logging
 import os
 import pkgutil
 import string
@@ -23,6 +22,7 @@ import string
 from novaclient import client as nc
 from novaclient import exceptions
 from oslo_config import cfg
+from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
 from retrying import retry
@@ -34,12 +34,14 @@ from heat.common.i18n import _
 from heat.common.i18n import _LI
 from heat.common.i18n import _LW
 from heat.engine.clients import client_plugin
+from heat.engine.clients import os as os_client
 from heat.engine import constraints
 
 LOG = logging.getLogger(__name__)
 
 
 NOVACLIENT_VERSION = "2"
+CLIENT_NAME = 'nova'
 
 
 class NovaClientPlugin(client_plugin.ClientPlugin):
@@ -60,7 +62,7 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
     service_types = [COMPUTE] = ['compute']
 
     def _create(self):
-        endpoint_type = self._get_client_option('nova', 'endpoint_type')
+        endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
         management_url = self.url_for(service_type=self.COMPUTE,
                                       endpoint_type=endpoint_type)
         extensions = nc.discover_extensions(NOVACLIENT_VERSION)
@@ -74,10 +76,10 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
             'api_key': None,
             'extensions': extensions,
             'endpoint_type': endpoint_type,
-            'http_log_debug': self._get_client_option('nova',
+            'http_log_debug': self._get_client_option(CLIENT_NAME,
                                                       'http_log_debug'),
-            'cacert': self._get_client_option('nova', 'ca_file'),
-            'insecure': self._get_client_option('nova', 'insecure')
+            'cacert': self._get_client_option(CLIENT_NAME, 'ca_file'),
+            'insecure': self._get_client_option(CLIENT_NAME, 'insecure')
         }
 
         client = nc.Client(NOVACLIENT_VERSION, **args)
@@ -104,6 +106,8 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
         return (isinstance(ex, exceptions.ClientException) and
                 http_status == 422)
 
+    @retry(stop_max_attempt_number=max(cfg.CONF.client_retry_limit + 1, 0),
+           retry_on_exception=client_plugin.retry_if_connection_err)
     def get_server(self, server):
         """Return fresh server object.
 
@@ -112,9 +116,7 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
         """
         try:
             return self.client().servers.get(server)
-        except exceptions.NotFound as ex:
-            LOG.warn(_LW('Server (%(server)s) not found: %(ex)s'),
-                     {'server': server, 'ex': ex})
+        except exceptions.NotFound:
             raise exception.EntityNotFound(entity='Server', name=server)
 
     def fetch_server(self, server_id):
@@ -128,17 +130,17 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
         try:
             server = self.client().servers.get(server_id)
         except exceptions.OverLimit as exc:
-            LOG.warn(_LW("Received an OverLimit response when "
-                         "fetching server (%(id)s) : %(exception)s"),
-                     {'id': server_id,
-                      'exception': exc})
+            LOG.warning(_LW("Received an OverLimit response when "
+                            "fetching server (%(id)s) : %(exception)s"),
+                        {'id': server_id,
+                         'exception': exc})
         except exceptions.ClientException as exc:
             if ((getattr(exc, 'http_status', getattr(exc, 'code', None)) in
                  (500, 503))):
-                LOG.warn(_LW("Received the following exception when "
-                         "fetching server (%(id)s) : %(exception)s"),
-                         {'id': server_id,
-                          'exception': exc})
+                LOG.warning(_LW("Received the following exception when "
+                            "fetching server (%(id)s) : %(exception)s"),
+                            {'id': server_id,
+                             'exception': exc})
             else:
                 raise
         return server
@@ -151,20 +153,20 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
         try:
             server.get()
         except exceptions.OverLimit as exc:
-            LOG.warn(_LW("Server %(name)s (%(id)s) received an OverLimit "
-                         "response during server.get(): %(exception)s"),
-                     {'name': server.name,
-                      'id': server.id,
-                      'exception': exc})
+            LOG.warning(_LW("Server %(name)s (%(id)s) received an OverLimit "
+                            "response during server.get(): %(exception)s"),
+                        {'name': server.name,
+                         'id': server.id,
+                         'exception': exc})
         except exceptions.ClientException as exc:
             if ((getattr(exc, 'http_status', getattr(exc, 'code', None)) in
                  (500, 503))):
-                LOG.warn(_LW('Server "%(name)s" (%(id)s) received the '
-                             'following exception during server.get(): '
-                             '%(exception)s'),
-                         {'name': server.name,
-                          'id': server.id,
-                          'exception': exc})
+                LOG.warning(_LW('Server "%(name)s" (%(id)s) received the '
+                                'following exception during server.get(): '
+                                '%(exception)s'),
+                            {'name': server.name,
+                             'id': server.id,
+                             'exception': exc})
             else:
                 raise
 
@@ -225,39 +227,60 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
                 resource_status=server.status,
                 result=_('%s is not active') % res_name)
 
-    def get_flavor_id(self, flavor):
-        """Get the id for the specified flavor name.
-
-        If the specified value is flavor id, just return it.
+    def find_flavor_by_name_or_id(self, flavor):
+        """Find the specified flavor by name or id.
 
         :param flavor: the name of the flavor to find
         :returns: the id of :flavor:
-        :raises: exception.FlavorMissing
         """
-        flavor_id = None
-        flavor_list = self.client().flavors.list()
-        for o in flavor_list:
-            if o.name == flavor:
-                flavor_id = o.id
-                break
-            if o.id == flavor:
-                flavor_id = o.id
-                break
-        if flavor_id is None:
-            raise exception.FlavorMissing(flavor_id=flavor)
-        return flavor_id
+        return self._find_flavor_id(self.context.tenant_id,
+                                    flavor)
+
+    @os_client.MEMOIZE_FINDER
+    def _find_flavor_id(self, tenant_id, flavor):
+        # tenant id in the signature is used for the memoization key,
+        # that would differentiate similar resource names across tenants.
+        return self.get_flavor(flavor).id
+
+    def get_flavor(self, flavor_identifier):
+        """Get the flavor object for the specified flavor name or id.
+
+        :param flavor_identifier: the name or id of the flavor to find
+        :returns: a flavor object with name or id :flavor:
+        """
+        try:
+            flavor = self.client().flavors.get(flavor_identifier)
+        except exceptions.NotFound:
+            flavor = self.client().flavors.find(name=flavor_identifier)
+
+        return flavor
+
+    def get_host(self, host_name):
+        """Get the host id specified by name.
+
+        :param host_name: the name of host to find
+        :returns: the list of match hosts
+        :raises: exception.EntityNotFound
+        """
+
+        host_list = self.client().hosts.list()
+        for host in host_list:
+            if host.host_name == host_name and host.service == self.COMPUTE:
+                return host
+
+        raise exception.EntityNotFound(entity='Host', name=host_name)
 
     def get_keypair(self, key_name):
         """Get the public key specified by :key_name:
 
         :param key_name: the name of the key to look for
         :returns: the keypair (name, public_key) for :key_name:
-        :raises: exception.UserKeyPairMissing
+        :raises: exception.EntityNotFound
         """
         try:
             return self.client().keypairs.get(key_name)
         except exceptions.NotFound:
-            raise exception.UserKeyPairMissing(key_name=key_name)
+            raise exception.EntityNotFound(entity='Key', name=key_name)
 
     def build_userdata(self, metadata, userdata=None, instance_user=None,
                        user_data_format='HEAT_CFNTOOLS'):
@@ -349,17 +372,22 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
             attachments.append((jsonutils.dumps(metadata),
                                 'cfn-init-data', 'x-cfninitdata'))
 
-        attachments.append((cfg.CONF.heat_watch_server_url,
-                            'cfn-watch-server', 'x-cfninitdata'))
+        heat_client_plugin = self.context.clients.client_plugin('heat')
+        watch_url = cfg.CONF.heat_watch_server_url
+        if not watch_url:
+            watch_url = heat_client_plugin.get_watch_server_url()
+
+        attachments.append((watch_url, 'cfn-watch-server', 'x-cfninitdata'))
 
         if is_cfntools:
-            attachments.append((cfg.CONF.heat_metadata_server_url,
+            cfn_md_url = heat_client_plugin.get_cfn_metadata_server_url()
+            attachments.append((cfn_md_url,
                                 'cfn-metadata-server', 'x-cfninitdata'))
 
             # Create a boto config which the cfntools on the host use to know
             # where the cfn and cw API's are to be accessed
-            cfn_url = urlparse.urlparse(cfg.CONF.heat_metadata_server_url)
-            cw_url = urlparse.urlparse(cfg.CONF.heat_watch_server_url)
+            cfn_url = urlparse.urlparse(cfn_md_url)
+            cw_url = urlparse.urlparse(watch_url)
             is_secure = cfg.CONF.instance_connection_is_secure
             vcerts = cfg.CONF.instance_connection_https_validate_certificates
             boto_cfg = "\n".join(["[Boto]",
@@ -422,7 +450,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         else:
             return False
 
-    def check_resize(self, server_id, flavor_id, flavor):
+    def check_resize(self, server_id, flavor):
         """Verify that a resizing server is properly resized.
 
         If that's the case, confirm the resize, if not raise an error.
@@ -520,13 +548,15 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         try:
             server = self.client().servers.get(server)
         except exceptions.NotFound as ex:
-            LOG.warn(_LW('Instance (%(server)s) not found: %(ex)s'),
-                     {'server': server, 'ex': ex})
+            LOG.warning(_LW('Instance (%(server)s) not found: %(ex)s'),
+                        {'server': server, 'ex': ex})
         else:
             for n in sorted(server.networks, reverse=True):
                 if len(server.networks[n]) > 0:
                     return server.networks[n][0]
 
+    @retry(stop_max_attempt_number=max(cfg.CONF.client_retry_limit + 1, 0),
+           retry_on_exception=client_plugin.retry_if_connection_err)
     def absolute_limits(self):
         """Return the absolute limits as a dictionary."""
         limits = self.client().limits.get()
@@ -574,7 +604,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         except exceptions.NotFound as ex:
             LOG.debug('Nova network (%(net)s) not found: %(ex)s',
                       {'net': label, 'ex': ex})
-            raise exception.NovaNetworkNotFound(network=label)
+            raise exception.EntityNotFound(entity='Nova network', name=label)
         except exceptions.NoUniqueMatch as exc:
             LOG.debug('Nova network (%(net)s) is not unique matched: %(exc)s',
                       {'net': label, 'exc': exc})
@@ -657,7 +687,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         else:
             return False
 
-    @retry(stop_max_attempt_number=10,
+    @retry(stop_max_attempt_number=cfg.CONF.max_interface_check_attempts,
            wait_fixed=500,
            retry_on_result=client_plugin.retry_if_result_is_false)
     def check_interface_detach(self, server_id, port_id):
@@ -669,7 +699,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                     return False
         return True
 
-    @retry(stop_max_attempt_number=10,
+    @retry(stop_max_attempt_number=cfg.CONF.max_interface_check_attempts,
            wait_fixed=500,
            retry_on_result=client_plugin.retry_if_result_is_false)
     def check_interface_attach(self, server_id, port_id):
@@ -681,44 +711,53 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                     return True
         return False
 
-    def has_extension(self, alias):
-        """Check if extension is present."""
+    @os_client.MEMOIZE_EXTENSIONS
+    def _list_extensions(self):
         extensions = self.client().list_extensions.show_all()
-        return alias in [extension.alias for extension in extensions]
+        return set(extension.alias for extension in extensions)
+
+    def has_extension(self, alias):
+        """Check if specific extension is present."""
+        return alias in self._list_extensions()
 
 
-class ServerConstraint(constraints.BaseCustomConstraint):
+class NovaBaseConstraint(constraints.BaseCustomConstraint):
 
-    expected_exceptions = (exception.EntityNotFound,)
-
-    def validate_with_client(self, client, server):
-        client.client_plugin('nova').get_server(server)
+    resource_client_name = CLIENT_NAME
 
 
-class KeypairConstraint(constraints.BaseCustomConstraint):
+class ServerConstraint(NovaBaseConstraint):
 
-    expected_exceptions = (exception.UserKeyPairMissing,)
+    resource_getter_name = 'get_server'
+
+
+class KeypairConstraint(NovaBaseConstraint):
+
+    resource_getter_name = 'get_keypair'
 
     def validate_with_client(self, client, key_name):
         if not key_name:
             # Don't validate empty key, which can happen when you
             # use a KeyPair resource
             return True
-        client.client_plugin('nova').get_keypair(key_name)
+        super(KeypairConstraint, self).validate_with_client(client, key_name)
 
 
-class FlavorConstraint(constraints.BaseCustomConstraint):
+class FlavorConstraint(NovaBaseConstraint):
 
-    expected_exceptions = (exception.FlavorMissing,)
+    expected_exceptions = (exceptions.NotFound,)
 
-    def validate_with_client(self, client, flavor):
-        client.client_plugin('nova').get_flavor_id(flavor)
+    resource_getter_name = 'find_flavor_by_name_or_id'
 
 
-class NetworkConstraint(constraints.BaseCustomConstraint):
+class NetworkConstraint(NovaBaseConstraint):
 
-    expected_exceptions = (exception.NovaNetworkNotFound,
+    expected_exceptions = (exception.EntityNotFound,
                            exception.PhysicalResourceNameAmbiguity)
 
-    def validate_with_client(self, client, network):
-        client.client_plugin('nova').get_nova_network_id(network)
+    resource_getter_name = 'get_nova_network_id'
+
+
+class HostConstraint(NovaBaseConstraint):
+
+    resource_getter_name = 'get_host'
