@@ -13,8 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import logging
-
 from oslo_utils import uuidutils
 from saharaclient.api import base as sahara_base
 from saharaclient import client as sahara_client
@@ -22,11 +20,10 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.common.i18n import _LI
 from heat.engine.clients import client_plugin
 from heat.engine import constraints
 
-LOG = logging.getLogger(__name__)
+CLIENT_NAME = 'sahara'
 
 
 class SaharaClientPlugin(client_plugin.ClientPlugin):
@@ -37,7 +34,7 @@ class SaharaClientPlugin(client_plugin.ClientPlugin):
 
     def _create(self):
         con = self.context
-        endpoint_type = self._get_client_option('sahara', 'endpoint_type')
+        endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
         endpoint = self.url_for(service_type=self.DATA_PROCESSING,
                                 endpoint_type=endpoint_type)
         args = {
@@ -46,8 +43,8 @@ class SaharaClientPlugin(client_plugin.ClientPlugin):
             'auth_url': con.auth_url,
             'project_name': con.tenant,
             'sahara_url': endpoint,
-            'insecure': self._get_client_option('sahara', 'insecure'),
-            'cacert': self._get_client_option('sahara', 'ca_file')
+            'insecure': self._get_client_option(CLIENT_NAME, 'insecure'),
+            'cacert': self._get_client_option(CLIENT_NAME, 'ca_file')
         }
         client = sahara_client.Client('1.1', **args)
         return client
@@ -115,13 +112,9 @@ class SaharaClientPlugin(client_plugin.ClientPlugin):
                   "%s") % six.text_type(ex))
         num_matches = len(image_list)
         if num_matches == 0:
-            LOG.info(_LI("Image %s was not found in sahara images"),
-                     image_identifier)
             raise exception.EntityNotFound(entity='Image',
                                            name=image_identifier)
         elif num_matches > 1:
-            LOG.info(_LI("Multiple images %s were found in sahara with name"),
-                     image_identifier)
             raise exception.PhysicalResourceNameAmbiguity(
                 name=image_identifier)
         else:
@@ -137,23 +130,21 @@ class SaharaClientPlugin(client_plugin.ClientPlugin):
         try:
             self.client().plugins.get(plugin_name)
         except sahara_base.APIException:
-            LOG.info(_LI("Plugin %s was not found in sahara"), plugin_name)
             raise exception.EntityNotFound(entity='Plugin',
                                            name=plugin_name)
 
 
-class ImageConstraint(constraints.BaseCustomConstraint):
+class SaharaBaseConstraint(constraints.BaseCustomConstraint):
+    resource_client_name = CLIENT_NAME
+
+
+class ImageConstraint(SaharaBaseConstraint):
 
     expected_exceptions = (exception.EntityNotFound,
                            exception.PhysicalResourceNameAmbiguity,)
-
-    def validate_with_client(self, client, value):
-        client.client_plugin('sahara').get_image_id(value)
+    resource_getter_name = 'get_image_id'
 
 
-class PluginConstraint(constraints.BaseCustomConstraint):
+class PluginConstraint(SaharaBaseConstraint):
 
-    expected_exceptions = (exception.EntityNotFound,)
-
-    def validate_with_client(self, client, value):
-        client.client_plugin('sahara').get_plugin_id(value)
+    resource_getter_name = 'get_plugin_id'
