@@ -14,16 +14,18 @@
 import collections
 import numbers
 import re
-import warnings
 
 from oslo_cache import core
 from oslo_config import cfg
+from oslo_log import log
+from oslo_utils import reflection
 from oslo_utils import strutils
 import six
 
 from heat.common import cache
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common.i18n import _LW
 from heat.engine import resources
 
 # decorator that allows to cache the value
@@ -32,6 +34,8 @@ MEMOIZE = core.get_memoization_decorator(conf=cfg.CONF,
                                          region=cache.get_cache_region(),
                                          group="constraint_validation_cache")
 
+LOG = log.getLogger(__name__)
+
 
 class Schema(collections.Mapping):
     """Schema base class for validating properties or parameters.
@@ -61,8 +65,10 @@ class Schema(collections.Mapping):
 
     KEYS = (
         TYPE, DESCRIPTION, DEFAULT, SCHEMA, REQUIRED, CONSTRAINTS,
+        IMMUTABLE,
     ) = (
         'type', 'description', 'default', 'schema', 'required', 'constraints',
+        'immutable',
     )
 
     # Keywords for data types; each Schema subclass can define its respective
@@ -85,7 +91,8 @@ class Schema(collections.Mapping):
 
     def __init__(self, data_type, description=None,
                  default=None, schema=None,
-                 required=False, constraints=None, label=None):
+                 required=False, constraints=None, label=None,
+                 immutable=False):
         self._len = None
         self.label = label
         self.type = data_type
@@ -94,11 +101,12 @@ class Schema(collections.Mapping):
                 message=_('Invalid type (%s)') % self.type)
 
         if required and default is not None:
-            warnings.warn("Option 'required=True' should not be used with "
-                          "any 'default' value ({0})".format(default))
+            LOG.warning(_LW("Option 'required=True' should not be used with "
+                            "any 'default' value (%s)") % default)
 
         self.description = description
         self.required = required
+        self.immutable = immutable
 
         if isinstance(schema, type(self)):
             if self.type != self.LIST:
@@ -571,7 +579,9 @@ class BaseCustomConstraint(object):
     Subclass must provide `expected_exceptions` and implement
     `validate_with_client`.
     """
-    expected_exceptions = ()
+    expected_exceptions = (exception.EntityNotFound,)
+    resource_client_name = None
+    resource_getter_name = None
 
     _error_message = None
 
@@ -607,8 +617,8 @@ class BaseCustomConstraint(object):
                 return False
             else:
                 return True
-
-        cache_value_prefix = "{0}:{1}".format(self.__class__.__name__,
+        class_name = reflection.get_class_name(self, fully_qualified=False)
+        cache_value_prefix = "{0}:{1}".format(class_name,
                                               six.text_type(context.tenant_id))
         validation_result = check_cache_or_validate_value(
             cache_value_prefix, value)
@@ -619,3 +629,12 @@ class BaseCustomConstraint(object):
             check_cache_or_validate_value.invalidate(cache_value_prefix,
                                                      value)
         return validation_result
+
+    def validate_with_client(self, client, resource_id):
+        if self.resource_client_name and self.resource_getter_name:
+            getattr(client.client_plugin(self.resource_client_name),
+                    self.resource_getter_name)(resource_id)
+        else:
+            raise exception.InvalidSchemaError(
+                message=_('Client name and resource getter name must be '
+                          'specified.'))
