@@ -33,11 +33,10 @@ class Event(object):
     def __init__(self, context, stack, action, status, reason,
                  physical_resource_id, resource_properties, resource_name,
                  resource_type, uuid=None, timestamp=None, id=None):
-        """Initialisation of the event.
+        """Initialise from a context, stack, and event information.
 
-        Initialise from a context, stack, and event information. The timestamp
-        and database ID may also be initialised if the event is already in the
-        database.
+        The timestamp and database ID may also be initialised if the event is
+        already in the database.
         """
         self.context = context
         self.stack = stack
@@ -126,6 +125,8 @@ class Event(object):
             new_ev = event_object.Event.create(self.context, ev)
 
         self.id = new_ev.id
+        self.timestamp = new_ev.created_at
+        self.uuid = new_ev.uuid
         return self.id
 
     def identifier(self):
@@ -137,3 +138,22 @@ class Event(object):
             resource_name=self.resource_name, **self.stack.identifier())
 
         return identifier.EventIdentifier(event_id=str(self.uuid), **res_id)
+
+    def as_dict(self):
+        return {
+            'timestamp': self.timestamp.isoformat(),
+            'version': '0.1',
+            'type': 'os.heat.event',
+            'id': self.uuid,
+            'payload': {
+                'resource_name': self.resource_name,
+                'physical_resource_id': self.physical_resource_id,
+                'stack_id': self.stack.id,
+                'resource_action': self.action,
+                'resource_status': self.status,
+                'resource_status_reason': self.reason,
+                'resource_type': self.resource_type,
+                'resource_properties': self.resource_properties,
+                'version': '0.1'
+            }
+        }
