@@ -80,15 +80,6 @@ class GetParam(function.Function):
                 raise TypeError(_('Path components in "%s" '
                                   'must be strings') % self.fn_name)
 
-            if isinstance(collection, collections.Sequence
-                          ) and isinstance(key, six.string_types):
-                try:
-                    key = int(key)
-                except ValueError:
-                    raise TypeError(_("Path components in '%s' "
-                                      "must be a string that can be "
-                                      "parsed into an "
-                                      "integer.") % self.fn_name)
             return collection[key]
 
         try:
@@ -173,7 +164,7 @@ class GetAtt(GetAttThenSelect):
 
 
 class GetAttAllAttributes(GetAtt):
-    """A  function for resolving resource attributes.
+    """A function for resolving resource attributes.
 
     Takes the form::
 
@@ -247,9 +238,8 @@ class Replace(cfn_funcs.Replace):
 
         "<value_1> <value_2>"
 
-    This is implemented using python str.replace on each key. Longer keys are
-    substituted before shorter ones, but the order in which replacements are
-    performed is otherwise undefined.
+    This is implemented using Python's str.replace on each key. The order in
+    which replacements are performed is undefined.
     """
 
     def _parse_args(self):
@@ -273,29 +263,12 @@ class Replace(cfn_funcs.Replace):
 
 
 class ReplaceJson(Replace):
-    '''
-    A function for performing string substitutions.
+    """A function for performing string substitutions.
 
-    Takes the form::
-
-        str_replace:
-          template: <key_1> <key_2>
-          params:
-            <key_1>: <value_1>
-            <key_2>: <value_2>
-            ...
-
-    And resolves to::
-
-        "<value_1> <value_2>"
-
-    This is implemented using python str.replace on each key. Longer keys are
-    substituted before shorter ones, but the order in which replacements are
-    performed is otherwise undefined.
-
-    Non-string param values (e.g maps or lists) are serialized as JSON before
-    being substituted in.
-    '''
+    Behaves the same as Replace, but tolerates non-string parameter
+    values, e.g map/list - these are serialized as json before doing
+    the string substitution.
+    """
 
     def result(self):
         template = function.resolve(self._string)
@@ -332,13 +305,10 @@ class ReplaceJson(Replace):
                                                'value': value})
                 else:
                     raise TypeError(_('"%s" params must be strings, numbers, '
-                                      'list or map. ') % self.fn_name)
+                                      'list or map.') % self.fn_name)
 
             return string.replace(placeholder, six.text_type(value))
 
-        mapping = collections.OrderedDict(sorted(mapping.items(),
-                                                 key=lambda t: len(t[0]),
-                                                 reverse=True))
         return six.moves.reduce(replace, six.iteritems(mapping), template)
 
 
@@ -457,6 +427,47 @@ class JoinMultiple(function.Function):
         return delim.join(ensure_string(s) for s in strings)
 
 
+class MapMerge(function.Function):
+    """A function for merging maps.
+
+    Takes the form::
+
+        { "map_merge" : [{'k1': 'v1', 'k2': 'v2'}, {'k1': 'v2'}] }
+
+    And resolves to::
+
+        {'k1': 'v2', 'k2': 'v2'}
+
+    """
+
+    def __init__(self, stack, fn_name, args):
+        super(MapMerge, self).__init__(stack, fn_name, args)
+        example = (_('"%s" : [ { "key1": "val1" }, { "key2": "val2" } ]')
+                   % fn_name)
+        self.fmt_data = {'fn_name': fn_name, 'example': example}
+
+    def result(self):
+        args = function.resolve(self.args)
+
+        if not isinstance(args, collections.Sequence):
+            raise TypeError(_('Incorrect arguments to "%(fn_name)s" '
+                              'should be: %(example)s') % self.fmt_data)
+
+        def ensure_map(m):
+            if m is None:
+                return {}
+            elif isinstance(m, collections.Mapping):
+                return m
+            else:
+                msg = _('Incorrect arguments: Items to merge must be maps.')
+                raise TypeError(msg)
+
+        ret_map = {}
+        for m in args:
+            ret_map.update(ensure_map(m))
+        return ret_map
+
+
 class ResourceFacade(cfn_funcs.ResourceFacade):
     """A function for retrieving data in a parent provider template.
 
@@ -511,34 +522,39 @@ class Repeat(function.Function):
     def __init__(self, stack, fn_name, args):
         super(Repeat, self).__init__(stack, fn_name, args)
 
-        self._for_each, self._template = self._parse_args()
-
-    def _parse_args(self):
         if not isinstance(self.args, collections.Mapping):
             raise TypeError(_('Arguments to "%s" must be a map') %
                             self.fn_name)
 
+        # We don't check for invalid keys appearing here, which is wrong but
+        # it's probably too late to change
         try:
-            for_each = self.args['for_each']
-            template = self.args['template']
-        except (KeyError, TypeError):
+            self._for_each = self.args['for_each']
+            self._template = self.args['template']
+        except KeyError:
             example = ('''repeat:
               template: This is %var%
               for_each:
                 %var%: ['a', 'b', 'c']''')
-            raise KeyError(_('"repeat" syntax should be %s') %
-                           example)
+            raise KeyError(_('"repeat" syntax should be %s') % example)
 
-        if not isinstance(for_each, function.Function):
-            if not isinstance(for_each, collections.Mapping):
+    def validate(self):
+        super(Repeat, self).validate()
+
+        if not isinstance(self._for_each, function.Function):
+            if not isinstance(self._for_each, collections.Mapping):
                 raise TypeError(_('The "for_each" argument to "%s" must '
                                   'contain a map') % self.fn_name)
-            for v in six.itervalues(for_each):
-                if not isinstance(v, (list, function.Function)):
-                    raise TypeError(_('The values of the "for_each" argument '
-                                      'to "%s" must be lists') % self.fn_name)
 
-        return for_each, template
+            if not all(self._valid_list(v) for v in self._for_each.values()):
+                raise TypeError(_('The values of the "for_each" argument '
+                                  'to "%s" must be lists') % self.fn_name)
+
+    @staticmethod
+    def _valid_list(arg):
+        return (isinstance(arg, (collections.Sequence,
+                                 function.Function)) and
+                not isinstance(arg, six.string_types))
 
     def _do_replacement(self, keys, values, template):
         if isinstance(template, six.string_types):
@@ -555,16 +571,15 @@ class Repeat(function.Function):
 
     def result(self):
         for_each = function.resolve(self._for_each)
-        keys = list(six.iterkeys(for_each))
-        lists = [for_each[key] for key in keys]
-        if not all(isinstance(l, list) for l in lists):
+        if not all(self._valid_list(l) for l in for_each.values()):
             raise TypeError(_('The values of the "for_each" argument to '
                               '"%s" must be lists') % self.fn_name)
 
         template = function.resolve(self._template)
 
-        return [self._do_replacement(keys, items, template)
-                for items in itertools.product(*lists)]
+        keys, lists = six.moves.zip(*for_each.items())
+        return [self._do_replacement(keys, replacements, template)
+                for replacements in itertools.product(*lists)]
 
 
 class Digest(function.Function):
