@@ -39,10 +39,11 @@ class Schema(constr.Schema):
     """Parameter schema."""
 
     KEYS = (
-        TYPE, DESCRIPTION, DEFAULT, SCHEMA, CONSTRAINTS, HIDDEN, LABEL
+        TYPE, DESCRIPTION, DEFAULT, SCHEMA, CONSTRAINTS, HIDDEN,
+        LABEL, IMMUTABLE
     ) = (
         'Type', 'Description', 'Default', 'Schema', 'Constraints', 'NoEcho',
-        'Label'
+        'Label', 'Immutable'
     )
 
     PARAMETER_KEYS = PARAMETER_KEYS
@@ -56,14 +57,15 @@ class Schema(constr.Schema):
     )
 
     def __init__(self, data_type, description=None, default=None, schema=None,
-                 constraints=None, hidden=False, label=None):
+                 constraints=None, hidden=False, label=None, immutable=False):
         super(Schema, self).__init__(data_type=data_type,
                                      description=description,
                                      default=default,
                                      schema=schema,
                                      required=default is None,
                                      constraints=constraints,
-                                     label=label)
+                                     label=label,
+                                     immutable=immutable)
         self.hidden = hidden
 
     # Schema class validates default value for lists assuming list type. For
@@ -202,7 +204,7 @@ class Parameter(object):
         return ParamClass(name, schema, value)
 
     def __init__(self, name, schema, value=None):
-        """Initialisation of the parameter.
+        """Initialise the parameter.
 
         Initialise the Parameter with a name, schema and optional user-supplied
         value.
@@ -255,10 +257,9 @@ class Parameter(object):
         return self.user_value is not None or self.has_default()
 
     def hidden(self):
-        """Return if parameter is hidden.
+        """Return whether the parameter is hidden.
 
-        Return whether the parameter should be sanitised in any output to
-        the user.
+        Hidden parameters should be sanitised in any output to the user.
         """
         return self.schema.hidden
 
@@ -400,7 +401,10 @@ class CommaDelimitedListParam(ParsedParameter, collections.Sequence):
         return ",".join(self.value())
 
     def _validate(self, val, context):
-        parsed = self.parse(val)
+        try:
+            parsed = self.parse(val)
+        except ValueError as ex:
+            raise exception.StackValidationFailed(message=six.text_type(ex))
         self.schema.validate_value(parsed, context)
 
 
@@ -445,14 +449,17 @@ class JsonParam(ParsedParameter):
         return encodeutils.safe_decode(jsonutils.dumps(self.value()))
 
     def _validate(self, val, context):
-        val = self.parse(val)
-        self.schema.validate_value(val, context)
+        try:
+            parsed = self.parse(val)
+        except ValueError as ex:
+            raise exception.StackValidationFailed(message=six.text_type(ex))
+        self.schema.validate_value(parsed, context)
 
 
 class Parameters(collections.Mapping):
     """Parameters of a stack.
 
-    The parameters of a stack, with type checking, defaults etc., specified by
+    The parameters of a stack, with type checking, defaults, etc. specified by
     the stack's template.
     """
 
@@ -488,6 +495,8 @@ class Parameters(collections.Mapping):
         self.params = dict((p.name,
                             p) for p in itertools.chain(pseudo_parameters,
                                                         user_parameters))
+        self.non_pseudo_param_keys = [p for p in self.params if p not in
+                                      self.PSEUDO_PARAMETERS]
 
         for pd in six.iterkeys(param_defaults):
             if pd in self.params:
@@ -522,10 +531,10 @@ class Parameters(collections.Mapping):
         return self.params[key].value()
 
     def map(self, func, filter_func=lambda p: True):
-        """Map the supplied filter function onto each Parameter.
+        """Map the supplied function onto each Parameter.
 
-        Map the supplied filter function onto each Parameter (with an
-        optional filter function) and return the resulting dictionary.
+        Map the supplied function onto each Parameter (with an optional filter
+        function) and return the resulting dictionary.
         """
         return dict((n, func(p))
                     for n, p in six.iteritems(self.params) if filter_func(p))
@@ -580,3 +589,21 @@ class Parameters(collections.Mapping):
                                                              'ap-southeast-1',
                                                              'ap-northeast-1']
                                                             )]))
+
+    def immutable_params_modified(self, new_parameters, input_params):
+        # A parameter must have been present in the old stack for its
+        # immutability to be enforced
+        common_params = list(set(new_parameters.non_pseudo_param_keys)
+                             & set(self.non_pseudo_param_keys))
+        invalid_params = []
+        for param in common_params:
+            old_value = self.params[param]
+            if param in input_params:
+                new_value = input_params[param]
+            else:
+                new_value = new_parameters[param]
+            immutable = new_parameters.params[param].schema.immutable
+            if immutable and old_value.value() != new_value:
+                invalid_params.append(param)
+        if invalid_params:
+            return invalid_params
