@@ -18,10 +18,8 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
-from heat.engine.cfn import functions as cfn_funcs
 from heat.engine import constraints as constr
 from heat.engine import function
-from heat.engine.hot import functions as hot_funcs
 from heat.engine.hot import parameters as hot_param
 from heat.engine import parameters
 from heat.engine import support
@@ -64,10 +62,10 @@ class Schema(constr.Schema):
                  support_status=support.SupportStatus(),
                  allow_conversion=False):
         super(Schema, self).__init__(data_type, description, default,
-                                     schema, required, constraints)
+                                     schema, required, constraints,
+                                     immutable=immutable)
         self.implemented = implemented
         self.update_allowed = update_allowed
-        self.immutable = immutable
         self.support_status = support_status
         self.allow_conversion = allow_conversion
         # validate structural correctness of schema itself
@@ -350,10 +348,10 @@ class Properties(collections.Mapping):
 
     @staticmethod
     def schema_from_params(params_snippet):
-        """Convert a template snippet with parameters into a properties schema.
+        """Create properties schema from the parameters section of a template.
 
         :param params_snippet: parameter definition from a template
-        :returns: an equivalent properties schema for the specified params
+        :returns: equivalent properties schemata for the specified parameters
         """
         if params_snippet:
             return dict((n, Schema.from_parameter(p)) for n, p
@@ -536,21 +534,27 @@ class Properties(collections.Mapping):
             if schema.default is not None:
                 yield hot_param.HOTParamSchema.DEFAULT, schema.default
 
-            for constraint in schema.constraints:
-                if (isinstance(constraint, constr.Length) or
-                        isinstance(constraint, constr.Range)):
+            def constraint_items(constraint):
+
+                def range_min_max(constraint):
                     if constraint.min is not None:
                         yield hot_param.MIN, constraint.min
                     if constraint.max is not None:
                         yield hot_param.MAX, constraint.max
+
+                if isinstance(constraint, constr.Length):
+                    yield hot_param.LENGTH, dict(range_min_max(constraint))
+                elif isinstance(constraint, constr.Range):
+                    yield hot_param.RANGE, dict(range_min_max(constraint))
                 elif isinstance(constraint, constr.AllowedValues):
                     yield hot_param.ALLOWED_VALUES, list(constraint.allowed)
                 elif isinstance(constraint, constr.AllowedPattern):
                     yield hot_param.ALLOWED_PATTERN, constraint.pattern
 
-            if schema.type == schema.BOOLEAN:
-                yield hot_param.ALLOWED_VALUES, ['True', 'true',
-                                                 'False', 'false']
+            if schema.constraints:
+                yield (hot_param.HOTParamSchema.CONSTRAINTS,
+                       [dict(constraint_items(constraint)) for constraint
+                        in schema.constraints])
 
         return dict(param_items())
 
@@ -561,7 +565,10 @@ class Properties(collections.Mapping):
 
     @classmethod
     def schema_to_parameters_and_properties(cls, schema, template_type='cfn'):
-        """Generates properties with params resolved for a schema.
+        """Convert a schema to template parameters and properties.
+
+        This can be used to generate a provider template that matches the
+        given properties schemata.
 
         :param schema: A resource type's properties_schema
         :returns: A tuple of params and properties dicts
@@ -591,191 +598,3 @@ class Properties(collections.Mapping):
                            if s.implemented]
         param_items, prop_items = zip(*param_prop_defs)
         return dict(param_items), dict(prop_items)
-
-
-class TranslationRule(object):
-    """Translating mechanism one properties to another.
-
-    Mechanism uses list of rules, each defines by this class, and can be
-    executed. Working principe: during resource creating after properties
-    defining resource take list of rules, specified by method
-    translation_rules, which should be overloaded for each resource, if it's
-    needed, and execute each rule using translate_properties method. Next
-    operations are allowed:
-
-    - ADD. This rule allows to add some value to list-type properties. Only
-           list-type values can be added to such properties. Using for other
-           cases is prohibited and will be returned with error.
-    - REPLACE. This rule allows to replace some property value to another. Used
-           for all types of properties. Note, that if property has list type,
-           then value will be replaced for all elements of list, where it
-           needed. If element in such property must be replaced by value of
-           another element of this property, value_name must be defined.
-    - DELETE. This rule allows to delete some property. If property has list
-           type, then deleting affects value in all list elements.
-    """
-
-    RULE_KEYS = (ADD, REPLACE, DELETE) = ('Add', 'Replace', 'Delete')
-
-    def __init__(self, properties, rule, source_path, value=None,
-                 value_name=None, value_path=None):
-        """Add new rule for translating mechanism.
-
-        :param properties: properties of resource
-        :param rule: rule from RULE_KEYS
-        :param source_path: list with path to property, which value will be
-               affected in rule.
-        :param value: value which will be involved in rule
-        :param value_name: value_name which used for replacing properties
-               inside list-type properties.
-        :param value_path: path to value, which should be used for translation.
-        """
-        self.properties = properties
-        self.rule = rule
-        self.source_path = source_path
-        self.value = value or None
-        self.value_name = value_name
-        self.value_path = value_path
-
-        self.validate()
-
-    def validate(self):
-        if self.rule not in self.RULE_KEYS:
-            raise ValueError(_('There is no rule %(rule)s. List of allowed '
-                               'rules is: %(rules)s.') % {
-                'rule': self.rule,
-                'rules': ', '.join(self.RULE_KEYS)})
-        elif not isinstance(self.properties, Properties):
-            raise ValueError(_('Properties must be Properties type. '
-                               'Found %s.') % type(self.properties))
-        elif not isinstance(self.source_path, list):
-            raise ValueError(_('source_path should be a list with path '
-                               'instead of %s.') % type(self.source_path))
-        elif len(self.source_path) == 0:
-            raise ValueError(_('source_path must be non-empty list with '
-                               'path.'))
-        elif self.value_name and self.rule != self.REPLACE:
-            raise ValueError(_('Use value_name only for replacing list '
-                               'elements.'))
-        elif self.rule == self.ADD and not isinstance(self.value, list):
-            raise ValueError(_('value must be list type when rule is Add.'))
-
-    def execute_rule(self):
-        try:
-            (source_key, source_data) = self.get_data_from_source_path(
-                self.source_path)
-            if self.value_path:
-                (value_key, value_data) = self.get_data_from_source_path(
-                    self.value_path)
-                value = (value_data[value_key]
-                         if value_data and value_data.get(value_key)
-                         else self.value)
-            else:
-                (value_key, value_data) = None, None
-                value = self.value
-        except AttributeError:
-            return
-
-        if (source_data is None or (self.rule != self.DELETE and
-                                    (value is None and
-                                     self.value_name is None and
-                                     (value_data is None or
-                                      value_data.get(value_key) is None)))):
-            return
-
-        if self.rule == TranslationRule.ADD:
-            if isinstance(source_data, list):
-                source_data.extend(value)
-            else:
-                raise ValueError(_('Add rule must be used only for '
-                                   'lists.'))
-        elif self.rule == TranslationRule.REPLACE:
-            if isinstance(source_data, list):
-                for item in source_data:
-                    if item.get(self.value_name) and item.get(source_key):
-                        raise ValueError(_('Cannot use %(key)s and '
-                                           '%(name)s at the same time.')
-                                         % dict(key=source_key,
-                                                name=self.value_name))
-                    elif item.get(self.value_name) is not None:
-                        item[source_key] = item[self.value_name]
-                        del item[self.value_name]
-                    elif value is not None:
-                        item[source_key] = value
-            else:
-                if (source_data and source_data.get(source_key) and
-                        value_data and value_data.get(value_key)):
-                    raise ValueError(_('Cannot use %(key)s and '
-                                       '%(name)s at the same time.')
-                                     % dict(key=source_key,
-                                            name=value_key))
-                source_data[source_key] = value
-                # If value defined with value_path, need to delete value_path
-                # property data after it's replacing.
-                if value_data and value_data.get(value_key):
-                    del value_data[value_key]
-        elif self.rule == TranslationRule.DELETE:
-            if isinstance(source_data, list):
-                for item in source_data:
-                    if item.get(source_key) is not None:
-                        del item[source_key]
-            else:
-                del source_data[source_key]
-
-    def get_data_from_source_path(self, path):
-        def get_props(props, key):
-            props = props.get(key)
-            if props.schema.schema is not None:
-                keys = list(props.schema.schema)
-                schemata = dict((k, props.schema.schema[k])
-                                for k in keys)
-                props = dict((k, Property(s, k))
-                             for k, s in schemata.items())
-            return props
-
-        def resolve_param(param):
-            """Check whether if given item is param and resolve, if it is."""
-            # NOTE(prazumovsky): If property uses removed in HOT function,
-            # we should not translate it for correct validating and raising
-            # validation error.
-            if isinstance(param, hot_funcs.Removed):
-                raise AttributeError(_('Property uses removed function.'))
-            if isinstance(param, (hot_funcs.GetParam, cfn_funcs.ParamRef)):
-                return function.resolve(param)
-            elif isinstance(param, list):
-                return [resolve_param(param_item) for param_item in param]
-            else:
-                return param
-
-        source_key = path[0]
-        data = self.properties.data
-        props = self.properties.props
-        for key in path:
-            if isinstance(data, list):
-                source_key = key
-            elif data.get(key) is not None:
-                # NOTE(prazumovsky): There's no need to resolve other functions
-                # because we can translate all function to another path. But if
-                # list or map type property equals to get_param function, need
-                # to resolve it for correct translating.
-                data[key] = resolve_param(data[key])
-                if isinstance(data[key], (dict, list)):
-                    data = data[key]
-                    props = get_props(props, key)
-                else:
-                    source_key = key
-            elif data.get(key) is None:
-                if (self.rule == TranslationRule.DELETE or
-                        (self.rule == TranslationRule.REPLACE and
-                         self.value_name)):
-                    return None, None
-                elif props.get(key).type() == Schema.LIST:
-                    data[key] = []
-                elif props.get(key).type() == Schema.MAP:
-                    data[key] = {}
-                else:
-                    source_key = key
-                    continue
-                data = data.get(key)
-                props = get_props(props, key)
-        return source_key, data
