@@ -189,7 +189,6 @@ class AutoScalingGroup(instgrp.InstanceGroup, cooldown.CooldownMixin):
     }
 
     def handle_create(self):
-        self.validate_launchconfig()
         return self.create_with_template(self.child_template())
 
     def _make_launch_config_resource(self, name, props):
@@ -253,7 +252,8 @@ class AutoScalingGroup(instgrp.InstanceGroup, cooldown.CooldownMixin):
                                               lower, upper)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        """
+        """Updates self.properties, if Properties has changed.
+
         If Properties has changed, update self.properties, so we get the new
         values during any subsequent adjustment.
         """
@@ -280,9 +280,7 @@ class AutoScalingGroup(instgrp.InstanceGroup, cooldown.CooldownMixin):
     def adjust(self, adjustment,
                adjustment_type=sc_util.CFN_CHANGE_IN_CAPACITY,
                min_adjustment_step=None):
-        """
-        Adjust the size of the scaling group if the cooldown permits.
-        """
+        """Adjust the size of the scaling group if the cooldown permits."""
         if not self._is_scaling_allowed():
             LOG.info(_LI("%(name)s NOT performing scaling adjustment, "
                          "cooldown %(cooldown)s"),
@@ -334,7 +332,7 @@ class AutoScalingGroup(instgrp.InstanceGroup, cooldown.CooldownMixin):
         return changed_size
 
     def _tags(self):
-        """Add Identifing Tags to all servers in the group.
+        """Add Identifying Tags to all servers in the group.
 
         This is so the Dimensions received from cfn-push-stats all include
         the groupname and stack id.
@@ -386,10 +384,11 @@ class AutoScalingGroup(instgrp.InstanceGroup, cooldown.CooldownMixin):
         super(AutoScalingGroup, self).validate()
 
     def _resolve_attribute(self, name):
-        '''
+        """Resolves the resource's attributes.
+
         heat extension: "InstanceList" returns comma delimited list of server
         ip addresses.
-        '''
+        """
         if name == self.INSTANCE_LIST:
             return u','.join(inst.FnGetAtt('PublicIp')
                              for inst in grouputils.get_members(self)) or None
