@@ -91,8 +91,8 @@ common_properties_schema = {
         _('Describe time constraints for the alarm. '
           'Only evaluate the alarm if the time at evaluation '
           'is within this time constraint. Start point(s) of '
-          'the constraint are specified with a cron expression,'
-          'whereas its duration is given in seconds. '
+          'the constraint are specified with a cron expression, '
+          'whereas its duration is given in seconds.'
           ),
         schema=properties.Schema(
             properties.Schema.MAP,
@@ -127,7 +127,7 @@ common_properties_schema = {
                 TIMEZONE: properties.Schema(
                     properties.Schema.STRING,
                     _("Timezone for the time constraint "
-                      "(eg. 'Taiwan/Taipei', 'Europe/Amsterdam')"),
+                      "(eg. 'Taiwan/Taipei', 'Europe/Amsterdam')."),
                     constraints=[
                         constraints.CustomConstraint('timezone')
                     ],
@@ -182,6 +182,15 @@ def actions_to_urls(stack, properties):
 
 
 class CeilometerAlarm(resource.Resource):
+    """A resource that implements alarming service of Ceilometer.
+
+    A resource that allows for the setting alarms based on threshold evaluation
+    for a collection of samples. Also, you can define actions to take if state
+    of watched resource will be satisfied specified conditions. For example, it
+    can watch for the memory consumption and when it reaches 70% on a given
+    instance if the instance has been up for more than 10 min, some action will
+    be called.
+    """
 
     PROPERTIES = (
         COMPARISON_OPERATOR, EVALUATION_PERIODS, METER_NAME, PERIOD,
@@ -270,12 +279,12 @@ class CeilometerAlarm(resource.Resource):
                     ),
                     QF_OP: properties.Schema(
                         properties.Schema.STRING,
-                        _('Comparison operator'),
+                        _('Comparison operator.'),
                         constraints=[QF_OP_VALS]
                     ),
                     QF_VALUE: properties.Schema(
                         properties.Schema.STRING,
-                        _('String value with which to compare')
+                        _('String value with which to compare.')
                     )
                 }
             )
@@ -369,7 +378,7 @@ class CeilometerAlarm(resource.Resource):
             wr = watchrule.WatchRule.load(
                 self.context, watch_name=self.physical_resource_name())
             wr.destroy()
-        except exception.WatchRuleNotFound:
+        except exception.EntityNotFound:
             pass
 
         return super(CeilometerAlarm, self).handle_delete()
@@ -425,6 +434,12 @@ class BaseCeilometerAlarm(resource.Resource):
 
 
 class CombinationAlarm(BaseCeilometerAlarm):
+    """A resource that implements combination of Ceilometer alarms.
+
+    Allows to use alarm as a combination of other alarms with some operator:
+    activate this alarm if any alarm in combination has been activated or
+    if all alarms in combination have been activated.
+    """
 
     support_status = support.SupportStatus(version='2014.1')
 
