@@ -32,6 +32,12 @@ from heat.scaling import template
 
 
 class InstanceGroup(stack_resource.StackResource):
+    """An instance group that can scale arbitrary instances.
+
+    A resource allowing for the creating number of defined with
+    AWS::AutoScaling::LaunchConfiguration instances. Allows to associate
+    scaled resources with loadbalancer resources.
+    """
 
     PROPERTIES = (
         AVAILABILITY_ZONES, LAUNCH_CONFIGURATION_NAME, SIZE,
@@ -91,10 +97,12 @@ class InstanceGroup(stack_resource.StackResource):
                 schema={
                     TAG_KEY: properties.Schema(
                         properties.Schema.STRING,
+                        _('Tag key.'),
                         required=True
                     ),
                     TAG_VALUE: properties.Schema(
                         properties.Schema.STRING,
+                        _('Tag value.'),
                         required=True
                     ),
                 },
@@ -122,20 +130,9 @@ class InstanceGroup(stack_resource.StackResource):
                                           schema=rolling_update_schema)
     }
 
-    def __init__(self, name, json_snippet, stack):
-        """
-        UpdatePolicy is currently only specific to InstanceGroup and
-        AutoScalingGroup. Therefore, init is overridden to parse for the
-        UpdatePolicy.
-        """
-        super(InstanceGroup, self).__init__(name, json_snippet, stack)
-        self.update_policy = self.t.update_policy(self.update_policy_schema,
-                                                  self.context)
-
     def validate(self):
-        """
-        Add validation for update_policy
-        """
+        """Add validation for update_policy."""
+        self.validate_launchconfig()
         super(InstanceGroup, self).validate()
 
         if self.update_policy is not None:
@@ -171,14 +168,12 @@ class InstanceGroup(stack_resource.StackResource):
 
     def handle_create(self):
         """Create a nested stack and add the initial resources to it."""
-        self.validate_launchconfig()
         num_instances = self.properties[self.SIZE]
         initial_template = self._create_template(num_instances)
         return self.create_with_template(initial_template)
 
     def check_create_complete(self, task):
-        """
-        When stack creation is done, update the load balancer.
+        """When stack creation is done, update the loadbalancer.
 
         If any instances failed to be created, delete them.
         """
@@ -188,7 +183,8 @@ class InstanceGroup(stack_resource.StackResource):
         return done
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        """
+        """Updates self.properties, if Properties has changed.
+
         If Properties has changed, update self.properties, so we
         get the new values during any subsequent adjustment.
         """
@@ -214,8 +210,8 @@ class InstanceGroup(stack_resource.StackResource):
             self.resize(curr_size)
 
     def _tags(self):
-        """
-        Make sure that we add a tag that Ceilometer can pick up.
+        """Make sure that we add a tag that Ceilometer can pick up.
+
         These need to be prepended with 'metering.'.
         """
         tags = self.properties.get(self.TAGS) or []
@@ -240,28 +236,23 @@ class InstanceGroup(stack_resource.StackResource):
 
         return conf, props
 
-    def _get_instance_definition(self):
+    def _get_resource_definition(self):
         conf, props = self._get_conf_properties()
         return rsrc_defn.ResourceDefinition(None,
                                             SCALED_RESOURCE_TYPE,
                                             props,
                                             conf.t.metadata())
 
-    def _get_instance_templates(self):
-        """Get templates for resource instances."""
-        return [(instance.name, instance.t)
-                for instance in grouputils.get_members(self)]
-
     def _create_template(self, num_instances, num_replace=0,
                          template_version=('HeatTemplateFormatVersion',
                                            '2012-12-12')):
-        """
-        Create a template to represent autoscaled instances.
+        """Create a template to represent autoscaled instances.
 
         Also see heat.scaling.template.member_definitions.
         """
-        instance_definition = self._get_instance_definition()
-        old_resources = self._get_instance_templates()
+        instance_definition = self._get_resource_definition()
+        old_resources = grouputils.get_member_definitions(self,
+                                                          include_failed=True)
         definitions = template.member_definitions(
             old_resources, instance_definition, num_instances, num_replace,
             short_id.generate_id)
@@ -291,8 +282,9 @@ class InstanceGroup(stack_resource.StackResource):
         return self.stack.timeout_secs() - total_pause_time
 
     def _replace(self, min_in_service, batch_size, pause_sec):
-        """
-        Replace the instances in the group using updated launch configuration
+        """Replace the instances in the group.
+
+        Replace the instances in the group using updated launch configuration.
         """
         def changing_instances(tmpl):
             instances = grouputils.get_members(self)
@@ -331,8 +323,7 @@ class InstanceGroup(stack_resource.StackResource):
 
     @staticmethod
     def _get_batches(capacity, batch_size, min_in_service):
-        """
-        Return an iterator over the batches in a batched update.
+        """Return an iterator over the batches in a batched update.
 
         Each batch is a tuple comprising the total size of the group after
         processing the batch, and the number of members that can receive the
@@ -376,14 +367,15 @@ class InstanceGroup(stack_resource.StackResource):
             lb_dict = dict((name, self.stack[name]) for name in lb_names)
             lbutils.reload_loadbalancers(self, lb_dict, exclude)
 
-    def FnGetRefId(self):
+    def get_reference_id(self):
         return self.physical_resource_name_or_FnGetRefId()
 
     def _resolve_attribute(self, name):
-        '''
-        heat extension: "InstanceList" returns comma delimited list of server
+        """Resolves the resource's attributes.
+
+        Heat extension: "InstanceList" returns comma delimited list of server
         ip addresses.
-        '''
+        """
         if name == self.INSTANCE_LIST:
             return u','.join(inst.FnGetAtt('PublicIp')
                              for inst in grouputils.get_members(self)) or None
