@@ -20,14 +20,13 @@ from heat.engine import support
 
 
 class NoneResource(resource.Resource):
-    '''
-    A resource which enables easily disabling certain resources via the
-    resource_registry.  It does nothing, but can effectively stub out
-    any other resource because it will accept any properties and return
-    any attribute (as None).  Note this resource always does nothing
-    on update (e.g it is not replaced even if a change to the stubbed
-    resource properties would cause replacement).
-    '''
+    """Enables easily disabling certain resources via the resource_registry.
+
+    It does nothing, but can effectively stub out any other resource because it
+    will accept any properties and return any attribute (as None). Note this
+    resource always does nothing on update (e.g it is not replaced even if a
+    change to the stubbed resource properties would cause replacement).
+    """
 
     support_status = support.SupportStatus(version='5.0.0')
     properties_schema = {}
@@ -37,8 +36,10 @@ class NoneResource(resource.Resource):
                       prev_resource, check_init_complete=True):
         return False
 
-    def reparse(self):
+    def reparse(self, translate=True, client_resolve=True):
         self.properties = properties.Properties(schema={}, data={})
+        if translate:
+            self.translate_properties(self.properties, client_resolve)
 
     def handle_create(self):
         self.resource_id_set(six.text_type(uuid.uuid4()))
@@ -46,7 +47,7 @@ class NoneResource(resource.Resource):
     def validate(self):
         pass
 
-    def FnGetAtt(self, key, *path):
+    def get_attribute(self, key, *path):
         return None
 
 
