@@ -13,6 +13,7 @@
 
 import copy
 import six
+import uuid
 
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -27,6 +28,7 @@ from heat.engine import resource
 from heat.engine.resources.openstack.heat import resource_group
 from heat.engine.resources.openstack.heat import software_config as sc
 from heat.engine.resources import signal_responder
+from heat.engine import rsrc_defn
 from heat.engine import support
 from heat.rpc import api as rpc_api
 
@@ -37,9 +39,9 @@ LOG = logging.getLogger(__name__)
 
 
 class SoftwareDeployment(signal_responder.SignalResponder):
-    '''
-    This resource associates a server with some configuration which
-    is to be deployed to that server.
+    """This resource associates a server with some configuration.
+
+    The configuration is to be deployed to that server.
 
     A deployment allows input values to be specified which map to the inputs
     schema defined in the config resource. These input values are interpreted
@@ -64,7 +66,7 @@ class SoftwareDeployment(signal_responder.SignalResponder):
     actions, or a config can read the value of the deploy_action input to
     allow conditional logic to perform different configuration for different
     actions.
-    '''
+    """
 
     support_status = support.SupportStatus(version='2014.1')
 
@@ -178,7 +180,7 @@ class SoftwareDeployment(signal_responder.SignalResponder):
             type=attributes.Schema.STRING
         ),
         STATUS_CODE: attributes.Schema(
-            _("Returned status code from the configuration execution"),
+            _("Returned status code from the configuration execution."),
             type=attributes.Schema.STRING
         ),
     }
@@ -247,9 +249,13 @@ class SoftwareDeployment(signal_responder.SignalResponder):
         else:
             config = {}
 
-        if (action not in self.properties[self.DEPLOY_ACTIONS]
-                and not config.get(
-                    rpc_api.SOFTWARE_CONFIG_GROUP) == 'component'):
+        if config.get(rpc_api.SOFTWARE_CONFIG_GROUP) == 'component':
+            valid_actions = set()
+            for conf in config['config']['configs']:
+                valid_actions.update(conf['actions'])
+            if action not in valid_actions:
+                return
+        elif action not in self.properties[self.DEPLOY_ACTIONS]:
             return
 
         props = self._build_properties(
@@ -258,12 +264,14 @@ class SoftwareDeployment(signal_responder.SignalResponder):
             action)
 
         if self.resource_id is None:
+            resource_id = str(uuid.uuid4())
+            self.resource_id_set(resource_id)
             sd = self.rpc_client().create_software_deployment(
                 self.context,
+                deployment_id=resource_id,
                 server_id=self.properties[SoftwareDeployment.SERVER],
                 stack_user_project_id=self.stack.stack_user_project_id,
                 **props)
-            self.resource_id_set(sd[rpc_api.SOFTWARE_DEPLOYMENT_ID])
         else:
             sd = self.rpc_client().show_software_deployment(
                 self.context, self.resource_id)
@@ -287,11 +295,9 @@ class SoftwareDeployment(signal_responder.SignalResponder):
             return True
         elif status == SoftwareDeployment.FAILED:
             status_reason = sd[rpc_api.SOFTWARE_DEPLOYMENT_STATUS_REASON]
-            message = _("Deployment to server "
-                        "failed: %s") % status_reason
-            LOG.error(message)
-            exc = exception.Error(message)
-            raise exc
+            message = _("Deployment to server failed: %s") % status_reason
+            LOG.info(message)
+            raise exception.Error(message)
 
     def empty_config(self):
         return ''
@@ -505,10 +511,8 @@ class SoftwareDeployment(signal_responder.SignalResponder):
             self.context, self.resource_id, details,
             timeutils.utcnow().isoformat())
 
-    def FnGetAtt(self, key, *path):
-        '''
-        Resource attributes map to deployment outputs values
-        '''
+    def get_attribute(self, key, *path):
+        """Resource attributes map to deployment outputs values."""
         sd = self.rpc_client().show_software_deployment(
             self.context, self.resource_id)
         ov = sd[rpc_api.SOFTWARE_DEPLOYMENT_OUTPUT_VALUES] or {}
@@ -528,11 +532,10 @@ class SoftwareDeployment(signal_responder.SignalResponder):
         return None
 
     def validate(self):
-        '''
-        Validate any of the provided params
+        """Validate any of the provided params.
 
         :raises StackValidationFailed: if any property failed validation.
-        '''
+        """
         super(SoftwareDeployment, self).validate()
         server = self.properties[self.SERVER]
         if server:
@@ -547,16 +550,16 @@ class SoftwareDeployment(signal_responder.SignalResponder):
 
 
 class SoftwareDeploymentGroup(resource_group.ResourceGroup):
-    '''
-    This resource associates a group of servers with some configuration which
-    is to be deployed to all servers in the group.
+    """This resource associates a group of servers with some configuration.
+
+    The configuration is to be deployed to all servers in the group.
 
     The properties work in a similar way to OS::Heat::SoftwareDeployment,
     and in addition to the attributes documented, you may pass any
     attribute supported by OS::Heat::SoftwareDeployment, including those
     exposing arbitrary outputs, and return a map of deployment names to
     the specified attribute.
-    '''
+    """
 
     support_status = support.SupportStatus(version='5.0.0')
 
@@ -611,7 +614,7 @@ class SoftwareDeploymentGroup(resource_group.ResourceGroup):
         ),
         STATUS_CODES: attributes.Schema(
             _("A map of Nova names and returned status code from the "
-              "configuration execution"),
+              "configuration execution."),
             type=attributes.Schema.MAP
         ),
     }
@@ -624,27 +627,18 @@ class SoftwareDeploymentGroup(resource_group.ResourceGroup):
     def _resource_names(self):
         return six.iterkeys(self.properties.get(self.SERVERS, {}))
 
-    def _do_prop_replace(self, res_name, res_def_template):
-        res_def = copy.deepcopy(res_def_template)
-        props = res_def[self.RESOURCE_DEF_PROPERTIES]
-        servers = self.properties.get(self.SERVERS, {})
+    def get_resource_def(self, include_all=False):
+        return dict(self.properties)
+
+    def build_resource_definition(self, res_name, res_defn):
+        props = copy.deepcopy(res_defn)
+        servers = props.pop(self.SERVERS)
         props[SoftwareDeployment.SERVER] = servers.get(res_name)
-        return res_def
+        return rsrc_defn.ResourceDefinition(res_name,
+                                            'OS::Heat::SoftwareDeployment',
+                                            props, None)
 
-    def _build_resource_definition(self, include_all=False):
-        p = self.properties
-        return {
-            self.RESOURCE_DEF_TYPE: 'OS::Heat::SoftwareDeployment',
-            self.RESOURCE_DEF_PROPERTIES: {
-                self.CONFIG: p[self.CONFIG],
-                self.INPUT_VALUES: p[self.INPUT_VALUES],
-                self.DEPLOY_ACTIONS: p[self.DEPLOY_ACTIONS],
-                self.SIGNAL_TRANSPORT: p[self.SIGNAL_TRANSPORT],
-                self.NAME: p[self.NAME],
-            }
-        }
-
-    def FnGetAtt(self, key, *path):
+    def get_attribute(self, key, *path):
         rg = super(SoftwareDeploymentGroup, self)
         if key == self.STDOUTS:
             n_attr = SoftwareDeployment.STDOUT
@@ -657,15 +651,14 @@ class SoftwareDeploymentGroup(resource_group.ResourceGroup):
             # including arbitrary outputs, so we can't validate here
             n_attr = key
 
-        rg_attr = rg.FnGetAtt(rg.ATTR_ATTRIBUTES, n_attr)
+        rg_attr = rg.get_attribute(rg.ATTR_ATTRIBUTES, n_attr)
         return attributes.select_from_attribute(rg_attr, path)
 
 
 class SoftwareDeployments(SoftwareDeploymentGroup):
 
-    deprecation_msg = _('This resource is deprecated and use is discouraged. '
-                        'Please use resource OS::Heat:SoftwareDeploymentGroup '
-                        'instead.')
+    deprecation_msg = _('Use of this resource is discouraged. Please use '
+                        'OS::Heat::SoftwareDeploymentGroup instead.')
     support_status = support.SupportStatus(status=support.DEPRECATED,
                                            message=deprecation_msg,
                                            version='2014.2')
