@@ -21,7 +21,12 @@ from heat.engine import support
 
 class KeystoneGroup(resource.Resource,
                     role_assignments.KeystoneRoleAssignmentMixin):
-    """Heat Template Resource for Keystone Group."""
+    """Heat Template Resource for Keystone Group.
+
+    Groups are a container representing a collection of users. A group itself
+    must be owned by a specific domain, and hence all group names are not
+    globally unique, but only unique to their domain.
+    """
 
     support_status = support.SupportStatus(
         version='2015.1',
@@ -68,76 +73,47 @@ class KeystoneGroup(resource.Resource,
     def client(self):
         return super(KeystoneGroup, self).client().client
 
-    def _create_group(self,
-                      group_name,
-                      description,
-                      domain):
-        domain = self.client_plugin().get_domain_id(domain)
+    def handle_create(self):
+        group_name = (self.properties[self.NAME] or
+                      self.physical_resource_name())
+        description = self.properties[self.DESCRIPTION]
+        domain = self.client_plugin().get_domain_id(
+            self.properties[self.DOMAIN])
 
-        return self.client().groups.create(
+        group = self.client().groups.create(
             name=group_name,
             domain=domain,
             description=description)
 
-    def _delete_group(self, group_id):
-        return self.client().groups.delete(group_id)
-
-    def _update_group(self,
-                      group_id,
-                      domain,
-                      new_name=None,
-                      new_description=None):
-        values = dict()
-
-        if new_name is not None:
-            values['name'] = new_name
-        if new_description is not None:
-            values['description'] = new_description
-
-        values['group'] = group_id
-        domain = self.client_plugin().get_domain_id(domain)
-        values['domain_id'] = domain
-        return self.client().groups.update(**values)
-
-    def handle_create(self):
-        group_name = (self.properties.get(self.NAME) or
-                      self.physical_resource_name())
-        description = self.properties.get(self.DESCRIPTION)
-        domain = self.properties.get(self.DOMAIN)
-
-        group = self._create_group(
-            group_name=group_name,
-            description=description,
-            domain=domain
-        )
-
         self.resource_id_set(group.id)
 
         self.create_assignment(group_id=group.id)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        name = prop_diff.get(self.NAME) or self.physical_resource_name()
-        description = prop_diff.get(self.DESCRIPTION)
-        domain = (prop_diff.get(self.DOMAIN) or
-                  self._stored_properties_data.get(self.DOMAIN))
+        if prop_diff:
+            name = None
+            # Don't update the name if no change
+            if self.NAME in prop_diff:
+                name = prop_diff[self.NAME] or self.physical_resource_name()
 
-        self._update_group(
-            group_id=self.resource_id,
-            new_name=name,
-            new_description=description,
-            domain=domain
-        )
+            description = prop_diff.get(self.DESCRIPTION)
+            domain = (prop_diff.get(self.DOMAIN) or
+                      self._stored_properties_data.get(self.DOMAIN))
+            domain_id = self.client_plugin().get_domain_id(domain)
 
-        self.update_assignment(prop_diff=prop_diff, group_id=self.resource_id)
+            self.client().groups.update(
+                group=self.resource_id,
+                name=name,
+                description=description,
+                domain_id=domain_id)
+
+            self.update_assignment(prop_diff=prop_diff,
+                                   group_id=self.resource_id)
 
     def handle_delete(self):
         if self.resource_id is not None:
-            try:
-                self.delete_assignment(group_id=self.resource_id)
-
-                self._delete_group(group_id=self.resource_id)
-            except Exception as ex:
-                self.client_plugin().ignore_not_found(ex)
+            with self.client_plugin().ignore_not_found:
+                self.client().groups.delete(self.resource_id)
 
 
 def resource_mapping():
