@@ -20,9 +20,7 @@ from heat.engine import support
 
 
 class KeystoneRoleAssignmentMixin(object):
-    '''
-    Keystone Role assignment class implements role assignments between
-    user/groups and project/domain.
+    """Implements role assignments between user/groups and project/domain.
 
     heat_template_version: 2013-05-23
 
@@ -48,7 +46,7 @@ class KeystoneRoleAssignmentMixin(object):
               domain: {get_param: group_role_domain}
             - role: {get_param: group_role}
               project: {get_param: group_role_project}
-    '''
+    """
 
     PROPERTIES = (
         ROLES
@@ -72,20 +70,20 @@ class KeystoneRoleAssignmentMixin(object):
                 schema={
                     ROLE: properties.Schema(
                         properties.Schema.STRING,
-                        _('Keystone role'),
+                        _('Keystone role.'),
                         required=True,
                         constraints=([constraints.
                                      CustomConstraint('keystone.role')])
                     ),
                     PROJECT: properties.Schema(
                         properties.Schema.STRING,
-                        _('Keystone project'),
+                        _('Keystone project.'),
                         constraints=([constraints.
                                      CustomConstraint('keystone.project')])
                     ),
                     DOMAIN: properties.Schema(
                         properties.Schema.STRING,
-                        _('Keystone domain'),
+                        _('Keystone domain.'),
                         constraints=([constraints.
                                      CustomConstraint('keystone.domain')])
                     ),
@@ -314,7 +312,10 @@ class KeystoneRoleAssignmentMixin(object):
 
 class KeystoneUserRoleAssignment(resource.Resource,
                                  KeystoneRoleAssignmentMixin):
-    '''Resource for granting roles to a user.'''
+    """Resource for granting roles to a user.
+
+    Resource for specifying users and their's roles.
+    """
 
     support_status = support.SupportStatus(
         version='5.0.0',
@@ -365,7 +366,10 @@ class KeystoneUserRoleAssignment(resource.Resource,
 
 class KeystoneGroupRoleAssignment(resource.Resource,
                                   KeystoneRoleAssignmentMixin):
-    '''Resource for granting roles to a group.'''
+    """Resource for granting roles to a group.
+
+    Resource for specifying groups and their's roles.
+    """
 
     support_status = support.SupportStatus(
         version='5.0.0',
