@@ -11,6 +11,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
+
 from oslo_serialization import jsonutils
 import six
 import yaml
@@ -27,6 +29,15 @@ from heat.engine import support
 
 class Workflow(signal_responder.SignalResponder,
                resource.Resource):
+    """A resource that implements Mistral workflow.
+
+    Workflow represents a process that can be described in a various number of
+    ways and that can do some job interesting to the end user. Each workflow
+    consists of tasks (at least one) describing what exact steps should be made
+    during workflow execution.
+
+    For detailed description how to use Workflow, read Mistral documentation.
+    """
 
     support_status = support.SupportStatus(version='2015.1')
 
@@ -35,22 +46,23 @@ class Workflow(signal_responder.SignalResponder,
     entity = 'workflows'
 
     PROPERTIES = (
-        NAME, TYPE, DESCRIPTION, INPUT, OUTPUT, TASKS, PARAMS, TASK_DEFAULTS
+        NAME, TYPE, DESCRIPTION, INPUT, OUTPUT, TASKS, PARAMS,
+        TASK_DEFAULTS, USE_REQUEST_BODY_AS_INPUT
     ) = (
         'name', 'type', 'description', 'input', 'output', 'tasks', 'params',
-        'task_defaults'
+        'task_defaults', 'use_request_body_as_input'
     )
 
     _TASKS_KEYS = (
         TASK_NAME, TASK_DESCRIPTION, ON_ERROR, ON_COMPLETE, ON_SUCCESS,
         POLICIES, ACTION, WORKFLOW, PUBLISH, TASK_INPUT, REQUIRES,
         RETRY, WAIT_BEFORE, WAIT_AFTER, PAUSE_BEFORE, TIMEOUT,
-        WITH_ITEMS, KEEP_RESULT, TARGET
+        WITH_ITEMS, KEEP_RESULT, TARGET, JOIN
     ) = (
         'name', 'description', 'on_error', 'on_complete', 'on_success',
         'policies', 'action', 'workflow', 'publish', 'input', 'requires',
         'retry', 'wait_before', 'wait_after', 'pause_before', 'timeout',
-        'with_items', 'keep_result', 'target'
+        'with_items', 'keep_result', 'target', 'join'
     )
 
     _TASKS_TASK_DEFAULTS = [
@@ -84,6 +96,17 @@ class Workflow(signal_responder.SignalResponder,
             required=True,
             update_allowed=True
         ),
+        USE_REQUEST_BODY_AS_INPUT: properties.Schema(
+            properties.Schema.BOOLEAN,
+            _('Defines the method in which the request body for signaling a '
+              'workflow would be parsed. In case this property is set to '
+              'True, the body would be parsed as a simple json where each '
+              'key is a workflow input, in other cases body would be parsed '
+              'expecting a specific json format with two keys: "input" and '
+              '"params".'),
+            update_allowed=True,
+            support_status=support.SupportStatus(version='6.0.0')
+        ),
         DESCRIPTION: properties.Schema(
             properties.Schema.STRING,
             _('Workflow description.'),
@@ -142,20 +165,20 @@ class Workflow(signal_responder.SignalResponder,
                 ),
                 WAIT_BEFORE: properties.Schema(
                     properties.Schema.INTEGER,
-                    _('Defines a delay in seconds that Mistral Engine'
-                      ' should wait before starting a task.')
+                    _('Defines a delay in seconds that Mistral Engine '
+                      'should wait before starting a task.')
                 ),
                 WAIT_AFTER: properties.Schema(
                     properties.Schema.INTEGER,
-                    _('Defines a delay in seconds that Mistral Engine'
-                      ' should wait after a task has completed before'
-                      ' starting next tasks defined in '
+                    _('Defines a delay in seconds that Mistral Engine '
+                      'should wait after a task has completed before '
+                      'starting next tasks defined in '
                       'on-success, on-error or on-complete.')
                 ),
                 PAUSE_BEFORE: properties.Schema(
                     properties.Schema.BOOLEAN,
                     _('Defines whether Mistral Engine should put the '
-                      'workflow on hold or not before starting a task')
+                      'workflow on hold or not before starting a task.')
                 ),
                 TIMEOUT: properties.Schema(
                     properties.Schema.INTEGER,
@@ -293,6 +316,18 @@ class Workflow(signal_responder.SignalResponder,
                           'should be sent to.'),
                         support_status=support.SupportStatus(version='5.0.0')
                     ),
+                    JOIN: properties.Schema(
+                        properties.Schema.STRING,
+                        _('Allows to synchronize multiple parallel workflow '
+                          'branches and aggregate their data. '
+                          'Valid inputs: all - the task will run only if '
+                          'all upstream tasks are completed. '
+                          'Any numeric value - then the task will run once '
+                          'at least this number of upstream tasks are '
+                          'completed and corresponding conditions have '
+                          'triggered.'),
+                        support_status=support.SupportStatus(version='6.0.0')
+                    ),
                 },
             ),
             required=True,
@@ -319,33 +354,41 @@ class Workflow(signal_responder.SignalResponder,
         )
     }
 
-    def FnGetRefId(self):
+    def get_reference_id(self):
         return self._workflow_name()
 
-    def _validate_signal_data(self, data):
-        if data is not None:
-            input_value = data.get(self.SIGNAL_DATA_INPUT)
-            params_value = data.get(self.SIGNAL_DATA_PARAMS)
-            if input_value is not None:
-                if not isinstance(input_value, dict):
-                    message = (_('Input in signal data must be a map, '
-                                 'find a %s') % type(input_value))
-                    raise exception.StackValidationFailed(
-                        error=_('Signal data error'),
-                        message=message)
-                for key in six.iterkeys(input_value):
-                    if (self.properties.get(self.INPUT) is None
-                            or key not in self.properties.get(self.INPUT)):
-                        message = _('Unknown input %s') % key
-                        raise exception.StackValidationFailed(
-                            error=_('Signal data error'),
-                            message=message)
-            if params_value is not None and not isinstance(params_value, dict):
-                    message = (_('Params must be a map, find a '
-                                 '%s') % type(params_value))
+    def _get_inputs_and_params(self, data):
+        inputs = None
+        params = None
+        if self.properties.get(self.USE_REQUEST_BODY_AS_INPUT):
+            inputs = data
+        else:
+            if data is not None:
+                inputs = data.get(self.SIGNAL_DATA_INPUT)
+                params = data.get(self.SIGNAL_DATA_PARAMS)
+        return inputs, params
+
+    def _validate_signal_data(self, inputs, params):
+        if inputs is not None:
+            if not isinstance(inputs, dict):
+                message = (_('Input in signal data must be a map, '
+                             'find a %s') % type(inputs))
+                raise exception.StackValidationFailed(
+                    error=_('Signal data error'),
+                    message=message)
+            for key in six.iterkeys(inputs):
+                if (self.properties.get(self.INPUT) is None or
+                        key not in self.properties.get(self.INPUT)):
+                    message = _('Unknown input %s') % key
                     raise exception.StackValidationFailed(
                         error=_('Signal data error'),
                         message=message)
+        if params is not None and not isinstance(params, dict):
+                message = (_('Params must be a map, find a '
+                             '%s') % type(params))
+                raise exception.StackValidationFailed(
+                    error=_('Signal data error'),
+                    message=message)
 
     def validate(self):
         super(Workflow, self).validate()
@@ -471,36 +514,24 @@ class Workflow(signal_responder.SignalResponder,
         self.resource_id_set(workflow[0].name)
 
     def handle_signal(self, details=None):
-        self._validate_signal_data(details)
+        inputs, params = self._get_inputs_and_params(details)
+        self._validate_signal_data(inputs, params)
 
-        result_input = {}
-        result_params = {}
-
-        if details is not None:
-            if details.get(self.INPUT) is not None:
-                # NOTE(prazumovsky): Signal can contains some data, interesting
-                # for workflow, e.g. inputs. So, if signal data contains input
-                # we update override inputs, other leaved defined in template.
-                for key, value in six.iteritems(
-                        self.properties.get(self.INPUT)):
-                    result_input.update(
-                        {key: details.get(
-                            self.SIGNAL_DATA_INPUT).get(key) or value})
-            if details.get(self.SIGNAL_DATA_PARAMS) is not None:
-                if self.properties.get(self.PARAMS) is not None:
-                    result_params.update(self.properties.get(self.PARAMS))
-                result_params.update(details.get(self.SIGNAL_DATA_PARAMS))
-
-        if not result_input and self.properties.get(self.INPUT):
-            result_input.update(self.properties.get(self.INPUT))
-        if not result_params and self.properties.get(self.PARAMS):
-            result_params.update(self.properties.get(self.PARAMS))
+        inputs_result = copy.deepcopy(self.properties[self.INPUT])
+        params_result = copy.deepcopy(self.properties[self.PARAMS]) or {}
+        # NOTE(prazumovsky): Signal can contains some data, interesting
+        # for workflow, e.g. inputs. So, if signal data contains input
+        # we update override inputs, other leaved defined in template.
+        if inputs:
+            inputs_result.update(inputs)
+        if params:
+            params_result.update(params)
 
         try:
             execution = self.client().executions.create(
                 self._workflow_name(),
-                jsonutils.dumps(result_input),
-                **result_params)
+                jsonutils.dumps(inputs_result),
+                **params_result)
         except Exception as ex:
             raise exception.ResourceFailure(ex, self)
         executions = [execution.id]
@@ -522,19 +553,17 @@ class Workflow(signal_responder.SignalResponder,
             self.data_set(self.NAME, workflow[0].name)
             self.resource_id_set(workflow[0].name)
 
-    def handle_delete(self):
-        super(Workflow, self).handle_delete()
-
-        if self.resource_id is None:
-            return
-
-        try:
-            self.client().workflows.delete(self.resource_id)
-            if self.data().get(self.EXECUTIONS):
-                for id in self.data().get(self.EXECUTIONS).split(','):
+    def _delete_executions(self):
+        if self.data().get(self.EXECUTIONS):
+            for id in self.data().get(self.EXECUTIONS).split(','):
+                with self.client_plugin().ignore_not_found:
                     self.client().executions.delete(id)
-        except Exception as e:
-            self.client_plugin().ignore_not_found(e)
+
+            self.data_delete('executions')
+
+    def handle_delete(self):
+        self._delete_executions()
+        return super(Workflow, self).handle_delete()
 
     def _resolve_attribute(self, name):
         if name == self.EXECUTIONS:
@@ -561,7 +590,7 @@ class Workflow(signal_responder.SignalResponder,
             return {self.NAME: self.resource_id,
                     self.INPUT: self.properties.get(self.INPUT)}
 
-        elif name == self.ALARM_URL:
+        elif name == self.ALARM_URL and self.resource_id is not None:
             return six.text_type(self._get_ec2_signed_url())
 
     # TODO(tlashchova): remove this method when mistralclient>1.0.0 is used.
