@@ -21,9 +21,19 @@ from heat.engine.resources.openstack.neutron import neutron
 from heat.engine.resources.openstack.neutron import port
 from heat.engine.resources.openstack.neutron import router
 from heat.engine import support
+from heat.engine import translation
 
 
 class FloatingIP(neutron.NeutronResource):
+    """A resource for managing Neutron floating ips.
+
+    Floating IP addresses can change their association between routers by
+    action of the user. One of the most common use cases for floating IPs is
+    to provide public IP addresses to a private cloud, where there are a
+    limited number of IP addresses available. Another is for a public cloud
+    user to have a "static" IP address that can be reassigned when an instance
+    is upgraded or moved.
+    """
     PROPERTIES = (
         FLOATING_NETWORK_ID, FLOATING_NETWORK, VALUE_SPECS,
         PORT_ID, FIXED_IP_ADDRESS, FLOATING_IP_ADDRESS,
@@ -131,11 +141,19 @@ class FloatingIP(neutron.NeutronResource):
 
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.FLOATING_NETWORK],
                 value_path=[self.FLOATING_NETWORK_ID]
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.FLOATING_NETWORK],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='network'
             )
         ]
 
@@ -165,7 +183,7 @@ class FloatingIP(neutron.NeutronResource):
                     if not fixed_ips:
                         p_net = (resource.properties.get(port.Port.NETWORK) or
                                  resource.properties.get(port.Port.NETWORK_ID))
-                        if p_net and p_net != 'None':
+                        if p_net:
                             subnets = self.client().show_network(p_net)[
                                 'network']['subnets']
                             return subnet in subnets
@@ -183,8 +201,8 @@ class FloatingIP(neutron.NeutronResource):
                     resource.properties.get(router.RouterInterface.SUBNET) or
                     resource.properties.get(router.RouterInterface.SUBNET_ID))
                 # during create we have only unresolved value for functions, so
-                # cat not use None value for building correct dependencies
-                if interface_subnet != 'None':
+                # can not use None value for building correct dependencies
+                if interface_subnet:
                     for d in deps.graph()[self]:
                         if port_on_subnet(d, interface_subnet):
                             deps += (self, resource)
@@ -214,8 +232,7 @@ class FloatingIP(neutron.NeutronResource):
         props = self.prepare_properties(
             self.properties,
             self.physical_resource_name())
-        self.client_plugin().resolve_network(props, self.FLOATING_NETWORK,
-                                             'floating_network_id')
+        props['floating_network_id'] = props.pop(self.FLOATING_NETWORK)
         fip = self.client().create_floatingip({
             'floatingip': props})['floatingip']
         self.resource_id_set(fip['id'])
@@ -224,10 +241,8 @@ class FloatingIP(neutron.NeutronResource):
         return self.client().show_floatingip(self.resource_id)['floatingip']
 
     def handle_delete(self):
-        try:
+        with self.client_plugin().ignore_not_found:
             self.client().delete_floatingip(self.resource_id)
-        except Exception as ex:
-            self.client_plugin().ignore_not_found(ex)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
@@ -247,6 +262,11 @@ class FloatingIP(neutron.NeutronResource):
 
 
 class FloatingIPAssociation(neutron.NeutronResource):
+    """A resource for associating floating ips and ports.
+
+    This resource allows associating a floating IP to a port with at least one
+    IP address to associate with this floating IP.
+    """
     PROPERTIES = (
         FLOATINGIP_ID, PORT_ID, FIXED_IP_ADDRESS,
     ) = (
@@ -308,9 +328,7 @@ class FloatingIPAssociation(neutron.NeutronResource):
 
     def handle_create(self):
         props = self.prepare_properties(self.properties, self.name)
-
         floatingip_id = props.pop(self.FLOATINGIP_ID)
-
         self.client().update_floatingip(floatingip_id, {
             'floatingip': props})
         self.resource_id_set(self.id)
@@ -319,12 +337,10 @@ class FloatingIPAssociation(neutron.NeutronResource):
         if not self.resource_id:
             return
 
-        try:
+        with self.client_plugin().ignore_not_found:
             self.client().update_floatingip(
                 self.properties[self.FLOATINGIP_ID],
                 {'floatingip': {'port_id': None}})
-        except Exception as ex:
-            self.client_plugin().ignore_not_found(ex)
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if prop_diff:
@@ -333,12 +349,10 @@ class FloatingIPAssociation(neutron.NeutronResource):
             # if the floatingip_id is changed, disassociate the port which
             # associated with the old floatingip_id
             if self.FLOATINGIP_ID in prop_diff:
-                try:
+                with self.client_plugin().ignore_not_found:
                     self.client().update_floatingip(
                         floatingip_id,
                         {'floatingip': {'port_id': None}})
-                except Exception as ex:
-                    self.client_plugin().ignore_not_found(ex)
 
             # associate the floatingip with the new port
             floatingip_id = (prop_diff.get(self.FLOATINGIP_ID) or
