@@ -22,12 +22,24 @@ from heat.engine import properties
 from heat.engine import resource
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine import support
+from heat.engine import translation
 
 
 class HealthMonitor(neutron.NeutronResource):
+    """A resource for managing health monitors for loadbalancers in Neutron.
+
+    A health monitor is used to determine whether or not back-end members of
+    the VIP's pool are usable for processing a request. A pool can have several
+    health monitors associated with it. There are different types of health
+    monitors supported by the OpenStack LBaaS service:
+
+      - PING: used to ping the members using ICMP.
+      - TCP: used to connect to the members using TCP.
+      - HTTP: used to send an HTTP request to the member.
+      - HTTPS: used to send a secure HTTP request to the member.
     """
-    A resource for managing health monitors for load balancers in Neutron.
-    """
+
+    required_service_extension = 'lbaas'
 
     PROPERTIES = (
         DELAY, TYPE, MAX_RETRIES, TIMEOUT, ADMIN_STATE_UP,
@@ -173,9 +185,16 @@ class HealthMonitor(neutron.NeutronResource):
 
 
 class Pool(neutron.NeutronResource):
+    """A resource for managing load balancer pools in Neutron.
+
+    A load balancing pool is a logical set of devices, such as web servers,
+    that you group together to receive and process traffic. The loadbalancing
+    function chooses a member of the pool according to the configured load
+    balancing method to handle the new requests or connections received on the
+    VIP address. There is only one pool for a VIP.
     """
-    A resource for managing load balancer pools in Neutron.
-    """
+
+    required_service_extension = 'lbaas'
 
     PROPERTIES = (
         PROTOCOL, SUBNET_ID, SUBNET, LB_METHOD, NAME, DESCRIPTION,
@@ -273,6 +292,9 @@ class Pool(neutron.NeutronResource):
             properties.Schema.STRING,
             _('LBaaS provider to implement this load balancer instance.'),
             support_status=support.SupportStatus(version='5.0.0'),
+            constraints=[
+                constraints.CustomConstraint('neutron.lb.provider')
+            ],
         ),
         VIP: properties.Schema(
             properties.Schema.MAP,
@@ -391,9 +413,9 @@ class Pool(neutron.NeutronResource):
 
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.SUBNET],
                 value_path=[self.SUBNET_ID]
             )
@@ -550,9 +572,12 @@ class Pool(neutron.NeutronResource):
 
 
 class PoolMember(neutron.NeutronResource):
+    """A resource to handle loadbalancer members.
+
+    A pool member represents the application running on backend server.
     """
-    A resource to handle load balancer members.
-    """
+
+    required_service_extension = 'lbaas'
 
     support_status = support.SupportStatus(version='2014.1')
 
@@ -675,9 +700,13 @@ class PoolMember(neutron.NeutronResource):
 
 
 class LoadBalancer(resource.Resource):
+    """A resource to link a neutron pool with servers.
+
+    A loadbalancer allows linking a neutron pool with specified servers to some
+    port.
     """
-    A resource to link a neutron pool with servers.
-    """
+
+    required_service_extension = 'lbaas'
 
     PROPERTIES = (
         POOL_ID, PROTOCOL_PORT, MEMBERS,
@@ -739,10 +768,8 @@ class LoadBalancer(resource.Resource):
             old_members = set(six.iterkeys(rd_members))
             for member in old_members - members:
                 member_id = rd_members[member]
-                try:
+                with self.client_plugin().ignore_not_found:
                     self.client().delete_member(member_id)
-                except Exception as ex:
-                    self.client_plugin().ignore_not_found(ex)
                 self.data_delete(member)
             pool = self.properties[self.POOL_ID]
             protocol_port = self.properties[self.PROTOCOL_PORT]
@@ -760,10 +787,8 @@ class LoadBalancer(resource.Resource):
         # FIXME(pshchelo): this deletes members in a tight loop,
         # so is prone to OverLimit bug similar to LP 1265937
         for member, member_id in self.data().items():
-            try:
+            with self.client_plugin().ignore_not_found:
                 self.client().delete_member(member_id)
-            except Exception as ex:
-                self.client_plugin().ignore_not_found(ex)
             self.data_delete(member)
 
 
