@@ -23,12 +23,14 @@ from heat.engine import constraints
 from heat.engine import properties
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine import support
+from heat.engine import translation
 
 
 class NetworkGateway(neutron.NeutronResource):
-    '''
-    A resource for the Network Gateway resource in Neutron Network Gateway.
-    '''
+    """Network Gateway resource in Neutron Network Gateway.
+
+    Resource for connecting internal networks with specified devices.
+    """
 
     support_status = support.SupportStatus(version='2014.1')
 
@@ -151,9 +153,9 @@ class NetworkGateway(neutron.NeutronResource):
 
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.CONNECTIONS, self.NETWORK],
                 value_name=self.NETWORK_ID
             )
@@ -164,9 +166,7 @@ class NetworkGateway(neutron.NeutronResource):
             self.resource_id)['network_gateway']
 
     def validate(self):
-        '''
-        Validate any of the provided params
-        '''
+        """Validate any of the provided params."""
         super(NetworkGateway, self).validate()
         connections = self.properties[self.CONNECTIONS]
 
@@ -210,7 +210,7 @@ class NetworkGateway(neutron.NeutronResource):
 
         connections = self.properties[self.CONNECTIONS]
         for connection in connections:
-            try:
+            with self.client_plugin().ignore_not_found:
                 self.client_plugin().resolve_network(
                     connection, self.NETWORK, 'network_id')
                 if self.NETWORK in six.iterkeys(connection):
@@ -218,8 +218,6 @@ class NetworkGateway(neutron.NeutronResource):
                 self.client().disconnect_network_gateway(
                     self.resource_id, connection
                 )
-            except Exception as ex:
-                self.client_plugin().ignore_not_found(ex)
 
         try:
             self.client().delete_network_gateway(self.resource_id)
@@ -229,24 +227,24 @@ class NetworkGateway(neutron.NeutronResource):
             return True
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        props = self.prepare_update_properties(json_snippet)
-        connections = props.pop(self.CONNECTIONS)
+        connections = None
+        if self.CONNECTIONS in prop_diff:
+            connections = prop_diff.pop(self.CONNECTIONS)
 
         if self.DEVICES in prop_diff:
             self.handle_delete()
-            self.properties.data.update(props)
+            self.properties.data.update(prop_diff)
             self.handle_create()
             return
-        else:
-            props.pop(self.DEVICES, None)
 
-        if self.NAME in prop_diff:
+        if prop_diff:
+            self.prepare_update_properties(prop_diff)
             self.client().update_network_gateway(
-                self.resource_id, {'network_gateway': props})
+                self.resource_id, {'network_gateway': prop_diff})
 
-        if self.CONNECTIONS in prop_diff:
+        if connections:
             for connection in self.properties[self.CONNECTIONS]:
-                try:
+                with self.client_plugin().ignore_not_found:
                     self.client_plugin().resolve_network(
                         connection, self.NETWORK, 'network_id')
                     if self.NETWORK in six.iterkeys(connection):
@@ -254,8 +252,6 @@ class NetworkGateway(neutron.NeutronResource):
                     self.client().disconnect_network_gateway(
                         self.resource_id, connection
                     )
-                except Exception as ex:
-                    self.client_plugin().ignore_not_found(ex)
             for connection in connections:
                 self.client_plugin().resolve_network(
                     connection, self.NETWORK, 'network_id')
