@@ -15,32 +15,46 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import six
 
-from heat.common import exception
 from heat.common.i18n import _
 from heat.common.i18n import _LW
 from heat.engine import attributes
 from heat.engine import constraints
 from heat.engine import properties
+from heat.engine import resource
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine.resources.openstack.neutron import subnet
 from heat.engine import support
+from heat.engine import translation
 
 LOG = logging.getLogger(__name__)
 
 
 class Port(neutron.NeutronResource):
+    """A resource for managing Neutron ports.
+
+    A port represents a virtual switch port on a logical network switch.
+    Virtual instances attach their interfaces into ports. The logical port also
+    defines the MAC address and the IP address(es) to be assigned to the
+    interfaces plugged into them. When IP addresses are associated to a port,
+    this also implies the port is associated with a subnet, as the IP address
+    was taken from the allocation pool for a specific subnet.
+    """
 
     PROPERTIES = (
-        NETWORK_ID, NETWORK, NAME, VALUE_SPECS,
-        ADMIN_STATE_UP, FIXED_IPS, MAC_ADDRESS,
-        DEVICE_ID, SECURITY_GROUPS, ALLOWED_ADDRESS_PAIRS,
-        DEVICE_OWNER, REPLACEMENT_POLICY, VNIC_TYPE,
+        NAME, NETWORK_ID, NETWORK, FIXED_IPS, SECURITY_GROUPS,
+        REPLACEMENT_POLICY, DEVICE_ID, DEVICE_OWNER
+    ) = (
+        'name', 'network_id', 'network', 'fixed_ips', 'security_groups',
+        'replacement_policy', 'device_id', 'device_owner'
+    )
+
+    EXTRA_PROPERTIES = (
+        VALUE_SPECS, ADMIN_STATE_UP, MAC_ADDRESS,
+        ALLOWED_ADDRESS_PAIRS, VNIC_TYPE, QOS_POLICY,
         PORT_SECURITY_ENABLED,
     ) = (
-        'network_id', 'network', 'name', 'value_specs',
-        'admin_state_up', 'fixed_ips', 'mac_address',
-        'device_id', 'security_groups', 'allowed_address_pairs',
-        'device_owner', 'replacement_policy', 'binding:vnic_type',
+        'value_specs', 'admin_state_up', 'mac_address',
+        'allowed_address_pairs', 'binding:vnic_type', 'qos_policy',
         'port_security_enabled',
     )
 
@@ -60,15 +74,20 @@ class Port(neutron.NeutronResource):
         ADMIN_STATE_UP_ATTR, DEVICE_ID_ATTR, DEVICE_OWNER_ATTR, FIXED_IPS_ATTR,
         MAC_ADDRESS_ATTR, NAME_ATTR, NETWORK_ID_ATTR, SECURITY_GROUPS_ATTR,
         STATUS, TENANT_ID, ALLOWED_ADDRESS_PAIRS_ATTR, SUBNETS_ATTR,
-        PORT_SECURITY_ENABLED_ATTR,
+        PORT_SECURITY_ENABLED_ATTR, QOS_POLICY_ATTR,
     ) = (
         'admin_state_up', 'device_id', 'device_owner', 'fixed_ips',
         'mac_address', 'name', 'network_id', 'security_groups',
         'status', 'tenant_id', 'allowed_address_pairs', 'subnets',
-        'port_security_enabled',
+        'port_security_enabled', 'qos_policy_id',
     )
 
     properties_schema = {
+        NAME: properties.Schema(
+            properties.Schema.STRING,
+            _('A symbolic name for this port.'),
+            update_allowed=True
+        ),
         NETWORK_ID: properties.Schema(
             properties.Schema.STRING,
             support_status=support.SupportStatus(
@@ -83,45 +102,36 @@ class Port(neutron.NeutronResource):
             constraints=[
                 constraints.CustomConstraint('neutron.network')
             ],
-            update_allowed=True,
         ),
 
         NETWORK: properties.Schema(
             properties.Schema.STRING,
             _('Network this port belongs to. If you plan to use current port '
               'to assign Floating IP, you should specify %(fixed_ips)s '
-              'with %(subnet)s.  Note if this changes to a different network '
-              'update, the port will be replaced') %
+              'with %(subnet)s. Note if this changes to a different network '
+              'update, the port will be replaced.') %
             {'fixed_ips': FIXED_IPS, 'subnet': FIXED_IP_SUBNET},
             support_status=support.SupportStatus(version='2014.2'),
             required=True,
             constraints=[
                 constraints.CustomConstraint('neutron.network')
             ],
-            update_allowed=True,
         ),
-
-        NAME: properties.Schema(
+        DEVICE_ID: properties.Schema(
             properties.Schema.STRING,
-            _('A symbolic name for this port.'),
+            _('Device ID of this port.'),
             update_allowed=True
         ),
-        VALUE_SPECS: properties.Schema(
-            properties.Schema.MAP,
-            _('Extra parameters to include in the "port" object in the '
-              'creation request.'),
-            default={}
-        ),
-        ADMIN_STATE_UP: properties.Schema(
-            properties.Schema.BOOLEAN,
-            _('The administrative state of this port.'),
-            default=True,
+        DEVICE_OWNER: properties.Schema(
+            properties.Schema.STRING,
+            _('Name of the network owning the port. '
+              'The value is typically network:floatingip '
+              'or network:router_interface or network:dhcp.'),
             update_allowed=True
         ),
         FIXED_IPS: properties.Schema(
             properties.Schema.LIST,
             _('Desired IPs for this port.'),
-            default=[],
             schema=properties.Schema(
                 properties.Schema.MAP,
                 schema={
@@ -160,6 +170,47 @@ class Port(neutron.NeutronResource):
             ),
             update_allowed=True
         ),
+        SECURITY_GROUPS: properties.Schema(
+            properties.Schema.LIST,
+            _('Security group IDs to associate with this port.'),
+            update_allowed=True
+        ),
+        REPLACEMENT_POLICY: properties.Schema(
+            properties.Schema.STRING,
+            _('Policy on how to respond to a stack-update for this resource. '
+              'REPLACE_ALWAYS will replace the port regardless of any '
+              'property changes. AUTO will update the existing port for any '
+              'changed update-allowed property.'),
+            default='AUTO',
+            constraints=[
+                constraints.AllowedValues(['REPLACE_ALWAYS', 'AUTO']),
+            ],
+            update_allowed=True,
+            support_status=support.SupportStatus(
+                status=support.DEPRECATED,
+                version='6.0.0',
+                message=_('Replacement policy used to work around flawed '
+                          'nova/neutron port interaction which has been '
+                          'fixed since Liberty.'),
+                previous_status=support.SupportStatus(version='2014.2'))
+        ),
+    }
+
+    # NOTE(prazumovsky): properties_schema has been separated because some
+    # properties used in server for creating internal port.
+    extra_properties_schema = {
+        VALUE_SPECS: properties.Schema(
+            properties.Schema.MAP,
+            _('Extra parameters to include in the request.'),
+            default={},
+            update_allowed=True
+        ),
+        ADMIN_STATE_UP: properties.Schema(
+            properties.Schema.BOOLEAN,
+            _('The administrative state of this port.'),
+            default=True,
+            update_allowed=True
+        ),
         MAC_ADDRESS: properties.Schema(
             properties.Schema.STRING,
             _('MAC address to give to this port.'),
@@ -167,16 +218,6 @@ class Port(neutron.NeutronResource):
                 constraints.CustomConstraint('mac_addr')
             ]
         ),
-        DEVICE_ID: properties.Schema(
-            properties.Schema.STRING,
-            _('Device ID of this port.'),
-            update_allowed=True
-        ),
-        SECURITY_GROUPS: properties.Schema(
-            properties.Schema.LIST,
-            _('Security group IDs to associate with this port.'),
-            update_allowed=True
-        ),
         ALLOWED_ADDRESS_PAIRS: properties.Schema(
             properties.Schema.LIST,
             _('Additional MAC/IP address pairs allowed to pass through the '
@@ -200,27 +241,7 @@ class Port(neutron.NeutronResource):
                         ]
                     ),
                 },
-            ),
-            update_allowed=True,
-        ),
-        DEVICE_OWNER: properties.Schema(
-            properties.Schema.STRING,
-            _('Name of the network owning the port. '
-              'The value is typically network:floatingip '
-              'or network:router_interface or network:dhcp'),
-            update_allowed=True
-        ),
-        REPLACEMENT_POLICY: properties.Schema(
-            properties.Schema.STRING,
-            _('Policy on how to respond to a stack-update for this resource. '
-              'REPLACE_ALWAYS will replace the port regardless of any '
-              'property changes. AUTO will update the existing port for any '
-              'changed update-allowed property.'),
-            default='AUTO',
-            constraints=[
-                constraints.AllowedValues(['REPLACE_ALWAYS', 'AUTO']),
-            ],
-            update_allowed=True
+            )
         ),
         VNIC_TYPE: properties.Schema(
             properties.Schema.STRING,
@@ -228,8 +249,8 @@ class Port(neutron.NeutronResource):
               'To support SR-IOV PCI passthrough networking, you can request '
               'that the neutron port to be realized as normal (virtual nic), '
               'direct (pci passthrough), or macvtap '
-              '(virtual interface with a tap-like software interface). Note'
-              ' that this only works for Neutron deployments that support '
+              '(virtual interface with a tap-like software interface). Note '
+              'that this only works for Neutron deployments that support '
               'the bindings extension.'),
             constraints=[
                 constraints.AllowedValues(['normal', 'direct', 'macvtap']),
@@ -245,8 +266,23 @@ class Port(neutron.NeutronResource):
             update_allowed=True,
             support_status=support.SupportStatus(version='5.0.0')
         ),
+        QOS_POLICY: properties.Schema(
+            properties.Schema.STRING,
+            _('The name or ID of QoS policy to attach to this port.'),
+            constraints=[
+                constraints.CustomConstraint('neutron.qos_policy')
+            ],
+            update_allowed=True,
+            support_status=support.SupportStatus(version='6.0.0')
+        ),
     }
 
+    # Need to update properties_schema with other properties before
+    # initialisation, because resource should contain all properties before
+    # creating. Also, documentation should correctly resolves resource
+    # properties schema.
+    properties_schema.update(extra_properties_schema)
+
     attributes_schema = {
         ADMIN_STATE_UP_ATTR: attributes.Schema(
             _("The administrative state of this port."),
@@ -302,25 +338,42 @@ class Port(neutron.NeutronResource):
             support_status=support.SupportStatus(version='5.0.0'),
             type=attributes.Schema.BOOLEAN
         ),
+        QOS_POLICY_ATTR: attributes.Schema(
+            _("The QoS policy ID attached to this port."),
+            type=attributes.Schema.STRING,
+            support_status=support.SupportStatus(version='6.0.0'),
+        ),
     }
 
-    # The network property can be updated, but only to switch between
-    # a name and ID for the same network, which is handled in _needs_update
-    update_exclude_properties = [NETWORK, NETWORK_ID]
-
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.NETWORK],
                 value_path=[self.NETWORK_ID]
             ),
-            properties.TranslationRule(
+            translation.TranslationRule(
                 props,
-                properties.TranslationRule.REPLACE,
+                translation.TranslationRule.REPLACE,
                 [self.FIXED_IPS, self.FIXED_IP_SUBNET],
                 value_name=self.FIXED_IP_SUBNET_ID
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.NETWORK],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='network'
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.FIXED_IPS, self.FIXED_IP_SUBNET],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='subnet'
             )
         ]
 
@@ -342,44 +395,63 @@ class Port(neutron.NeutronResource):
         props = self.prepare_properties(
             self.properties,
             self.physical_resource_name())
-        self.client_plugin().resolve_network(props, self.NETWORK, 'network_id')
+        props['network_id'] = props.pop(self.NETWORK)
         self._prepare_port_properties(props)
+        qos_policy = props.pop(self.QOS_POLICY, None)
+        if qos_policy:
+            props['qos_policy_id'] = self.client_plugin().get_qos_policy_id(
+                qos_policy)
 
         port = self.client().create_port({'port': props})['port']
         self.resource_id_set(port['id'])
 
     def _prepare_port_properties(self, props, prepare_for_update=False):
-        for fixed_ip in props.get(self.FIXED_IPS, []):
-            for key, value in list(fixed_ip.items()):
-                if value is None:
-                    fixed_ip.pop(key)
-            if fixed_ip.get(self.FIXED_IP_SUBNET):
-                self.client_plugin().resolve_subnet(
-                    fixed_ip, self.FIXED_IP_SUBNET, 'subnet_id')
+        if self.FIXED_IPS in props:
+            fixed_ips = props[self.FIXED_IPS]
+            if fixed_ips:
+                for fixed_ip in fixed_ips:
+                    for key, value in list(fixed_ip.items()):
+                        if value is None:
+                            fixed_ip.pop(key)
+                    if fixed_ip.get(self.FIXED_IP_SUBNET):
+                        self.client_plugin().resolve_subnet(
+                            fixed_ip, self.FIXED_IP_SUBNET, 'subnet_id')
+            else:
+                # Passing empty list would have created a port without
+                # fixed_ips during CREATE and released the existing
+                # fixed_ips during UPDATE (default neutron behaviour).
+                # However, for backward compatibility we will let neutron
+                # assign ip for CREATE and leave the assigned ips during
+                # UPDATE by not passing it. ref bug #1538473.
+                del props[self.FIXED_IPS]
         # delete empty MAC addresses so that Neutron validation code
         # wouldn't fail as it not accepts Nones
-        for pair in props.get(self.ALLOWED_ADDRESS_PAIRS, []):
-            if (self.ALLOWED_ADDRESS_PAIR_MAC_ADDRESS in pair and
-                    pair[self.ALLOWED_ADDRESS_PAIR_MAC_ADDRESS] is None):
-                del pair[self.ALLOWED_ADDRESS_PAIR_MAC_ADDRESS]
-
+        if self.ALLOWED_ADDRESS_PAIRS in props:
+            address_pairs = props[self.ALLOWED_ADDRESS_PAIRS]
+            if address_pairs:
+                for pair in address_pairs:
+                    if (self.ALLOWED_ADDRESS_PAIR_MAC_ADDRESS in pair
+                        and pair[
+                            self.ALLOWED_ADDRESS_PAIR_MAC_ADDRESS] is None):
+                        del pair[self.ALLOWED_ADDRESS_PAIR_MAC_ADDRESS]
+            else:
+                props[self.ALLOWED_ADDRESS_PAIRS] = []
         # if without 'security_groups', don't set the 'security_groups'
         # property when creating, neutron will create the port with the
         # 'default' securityGroup. If has the 'security_groups' and the
         # value is [], which means to create the port without securityGroup.
-        if props.get(self.SECURITY_GROUPS) is not None:
-            props[self.SECURITY_GROUPS] = self.client_plugin(
-            ).get_secgroup_uuids(props.get(self.SECURITY_GROUPS))
-        else:
-            # And the update should has the same behavior.
-            if prepare_for_update:
+        if self.SECURITY_GROUPS in props:
+            if props.get(self.SECURITY_GROUPS) is not None:
                 props[self.SECURITY_GROUPS] = self.client_plugin(
-                ).get_secgroup_uuids(['default'])
+                ).get_secgroup_uuids(props.get(self.SECURITY_GROUPS))
+            else:
+                # And the update should has the same behavior.
+                if prepare_for_update:
+                    props[self.SECURITY_GROUPS] = self.client_plugin(
+                    ).get_secgroup_uuids(['default'])
 
-        if not props[self.FIXED_IPS]:
-            del(props[self.FIXED_IPS])
-
-        del(props[self.REPLACEMENT_POLICY])
+        if self.REPLACEMENT_POLICY in props:
+            del(props[self.REPLACEMENT_POLICY])
 
     def _show_resource(self):
         return self.client().show_port(
@@ -408,38 +480,25 @@ class Port(neutron.NeutronResource):
                         subnets.append(self.client().show_subnet(
                             subnet_id)['subnet'])
             except Exception as ex:
-                LOG.warn(_LW("Failed to fetch resource attributes: %s"), ex)
+                LOG.warning(_LW("Failed to fetch resource attributes: %s"), ex)
                 return
             return subnets
         return super(Port, self)._resolve_attribute(name)
 
-    def _needs_update(self, after, before, after_props, before_props,
-                      prev_resource, check_init_complete=True):
-
-        if after_props.get(self.REPLACEMENT_POLICY) == 'REPLACE_ALWAYS':
-            raise exception.UpdateReplace(self.name)
-
-        if after_props != before_props:
-            # Switching between name and ID is OK, provided the value resolves
-            # to the same network.  If the network changes, port is replaced.
-            # Support NETWORK and NETWORK_ID, as switching between those is OK.
-            before_id = self.client_plugin().find_neutron_resource(
-                before_props, self.NETWORK, 'network')
-            after_id = self.client_plugin().find_neutron_resource(
-                after_props, self.NETWORK, 'network')
-            if before_id != after_id:
-                raise exception.UpdateReplace(self.name)
-
-        return super(Port, self)._needs_update(
-            after, before, after_props, before_props, prev_resource,
-            check_init_complete)
+    def needs_replace(self, after_props):
+        """Mandatory replace based on props."""
+        return after_props.get(self.REPLACEMENT_POLICY) == 'REPLACE_ALWAYS'
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        props = self.prepare_update_properties(json_snippet)
-
-        self._prepare_port_properties(props, prepare_for_update=True)
-        LOG.debug('updating port with %s' % props)
-        self.client().update_port(self.resource_id, {'port': props})
+        if prop_diff:
+            self.prepare_update_properties(prop_diff)
+            if self.QOS_POLICY in prop_diff:
+                qos_policy = prop_diff.pop(self.QOS_POLICY)
+                prop_diff['qos_policy_id'] = self.client_plugin(
+                    ).get_qos_policy_id(qos_policy) if qos_policy else None
+            self._prepare_port_properties(prop_diff, prepare_for_update=True)
+            LOG.debug('updating port with %s' % prop_diff)
+            self.client().update_port(self.resource_id, {'port': prop_diff})
 
     def check_update_complete(self, *args):
         attributes = self._show_resource()
@@ -456,16 +515,30 @@ class Port(neutron.NeutronResource):
         props = {'fixed_ips': []}
         self.client().update_port(self.resource_id, {'port': props})
 
-    def restore_after_rollback(self):
-        old_port = self.stack._backup_stack().resources.get(self.name)
-        fixed_ips = old_port.data().get('port_fip', [])
-        # restore fixed_ips for this port by setting fixed_ips to []
+    def restore_prev_rsrc(self, convergence=False):
+        # In case of convergence, during rollback, the previous rsrc is
+        # already selected and is being acted upon.
+        bakup_resources = self.stack._backup_stack().resources
+        prev_port = self if convergence else bakup_resources.get(self.name)
+        fixed_ips = prev_port.data().get('port_fip', [])
+
         props = {'fixed_ips': []}
-        old_props = {'fixed_ips': jsonutils.loads(fixed_ips)}
-        # remove ip from new port
-        self.client().update_port(self.resource_id, {'port': props})
-        # restore ip for old port
-        self.client().update_port(old_port.resource_id, {'port': old_props})
+        if convergence:
+            existing_port, rsrc_owning_stack, stack = resource.Resource.load(
+                prev_port.context, prev_port.replaced_by, True,
+                prev_port.stack.cache_data
+            )
+            existing_port_id = existing_port.resource_id
+        else:
+            existing_port_id = self.resource_id
+        if existing_port_id:
+            # reset fixed_ips to [] for new resource
+            self.client().update_port(existing_port_id, {'port': props})
+        if fixed_ips and prev_port.resource_id:
+            # restore ip for old port
+            prev_port_props = {'fixed_ips': jsonutils.loads(fixed_ips)}
+            self.client().update_port(prev_port.resource_id,
+                                      {'port': prev_port_props})
 
 
 def resource_mapping():
