@@ -22,16 +22,19 @@ import six
 
 from heat.common import exception
 from heat.common.i18n import _
+from heat.common.i18n import _LE
 from heat.engine import attributes
 from heat.engine.clients import progress
 from heat.engine import constraints
 from heat.engine import function
 from heat.engine import properties
+from heat.engine.resources.openstack.neutron import port as neutron_port
 from heat.engine.resources.openstack.neutron import subnet
 from heat.engine.resources.openstack.nova import server_network_mixin
 from heat.engine.resources import scheduler_hints as sh
 from heat.engine.resources import stack_user
 from heat.engine import support
+from heat.engine import translation
 from heat.rpc import api as rpc_api
 
 cfg.CONF.import_opt('default_software_config_transport', 'heat.common.config')
@@ -41,6 +44,11 @@ LOG = logging.getLogger(__name__)
 
 class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
              server_network_mixin.ServerNetworkMixin):
+    """A resource for managing Nova instances.
+
+    A Server resource manages the running virtual machine instance within an
+    OpenStack cloud.
+    """
 
     PROPERTIES = (
         NAME, IMAGE, BLOCK_DEVICE_MAPPING, BLOCK_DEVICE_MAPPING_V2,
@@ -48,14 +56,14 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         ADMIN_USER, AVAILABILITY_ZONE, SECURITY_GROUPS, NETWORKS,
         SCHEDULER_HINTS, METADATA, USER_DATA_FORMAT, USER_DATA,
         RESERVATION_ID, CONFIG_DRIVE, DISK_CONFIG, PERSONALITY,
-        ADMIN_PASS, SOFTWARE_CONFIG_TRANSPORT
+        ADMIN_PASS, SOFTWARE_CONFIG_TRANSPORT, USER_DATA_UPDATE_POLICY
     ) = (
         'name', 'image', 'block_device_mapping', 'block_device_mapping_v2',
         'flavor', 'flavor_update_policy', 'image_update_policy', 'key_name',
         'admin_user', 'availability_zone', 'security_groups', 'networks',
         'scheduler_hints', 'metadata', 'user_data_format', 'user_data',
         'reservation_id', 'config_drive', 'diskConfig', 'personality',
-        'admin_pass', 'software_config_transport'
+        'admin_pass', 'software_config_transport', 'user_data_update_policy'
     )
 
     _BLOCK_DEVICE_MAPPING_KEYS = (
@@ -96,10 +104,10 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
     _NETWORK_KEYS = (
         NETWORK_UUID, NETWORK_ID, NETWORK_FIXED_IP, NETWORK_PORT,
-        NETWORK_SUBNET
+        NETWORK_SUBNET, NETWORK_PORT_EXTRA, NETWORK_FLOATING_IP
     ) = (
         'uuid', 'network', 'fixed_ip', 'port',
-        'subnet'
+        'subnet', 'port_extra_properties', 'floating_ip'
     )
 
     _SOFTWARE_CONFIG_FORMATS = (
@@ -121,6 +129,10 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         'name', 'addresses', 'networks', 'first_address',
         'instance_name', 'accessIPv4', 'accessIPv6', 'console_urls',
     )
+
+    # valid image Status
+    IMAGE_STATUS_ACTIVE = 'active'
+
     properties_schema = {
         NAME: properties.Schema(
             properties.Schema.STRING,
@@ -220,8 +232,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                     ),
                     BLOCK_DEVICE_MAPPING_DEVICE_TYPE: properties.Schema(
                         properties.Schema.STRING,
-                        _('Device type: at the moment we can make distinction'
-                          ' only between disk and cdrom.'),
+                        _('Device type: at the moment we can make distinction '
+                          'only between disk and cdrom.'),
                         constraints=[
                             constraints.AllowedValues(['cdrom', 'disk']),
                         ],
@@ -276,7 +288,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         IMAGE_UPDATE_POLICY: properties.Schema(
             properties.Schema.STRING,
             _('Policy on how to apply an image-id update; either by '
-              'requesting a server rebuild or by replacing the entire server'),
+              'requesting a server rebuild or by replacing '
+              'the entire server.'),
             default='REBUILD',
             constraints=[
                 constraints.AllowedValues(['REBUILD', 'REPLACE',
@@ -364,6 +377,14 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                             constraints.CustomConstraint('neutron.port')
                         ]
                     ),
+                    NETWORK_PORT_EXTRA: properties.Schema(
+                        properties.Schema.MAP,
+                        _('Dict, which has expand properties for port. '
+                          'Used only if port property is not specified '
+                          'for creating port.'),
+                        schema=neutron_port.Port.extra_properties_schema,
+                        support_status=support.SupportStatus(version='6.0.0')
+                    ),
                     NETWORK_SUBNET: properties.Schema(
                         properties.Schema.STRING,
                         _('Subnet in which to allocate the IP address for '
@@ -371,6 +392,11 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                           'properties. If subnet is specified, network '
                           'property becomes optional.'),
                         support_status=support.SupportStatus(version='5.0.0')
+                    ),
+                    NETWORK_FLOATING_IP: properties.Schema(
+                        properties.Schema.STRING,
+                        _('ID of the floating IP to associate.'),
+                        support_status=support.SupportStatus(version='6.0.0')
                     )
                 },
             ),
@@ -384,7 +410,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         METADATA: properties.Schema(
             properties.Schema.MAP,
             _('Arbitrary key/value metadata to store for this server. Both '
-              'keys and values must be 255 characters or less.  Non-string '
+              'keys and values must be 255 characters or less. Non-string '
               'values will be serialized to JSON (and the serialized '
               'string must be 255 characters or less).'),
             update_allowed=True
@@ -420,10 +446,22 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                 constraints.AllowedValues(_SOFTWARE_CONFIG_TRANSPORTS),
             ]
         ),
+        USER_DATA_UPDATE_POLICY: properties.Schema(
+            properties.Schema.STRING,
+            _('Policy on how to apply a user_data update; either by '
+              'ignorning it or by replacing the entire server.'),
+            default='REPLACE',
+            constraints=[
+                constraints.AllowedValues(['REPLACE', 'IGNORE']),
+            ],
+            support_status=support.SupportStatus(version='6.0.0'),
+            update_allowed=True
+        ),
         USER_DATA: properties.Schema(
             properties.Schema.STRING,
             _('User data script to be executed by cloud-init.'),
-            default=''
+            default='',
+            update_allowed=True
         ),
         RESERVATION_ID: properties.Schema(
             properties.Schema.STRING,
@@ -473,7 +511,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
               '{"public": [ip1, ip2...], "private": [ip3, ip4], '
               '"public_uuid": [ip1, ip2...], "private_uuid": [ip3, ip4]}. '
               'Each network will have two keys in dict, they are network '
-              'name and network id. '),
+              'name and network id.'),
             type=attributes.Schema.MAP
         ),
         FIRST_ADDRESS: attributes.Schema(
@@ -529,11 +567,57 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
     entity = 'servers'
 
     def translation_rules(self, props):
-        return [properties.TranslationRule(
-            props,
-            properties.TranslationRule.REPLACE,
-            source_path=[self.NETWORKS, self.NETWORK_ID],
-            value_name=self.NETWORK_UUID)]
+        rules = [
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.REPLACE,
+                source_path=[self.NETWORKS, self.NETWORK_ID],
+                value_name=self.NETWORK_UUID),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                source_path=[self.FLAVOR],
+                client_plugin=self.client_plugin('nova'),
+                finder='find_flavor_by_name_or_id'),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                source_path=[self.IMAGE],
+                client_plugin=self.client_plugin('glance'),
+                finder='find_image_by_name_or_id'),
+        ]
+        if self.is_using_neutron():
+            rules.extend([
+                translation.TranslationRule(
+                    props,
+                    translation.TranslationRule.RESOLVE,
+                    source_path=[self.NETWORKS, self.NETWORK_ID],
+                    client_plugin=self.client_plugin('neutron'),
+                    finder='find_resourceid_by_name_or_id',
+                    entity='network'),
+                translation.TranslationRule(
+                    props,
+                    translation.TranslationRule.RESOLVE,
+                    source_path=[self.NETWORKS, self.NETWORK_SUBNET],
+                    client_plugin=self.client_plugin('neutron'),
+                    finder='find_resourceid_by_name_or_id',
+                    entity='subnet'),
+                translation.TranslationRule(
+                    props,
+                    translation.TranslationRule.RESOLVE,
+                    source_path=[self.NETWORKS, self.NETWORK_PORT],
+                    client_plugin=self.client_plugin('neutron'),
+                    finder='find_resourceid_by_name_or_id',
+                    entity='port')])
+        else:
+            rules.extend([
+                translation.TranslationRule(
+                    props,
+                    translation.TranslationRule.RESOLVE,
+                    source_path=[self.NETWORKS, self.NETWORK_ID],
+                    client_plugin=self.client_plugin('nova'),
+                    finder='get_nova_network_id')])
+        return rules
 
     def __init__(self, name, json_snippet, stack):
         super(Server, self).__init__(name, json_snippet, stack)
@@ -589,8 +673,10 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             collectors.append('zaqar')
 
         elif self.transport_poll_server_cfn(props):
+            heat_client_plugin = self.stack.clients.client_plugin('heat')
+            config_url = heat_client_plugin.get_cfn_metadata_server_url()
             occ.update({'cfn': {
-                'metadata_url': '%s/v1/' % cfg.CONF.heat_metadata_server_url,
+                'metadata_url': config_url,
                 'access_key_id': self.access_key,
                 'secret_access_key': self.secret_key,
                 'stack_name': self.stack.name,
@@ -635,9 +721,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                 container, object_name, jsonutils.dumps(meta))
 
     def _register_access_key(self):
-        '''
-        Access is limited to this resource, which created the keypair
-        '''
+        """Access is limited to this resource, which created the keypair."""
         def access_allowed(resource_name):
             return resource_name == self.name
 
@@ -733,15 +817,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             instance_user=None,
             user_data_format=user_data_format)
 
-        flavor = self.properties[self.FLAVOR]
         availability_zone = self.properties[self.AVAILABILITY_ZONE]
-
-        image = self.properties[self.IMAGE]
-        if image:
-            image = self.client_plugin('glance').get_image_id(image)
-
-        flavor_id = self.client_plugin().get_flavor_id(flavor)
-
         instance_meta = self.properties[self.METADATA]
         if instance_meta is not None:
             instance_meta = self.client_plugin().meta_serialize(
@@ -750,8 +826,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         scheduler_hints = self._scheduler_hints(
             self.properties[self.SCHEDULER_HINTS])
 
-        nics = self._build_nics(self.properties[self.NETWORKS],
-                                security_groups=security_groups)
+        nics = self._build_nics(self.properties[self.NETWORKS])
         block_device_mapping = self._build_block_device_mapping(
             self.properties[self.BLOCK_DEVICE_MAPPING])
         block_device_mapping_v2 = self._build_block_device_mapping_v2(
@@ -761,13 +836,15 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         admin_pass = self.properties[self.ADMIN_PASS] or None
         personality_files = self.properties[self.PERSONALITY]
         key_name = self.properties[self.KEY_NAME]
+        flavor = self.properties[self.FLAVOR]
+        image = self.properties[self.IMAGE]
 
         server = None
         try:
             server = self.client().servers.create(
                 name=self._server_name(),
                 image=image,
-                flavor=flavor_id,
+                flavor=flavor,
                 key_name=key_name,
                 security_groups=security_groups,
                 userdata=userdata,
@@ -794,8 +871,20 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         check = self.client_plugin()._check_active(server_id)
         if check:
             self.store_external_ports()
+            # Addresses binds to server not immediately, so we need to wait
+            # until server is created and after that associate floating ip.
+            self.floating_ips_nova_associate()
         return check
 
+    def floating_ips_nova_associate(self):
+        # If there is no neutron used, floating_ip still unassociated,
+        # so need associate it with nova.
+        if not self.is_using_neutron():
+            for net in self.properties.get(self.NETWORKS) or []:
+                if net.get(self.NETWORK_FLOATING_IP):
+                    self._floating_ip_nova_associate(
+                        net.get(self.NETWORK_FLOATING_IP))
+
     def handle_check(self):
         server = self.client().servers.get(self.resource_id)
         status = self.client_plugin().get_status(server)
@@ -921,7 +1010,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         for key in list(nets.keys()):
             try:
                 net_id = self.client_plugin().get_net_id_by_label(key)
-            except (exception.NovaNetworkNotFound,
+            except (exception.EntityNotFound,
                     exception.PhysicalResourceNameAmbiguity):
                 net_id = None
             if net_id:
@@ -978,10 +1067,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
     def _update_flavor(self, prop_diff):
         flavor = prop_diff[self.FLAVOR]
-        flavor_id = self.client_plugin().get_flavor_id(flavor)
-        handler_args = {'args': (flavor_id,)}
-        checker_args = {'args': (flavor_id, flavor)}
-
+        handler_args = checker_args = {'args': (flavor,)}
         prg_resize = progress.ServerUpdateProgress(self.resource_id,
                                                    'resize',
                                                    handler_extra=handler_args,
@@ -995,7 +1081,6 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             prop_diff.get(self.IMAGE_UPDATE_POLICY) or
             self.properties[self.IMAGE_UPDATE_POLICY])
         image = prop_diff[self.IMAGE]
-        image_id = self.client_plugin('glance').get_image_id(image)
         preserve_ephemeral = (
             image_update_policy == 'REBUILD_PRESERVE_EPHEMERAL')
         password = (prop_diff.get(self.ADMIN_PASS) or
@@ -1004,7 +1089,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                   'preserve_ephemeral': preserve_ephemeral}
         prg = progress.ServerUpdateProgress(self.resource_id,
                                             'rebuild',
-                                            handler_extra={'args': (image_id,),
+                                            handler_extra={'args': (image,),
                                                            'kwargs': kwargs})
         return prg
 
@@ -1012,13 +1097,12 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         updaters = []
         new_networks = prop_diff.get(self.NETWORKS)
         old_networks = self.properties[self.NETWORKS]
-        security_groups = self.properties[self.SECURITY_GROUPS]
 
         if not server:
             server = self.client().servers.get(self.resource_id)
         interfaces = server.interface_list()
         remove_ports, add_nets = self.calculate_networks(
-            old_networks, new_networks, interfaces, security_groups)
+            old_networks, new_networks, interfaces)
 
         for port in remove_ports:
             updaters.append(
@@ -1038,30 +1122,28 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
         return updaters
 
-    def _needs_update(self, after, before, after_props, before_props,
-                      prev_resource, check_init_complete=True):
-        result = super(Server, self)._needs_update(
-            after, before, after_props, before_props, prev_resource,
-            check_init_complete=check_init_complete)
-
-        prop_diff = self.update_template_diff_properties(after_props,
-                                                         before_props)
-
-        if self.FLAVOR in prop_diff:
+    def needs_replace_with_prop_diff(self, changed_properties_set,
+                                     after_props, before_props):
+        """Needs replace based on prop_diff."""
+        if self.FLAVOR in changed_properties_set:
             flavor_update_policy = (
-                prop_diff.get(self.FLAVOR_UPDATE_POLICY) or
-                self.properties[self.FLAVOR_UPDATE_POLICY])
+                after_props.get(self.FLAVOR_UPDATE_POLICY) or
+                before_props.get(self.FLAVOR_UPDATE_POLICY))
             if flavor_update_policy == 'REPLACE':
-                raise exception.UpdateReplace(self.name)
+                return True
 
-        if self.IMAGE in prop_diff:
+        if self.IMAGE in changed_properties_set:
             image_update_policy = (
-                prop_diff.get(self.IMAGE_UPDATE_POLICY) or
-                self.properties[self.IMAGE_UPDATE_POLICY])
+                after_props.get(self.IMAGE_UPDATE_POLICY) or
+                before_props.get(self.IMAGE_UPDATE_POLICY))
             if image_update_policy == 'REPLACE':
-                raise exception.UpdateReplace(self.name)
+                return True
 
-        return result
+        if self.USER_DATA in changed_properties_set:
+            ud_update_policy = (
+                after_props.get(self.USER_DATA_UPDATE_POLICY) or
+                before_props.get(self.USER_DATA_UPDATE_POLICY))
+            return ud_update_policy == 'REPLACE'
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         if 'Metadata' in tmpl_diff:
@@ -1083,7 +1165,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         server = None
 
         if self.METADATA in prop_diff:
-            server = self.client().servers.get(self.resource_id)
+            server = self.client_plugin().get_server(self.resource_id)
             self.client_plugin().meta_update(server,
                                              prop_diff[self.METADATA])
 
@@ -1094,12 +1176,12 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             updaters.append(self._update_image(prop_diff))
         elif self.ADMIN_PASS in prop_diff:
             if not server:
-                server = self.client().servers.get(self.resource_id)
+                server = self.client_plugin().get_server(self.resource_id)
             server.change_password(prop_diff[self.ADMIN_PASS])
 
         if self.NAME in prop_diff:
             if not server:
-                server = self.client().servers.get(self.resource_id)
+                server = self.client_plugin().get_server(self.resource_id)
             self.client_plugin().rename(server, prop_diff[self.NAME])
 
         if self.NETWORKS in prop_diff:
@@ -1137,11 +1219,11 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             # stack update. This is not desirable for a server. The old
             # transport will continue to work, and the new transport may work
             # despite exceptions in the above block.
-            LOG.error("Error while updating software config transport")
+            LOG.error(_LE('Error while updating software config transport'))
             LOG.exception(e)
 
     def check_update_complete(self, updaters):
-        '''Push all updaters to completion in list order.'''
+        """Push all updaters to completion in list order."""
         for prg in updaters:
             if not prg.called:
                 handler = getattr(self.client_plugin(), prg.handler)
@@ -1159,9 +1241,7 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         return status
 
     def metadata_update(self, new_metadata=None):
-        '''
-        Refresh the metadata if new_metadata is None
-        '''
+        """Refresh the metadata if new_metadata is None."""
         if new_metadata is None:
             # Re-resolve the template metadata and merge it with the
             # current resource metadata.  This is necessary because the
@@ -1175,10 +1255,10 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
     @staticmethod
     def _check_maximum(count, maximum, msg):
-        '''
-        Check a count against a maximum, unless maximum is -1 which indicates
-        that there is no limit
-        '''
+        """Check a count against a maximum.
+
+        Unless maximum is -1 which indicates that there is no limit.
+        """
         if maximum != -1 and count > maximum:
             raise exception.StackValidationFailed(message=msg)
 
@@ -1234,6 +1314,42 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
         return bootable_vol
 
+    def _validate_image_flavor(self, image, flavor):
+        try:
+            image_obj = self.client_plugin('glance').get_image(image)
+            flavor_obj = self.client_plugin().get_flavor(flavor)
+        except Exception as ex:
+            # Flavor or image may not have been created in the backend
+            # yet when they are part of the same stack/template.
+            if (self.client_plugin().is_not_found(ex) or
+                    self.client_plugin('glance').is_not_found(ex)):
+                return
+            raise
+        else:
+            if image_obj.status.lower() != self.IMAGE_STATUS_ACTIVE:
+                msg = _('Image status is required to be %(cstatus)s not '
+                        '%(wstatus)s.') % {
+                    'cstatus': self.IMAGE_STATUS_ACTIVE,
+                    'wstatus': image_obj.status}
+                raise exception.StackValidationFailed(message=msg)
+
+            # validate image/flavor combination
+            if flavor_obj.ram < image_obj.min_ram:
+                msg = _('Image %(image)s requires %(imram)s minimum ram. '
+                        'Flavor %(flavor)s has only %(flram)s.') % {
+                    'image': image, 'imram': image_obj.min_ram,
+                    'flavor': flavor, 'flram': flavor_obj.ram}
+                raise exception.StackValidationFailed(message=msg)
+
+            # validate image/flavor disk compatibility
+            if flavor_obj.disk < image_obj.min_disk:
+                msg = _('Image %(image)s requires %(imsz)s GB minimum '
+                        'disk space. Flavor %(flavor)s has only '
+                        '%(flsz)s GB.') % {
+                    'image': image, 'imsz': image_obj.min_disk,
+                    'flavor': flavor, 'flsz': flavor_obj.disk}
+                raise exception.StackValidationFailed(message=msg)
+
     def validate(self):
         """Validate any of the provided params."""
         super(Server, self).validate()
@@ -1253,6 +1369,10 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
                     ' instance %s') % self.name
             raise exception.StackValidationFailed(message=msg)
 
+        flavor = self.properties[self.FLAVOR]
+        if image:
+            self._validate_image_flavor(image, flavor)
+
         # network properties 'uuid' and 'network' shouldn't be used
         # both at once for all networks
         networks = self.properties[self.NETWORKS] or []
@@ -1308,15 +1428,13 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         object_name = self.data().get('metadata_object_name')
         if not object_name:
             return
-        try:
+        with self.client_plugin('swift').ignore_not_found:
             container = self.physical_resource_name()
             swift = self.client('swift')
             swift.delete_object(container, object_name)
             headers = swift.head_container(container)
             if int(headers['x-container-object-count']) == 0:
                 swift.delete_container(container)
-        except Exception as ex:
-            self.client_plugin('swift').ignore_not_found(ex)
 
     def _delete_queue(self):
         queue_id = self.data().get('metadata_queue_id')
@@ -1324,18 +1442,16 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             return
         client_plugin = self.client_plugin('zaqar')
         zaqar = client_plugin.create_for_tenant(
-            self.stack.stack_user_project_id)
-        try:
+            self.stack.stack_user_project_id, self._user_token())
+        with client_plugin.ignore_not_found:
             zaqar.queue(queue_id).delete()
-        except Exception as ex:
-            client_plugin.ignore_not_found(ex)
         self.data_delete('metadata_queue_id')
 
     def _delete(self):
         if self.user_data_software_config():
+            self._delete_queue()
             self._delete_user()
             self._delete_temp_url()
-            self._delete_queue()
 
         # remove internal and external ports
         self._delete_internal_ports()
@@ -1344,6 +1460,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
         if self.resource_id is None:
             return
 
+        self._floating_ips_disassociate()
+
         try:
             self.client().servers.delete(self.resource_id)
         except Exception as e:
@@ -1382,11 +1500,12 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             prg.server_id)
 
     def handle_suspend(self):
-        '''
-        Suspend a server - note we do not wait for the SUSPENDED state,
-        this is polled for by check_suspend_complete in a similar way to the
-        create logic so we can take advantage of coroutines
-        '''
+        """Suspend a server.
+
+        Note we do not wait for the SUSPENDED state, this is polled for by
+        check_suspend_complete in a similar way to the create logic so we can
+        take advantage of coroutines.
+        """
         if self.resource_id is None:
             raise exception.Error(_('Cannot suspend %s, resource_id not set') %
                                   self.name)
@@ -1424,11 +1543,12 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
             raise exc
 
     def handle_resume(self):
-        '''
-        Resume a server - note we do not wait for the ACTIVE state,
-        this is polled for by check_resume_complete in a similar way to the
-        create logic so we can take advantage of coroutines
-        '''
+        """Resume a server.
+
+        Note we do not wait for the ACTIVE state, this is polled for by
+        check_resume_complete in a similar way to the create logic so we can
+        take advantage of coroutines.
+        """
         if self.resource_id is None:
             raise exception.Error(_('Cannot resume %s, resource_id not set') %
                                   self.name)
@@ -1469,10 +1589,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
 
     def handle_delete_snapshot(self, snapshot):
         image_id = snapshot['resource_data'].get('snapshot_image_id')
-        try:
+        with self.client_plugin().ignore_not_found:
             self.client().images.delete(image_id)
-        except Exception as e:
-            self.client_plugin().ignore_not_found(e)
 
     def handle_restore(self, defn, restore_data):
         image_id = restore_data['resource_data']['snapshot_image_id']
@@ -1483,8 +1601,8 @@ class Server(stack_user.StackUser, sh.SchedulerHintsMixin,
     def prepare_for_replace(self):
         self.prepare_ports_for_replace()
 
-    def restore_after_rollback(self):
-        self.restore_ports_after_rollback()
+    def restore_prev_rsrc(self, convergence=False):
+        self.restore_ports_after_rollback(convergence=convergence)
 
 
 def resource_mapping():
