@@ -11,13 +11,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import hashlib
 import json
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
 from oslo_utils import excutils
+from oslo_utils import reflection
 import six
 
 from heat.common import exception
@@ -32,16 +31,19 @@ from heat.engine import resource
 from heat.engine import scheduler
 from heat.engine import stack as parser
 from heat.engine import template
+from heat.objects import stack as stack_object
+from heat.objects import stack_lock
 from heat.rpc import api as rpc_api
 
 LOG = logging.getLogger(__name__)
 
 
 class StackResource(resource.Resource):
-    '''
+    """Allows entire stack to be managed as a resource in a parent stack.
+
     An abstract Resource subclass that allows the management of an entire Stack
     as a resource in a parent stack.
-    '''
+    """
 
     # Assume True as this is evaluated before the stack is created
     # so there is no way to know for sure without subclass-specific
@@ -95,6 +97,11 @@ class StackResource(resource.Resource):
         if self.nested() is None and self.status == self.FAILED:
             raise exception.UpdateReplace(self)
 
+        # If stack resource is in CHECK_FAILED state, raise UpdateReplace
+        # to replace the failed stack.
+        if self.state == (self.CHECK, self.FAILED):
+            raise exception.UpdateReplace(self)
+
         if (check_init_complete and
                 self.nested() is None and
                 self.action == self.INIT and self.status == self.COMPLETE):
@@ -115,7 +122,7 @@ class StackResource(resource.Resource):
                     self.resource_id)
                 self.rpc_client().stack_cancel_update(
                     self.context,
-                    stack_identity,
+                    dict(stack_identity),
                     cancel_with_rollback=False)
 
     def has_nested(self):
@@ -124,49 +131,38 @@ class StackResource(resource.Resource):
 
         return False
 
-    def nested(self, force_reload=False, show_deleted=False):
-        '''Return a Stack object representing the nested (child) stack.
-        if we catch NotFound exception when loading, return None.
-
-        :param force_reload: Forces reloading from the DB instead of returning
-                             the locally cached Stack object
-        :param show_deleted: Returns the stack even if it's been deleted
-        '''
-        if force_reload:
-            self._nested = None
+    def nested(self):
+        """Return a Stack object representing the nested (child) stack.
 
+        If we catch NotFound exception when loading, return None.
+        """
         if self._nested is None and self.resource_id is not None:
             try:
                 self._nested = parser.Stack.load(self.context,
-                                                 self.resource_id,
-                                                 show_deleted=show_deleted,
-                                                 force_reload=force_reload)
+                                                 self.resource_id)
             except exception.NotFound:
                 return None
 
         return self._nested
 
     def child_template(self):
-        '''
-        Default implementation to get the child template.
+        """Default implementation to get the child template.
 
         Resources that inherit from StackResource should override this method
         with specific details about the template used by them.
-        '''
+        """
         raise NotImplementedError()
 
     def child_params(self):
-        '''
-        Default implementation to get the child params.
+        """Default implementation to get the child params.
 
         Resources that inherit from StackResource should override this method
         with specific details about the parameters used by them.
-        '''
+        """
         raise NotImplementedError()
 
     def preview(self):
-        '''
-        Preview a StackResource as resources within a Stack.
+        """Preview a StackResource as resources within a Stack.
 
         This method overrides the original Resource.preview to return a preview
         of all the resources contained in this Stack.  For this to be possible,
@@ -174,13 +170,13 @@ class StackResource(resource.Resource):
         ``child_params`` with specific information to allow the stack to be
         parsed correctly. If any of these methods is missing, the entire
         StackResource will be returned as if it were a regular Resource.
-        '''
+        """
         try:
             child_template = self.child_template()
             params = self.child_params()
         except NotImplementedError:
-            LOG.warn(_LW("Preview of '%s' not yet implemented"),
-                     self.__class__.__name__)
+            class_name = reflection.get_class_name(self, fully_qualified=False)
+            LOG.warning(_LW("Preview of '%s' not yet implemented"), class_name)
             return self
 
         name = "%s-%s" % (self.stack.name, self.name)
@@ -225,6 +221,7 @@ class StackResource(resource.Resource):
                               stack_user_project_id=stack_user_project_id,
                               adopt_stack_data=adopt_data,
                               nested_depth=new_nested_depth)
+        nested.set_parent_stack(self.stack)
         return nested
 
     def _child_nested_depth(self):
@@ -267,46 +264,58 @@ class StackResource(resource.Resource):
             timeout_mins = self.stack.timeout_mins
         stack_user_project_id = self.stack.stack_user_project_id
 
+        kwargs = self._stack_kwargs(user_params, child_template)
+
+        adopt_data_str = None
+        if adopt_data is not None:
+            if 'environment' not in adopt_data:
+                adopt_data['environment'] = kwargs['params']
+            if 'template' not in adopt_data:
+                if isinstance(child_template, template.Template):
+                    adopt_data['template'] = child_template.t
+                else:
+                    adopt_data['template'] = child_template
+            adopt_data_str = json.dumps(adopt_data)
+
+        args = {rpc_api.PARAM_TIMEOUT: timeout_mins,
+                rpc_api.PARAM_DISABLE_ROLLBACK: True,
+                rpc_api.PARAM_ADOPT_STACK_DATA: adopt_data_str}
+        kwargs.update({
+            'stack_name': name,
+            'args': args,
+            'environment_files': None,
+            'owner_id': self.stack.id,
+            'user_creds_id': self.stack.user_creds_id,
+            'stack_user_project_id': stack_user_project_id,
+            'nested_depth': self._child_nested_depth(),
+            'parent_resource_name': self.name
+        })
+        try:
+            result = self.rpc_client()._create_stack(self.context, **kwargs)
+        except Exception as ex:
+            self.raise_local_exception(ex)
+
+        self.resource_id_set(result['stack_id'])
+
+    def _stack_kwargs(self, user_params, child_template):
+
         if user_params is None:
             user_params = self.child_params()
+        if child_template is None:
+            child_template = self.child_template()
         child_env = environment.get_child_environment(
             self.stack.env,
             user_params,
             child_resource_name=self.name,
             item_to_remove=self.resource_info)
 
-        new_nested_depth = self._child_nested_depth()
         parsed_template = self._child_parsed_template(child_template,
                                                       child_env)
-
-        adopt_data_str = None
-        if adopt_data is not None:
-            if 'environment' not in adopt_data:
-                adopt_data['environment'] = child_env.user_env_as_dict()
-            if 'template' not in adopt_data:
-                adopt_data['template'] = child_template
-            adopt_data_str = json.dumps(adopt_data)
-
-        args = {rpc_api.PARAM_TIMEOUT: timeout_mins,
-                rpc_api.PARAM_DISABLE_ROLLBACK: True,
-                rpc_api.PARAM_ADOPT_STACK_DATA: adopt_data_str}
-        try:
-            result = self.rpc_client()._create_stack(
-                self.context,
-                name,
-                parsed_template.t,
-                child_env.user_env_as_dict(),
-                parsed_template.files,
-                args,
-                owner_id=self.stack.id,
-                user_creds_id=self.stack.user_creds_id,
-                stack_user_project_id=stack_user_project_id,
-                nested_depth=new_nested_depth,
-                parent_resource_name=self.name)
-        except Exception as ex:
-            self.raise_local_exception(ex)
-
-        self.resource_id_set(result['stack_id'])
+        return {
+            'template': parsed_template.t,
+            'params': child_env.user_env_as_dict(),
+            'files': parsed_template.files
+        }
 
     def raise_local_exception(self, ex):
         if (isinstance(ex, exception.ActionInProgress) and
@@ -316,7 +325,8 @@ class StackResource(resource.Resource):
             # finish
             return
 
-        if not ex.__class__.__name__.endswith('_Remote'):
+        class_name = reflection.get_class_name(ex, fully_qualified=False)
+        if not class_name.endswith('_Remote'):
             raise ex
 
         full_message = six.text_type(ex)
@@ -328,19 +338,23 @@ class StackResource(resource.Resource):
         raise exception.ResourceFailure(message, self, action=self.action)
 
     def check_create_complete(self, cookie=None):
-        return self._check_status_complete(resource.Resource.CREATE)
+        return self._check_status_complete(self.CREATE)
 
-    def _check_status_complete(self, action, show_deleted=False,
-                               cookie=None):
-        nested = self.nested(force_reload=True, show_deleted=show_deleted)
-        if nested is None:
-            if action == resource.Resource.DELETE:
+    def _check_status_complete(self, expected_action, cookie=None):
+
+        try:
+            data = stack_object.Stack.get_status(self.context,
+                                                 self.resource_id)
+        except exception.NotFound:
+            if expected_action == self.DELETE:
                 return True
             # It's possible the engine handling the create hasn't persisted
             # the stack to the DB when we first start polling for state
             return False
 
-        if nested.action != action:
+        action, status, status_reason, updated_time = data
+
+        if action != expected_action:
             return False
 
         # Has the action really started?
@@ -359,25 +373,29 @@ class StackResource(resource.Resource):
         if cookie is not None:
             prev_state = cookie['previous']['state']
             prev_updated_at = cookie['previous']['updated_at']
-            if (prev_updated_at == nested.updated_time and
-                    prev_state == nested.state):
+            if (prev_updated_at == updated_time and
+                    prev_state == (action, status)):
                 return False
 
-        if nested.status == resource.Resource.IN_PROGRESS:
+        if status == self.IN_PROGRESS:
             return False
-        elif nested.status == resource.Resource.COMPLETE:
-            return True
-        elif nested.status == resource.Resource.FAILED:
-            raise exception.ResourceFailure(nested.status_reason, self,
+        elif status == self.COMPLETE:
+            ret = stack_lock.StackLock.get_engine_id(self.resource_id) is None
+            if ret:
+                # Reset nested, to indicate we changed status
+                self._nested = None
+            return ret
+        elif status == self.FAILED:
+            raise exception.ResourceFailure(status_reason, self,
                                             action=action)
         else:
             raise exception.ResourceUnknownStatus(
-                resource_status=nested.status,
-                status_reason=nested.status_reason,
+                resource_status=status,
+                status_reason=status_reason,
                 result=_('Stack unknown status'))
 
     def check_adopt_complete(self, cookie=None):
-        return self._check_status_complete(resource.Resource.ADOPT)
+        return self._check_status_complete(self.ADOPT)
 
     def update_with_template(self, child_template, user_params=None,
                              timeout_mins=None):
@@ -390,79 +408,58 @@ class StackResource(resource.Resource):
             # if the create failed for some reason and the nested
             # stack was not created, we need to create an empty stack
             # here so that the update will work.
-            def _check_for_completion(creator_fn):
-                while not self.check_create_complete(creator_fn):
+            def _check_for_completion():
+                while not self.check_create_complete():
                     yield
 
             empty_temp = template_format.parse(
                 "heat_template_version: '2013-05-23'")
-            stack_creator = self.create_with_template(empty_temp, {})
-            checker = scheduler.TaskRunner(_check_for_completion,
-                                           stack_creator)
+            self.create_with_template(empty_temp, {})
+            checker = scheduler.TaskRunner(_check_for_completion)
             checker(timeout=self.stack.timeout_secs())
-
-            if stack_creator is not None:
-                stack_creator.run_to_completion()
             nested_stack = self.nested()
 
         if timeout_mins is None:
             timeout_mins = self.stack.timeout_mins
 
-        if user_params is None:
-            user_params = self.child_params()
-
-        child_env = environment.get_child_environment(
-            self.stack.env,
-            user_params,
-            child_resource_name=self.name,
-            item_to_remove=self.resource_info)
-        parsed_template = self._child_parsed_template(child_template,
-                                                      child_env)
-
+        kwargs = self._stack_kwargs(user_params, child_template)
         cookie = {'previous': {
             'updated_at': nested_stack.updated_time,
             'state': nested_stack.state}}
 
-        args = {rpc_api.PARAM_TIMEOUT: timeout_mins}
+        kwargs.update({
+            'stack_identity': dict(nested_stack.identifier()),
+            'args': {rpc_api.PARAM_TIMEOUT: timeout_mins}
+        })
         try:
-            self.rpc_client().update_stack(
-                self.context,
-                nested_stack.identifier(),
-                parsed_template.t,
-                child_env.user_env_as_dict(),
-                parsed_template.files,
-                args)
+            self.rpc_client().update_stack(self.context, **kwargs)
         except Exception as ex:
             LOG.exception(_LE('update_stack'))
             self.raise_local_exception(ex)
         return cookie
 
     def check_update_complete(self, cookie=None):
-        return self._check_status_complete(resource.Resource.UPDATE,
+        return self._check_status_complete(self.UPDATE,
                                            cookie=cookie)
 
     def delete_nested(self):
-        '''
-        Delete the nested stack.
-        '''
+        """Delete the nested stack."""
         stack = self.nested()
         if stack is None:
             return
 
-        stack_identity = stack.identifier()
+        stack_identity = dict(stack.identifier())
 
         try:
-            self.rpc_client().delete_stack(self.context, stack_identity,
-                                           cast=False)
+            self.rpc_client().delete_stack(self.context, stack_identity)
         except Exception as ex:
-            self.rpc_client().ignore_error_named(ex, 'StackNotFound')
+            self.rpc_client().ignore_error_named(ex, 'NotFound')
 
     def handle_delete(self):
         return self.delete_nested()
 
     def check_delete_complete(self, cookie=None):
-        return self._check_status_complete(resource.Resource.DELETE,
-                                           show_deleted=True)
+        return self._check_status_complete(self.DELETE)
 
     def handle_suspend(self):
         stack = self.nested()
@@ -473,10 +470,10 @@ class StackResource(resource.Resource):
             self.context.tenant_id,
             self.physical_resource_name(),
             self.resource_id)
-        self.rpc_client().stack_suspend(self.context, stack_identity)
+        self.rpc_client().stack_suspend(self.context, dict(stack_identity))
 
     def check_suspend_complete(self, cookie=None):
-        return self._check_status_complete(resource.Resource.SUSPEND)
+        return self._check_status_complete(self.SUSPEND)
 
     def handle_resume(self):
         stack = self.nested()
@@ -487,10 +484,10 @@ class StackResource(resource.Resource):
             self.context.tenant_id,
             self.physical_resource_name(),
             self.resource_id)
-        self.rpc_client().stack_resume(self.context, stack_identity)
+        self.rpc_client().stack_resume(self.context, dict(stack_identity))
 
     def check_resume_complete(self, cookie=None):
-        return self._check_status_complete(resource.Resource.RESUME)
+        return self._check_status_complete(self.RESUME)
 
     def handle_check(self):
         stack = self.nested()
@@ -502,12 +499,13 @@ class StackResource(resource.Resource):
             self.context.tenant_id,
             self.physical_resource_name(),
             self.resource_id)
-        self.rpc_client().stack_check(self.context, stack_identity)
+        self.rpc_client().stack_check(self.context, dict(stack_identity))
 
     def check_check_complete(self, cookie=None):
-        return self._check_status_complete(resource.Resource.CHECK)
+        return self._check_status_complete(self.CHECK)
 
     def prepare_abandon(self):
+        self.abandon_in_progress = True
         nested_stack = self.nested()
         if nested_stack:
             return self.nested().prepare_abandon()
@@ -515,12 +513,11 @@ class StackResource(resource.Resource):
         return {}
 
     def get_output(self, op):
-        '''
-        Return the specified Output value from the nested stack.
+        """Return the specified Output value from the nested stack.
 
         If the output key does not exist, raise an InvalidTemplateAttribute
         exception.
-        '''
+        """
         stack = self.nested()
         if stack is None:
             return None
@@ -535,12 +532,3 @@ class StackResource(resource.Resource):
 
     def _resolve_attribute(self, name):
         return self.get_output(name)
-
-    def implementation_signature(self):
-        schema_names = ([prop for prop in self.properties_schema] +
-                        [at for at in self.attributes_schema])
-        schema_hash = hashlib.sha256(';'.join(schema_names))
-        definition = {'template': self.child_template(),
-                      'files': self.stack.t.files}
-        definition_hash = hashlib.sha256(jsonutils.dumps(definition))
-        return (schema_hash.hexdigest(), definition_hash.hexdigest())
