@@ -72,19 +72,12 @@ class StackUser(resource.Resource):
         user_id = self.data().get('user_id')
         if user_id:
             return user_id
-        else:
-            # FIXME(shardy): This is a legacy hack for backwards compatibility
-            # remove after an appropriate transitional period...
-            # Assume this is a resource that was created with
-            # a previous version of heat and that the resource_id
-            # is the user_id
-            if self.resource_id:
-                self.data_set('user_id', self.resource_id)
-                return self.resource_id
 
     def handle_delete(self):
         self._delete_user()
 
+        return super(StackUser, self).handle_delete()
+
     def _delete_user(self):
         user_id = self._get_user_id()
         if user_id is None:
@@ -103,7 +96,7 @@ class StackUser(resource.Resource):
             # compatibility with resources created before the migration
             # to stack_user.StackUser domain users.  After an appropriate
             # transitional period, this should be removed.
-            LOG.warn(_LW('Reverting to legacy user delete path'))
+            LOG.warning(_LW('Reverting to legacy user delete path'))
             try:
                 self.keystone().delete_stack_user(user_id)
             except kc_exception.NotFound:
