@@ -41,13 +41,12 @@ def generate_class_from_template(name, data, env):
 
 
 class TemplateResource(stack_resource.StackResource):
-    '''
-    A resource implemented by a nested stack.
+    """A resource implemented by a nested stack.
 
     This implementation passes resource properties as parameters to the nested
     stack. Outputs of the nested stack are exposed as attributes of this
     resource.
-    '''
+    """
 
     def __init__(self, name, json_snippet, stack):
         self._parsed_nested = None
@@ -65,18 +64,13 @@ class TemplateResource(stack_resource.StackResource):
         if self.validation_exception is None:
             self._generate_schema(self.t)
 
-    def resource_class(self):
-        # All TemplateResource subclasses can be converted to each other with
-        # a stack update, so allow them to cross-update in place.
-        return TemplateResource
-
     def _get_resource_info(self, rsrc_defn):
         try:
             tri = self.stack.env.get_resource_info(
                 rsrc_defn.resource_type,
                 resource_name=rsrc_defn.name,
                 registry_type=environment.TemplateResourceInfo)
-        except exception.ResourceTypeNotFound:
+        except exception.EntityNotFound:
             self.validation_exception = ValueError(_(
                 'Only Templates with an extension of .yaml or '
                 '.template are supported'))
@@ -129,9 +123,10 @@ class TemplateResource(stack_resource.StackResource):
                                                 self._resolve_all_attributes)
 
     def child_params(self):
-        '''
+        """Override method of child_params for the resource.
+
         :return: parameter values for our nested stack based on our properties
-        '''
+        """
         params = {}
         for pname, pval in iter(self.properties.props.items()):
             if not pval.implemented():
@@ -183,10 +178,6 @@ class TemplateResource(stack_resource.StackResource):
         self._get_resource_info(definition)
         self._generate_schema(definition)
 
-    def implementation_signature(self):
-        self._generate_schema(self.t)
-        return super(TemplateResource, self).implementation_signature()
-
     def template_data(self):
         # we want to have the latest possible template.
         # 1. look in files
@@ -274,7 +265,7 @@ class TemplateResource(stack_resource.StackResource):
                 self.type(),
                 resource_name=self.name,
                 ignore=self.resource_info)
-        except exception.ResourceTypeNotFound:
+        except exception.EntityNotFound:
             pass
         else:
             facade_cls = fri.get_class(files=self.stack.t.files)
@@ -292,9 +283,7 @@ class TemplateResource(stack_resource.StackResource):
                                          self.child_params())
 
     def metadata_update(self, new_metadata=None):
-        '''
-        Refresh the metadata if new_metadata is None
-        '''
+        """Refresh the metadata if new_metadata is None."""
         if new_metadata is None:
             self.metadata_set(self.t.metadata())
 
@@ -311,7 +300,7 @@ class TemplateResource(stack_resource.StackResource):
 
         return self.nested().identifier().arn()
 
-    def FnGetAtt(self, key, *path):
+    def get_attribute(self, key, *path):
         stack = self.nested()
         if stack is None:
             return None
