@@ -88,8 +88,9 @@ class StackWatch(object):
             wrs = watch_rule_object.WatchRule.get_all_by_stack(admin_context,
                                                                sid)
         except Exception as ex:
-            LOG.warn(_LW('periodic_task db error watch rule removed? %(ex)s'),
-                     ex)
+            LOG.warning(_LW('periodic_task db error watch rule'
+                            ' removed? %(ex)s'),
+                        ex)
             return
 
         def run_alarm_action(stk, actions, details):
@@ -106,7 +107,7 @@ class StackWatch(object):
                                             actions, rule.get_details())
 
     def periodic_watcher_task(self, sid):
-        """Triggers watch-rule evaluation for all rules defined for stack ID.
+        """Evaluate all watch-rules defined for stack ID.
 
         Periodic task, created for each stack, triggers watch-rule evaluation
         for all rules defined for the stack sid = stack ID.
