@@ -14,6 +14,7 @@
 import collections
 import copy
 import datetime
+import functools
 import itertools
 import re
 
@@ -21,6 +22,7 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import excutils
+from oslo_utils import timeutils as oslo_timeutils
 from oslo_utils import uuidutils
 from osprofiler import profiler
 import six
@@ -118,9 +120,10 @@ class Stack(collections.Mapping):
                  use_stored_context=False, username=None,
                  nested_depth=0, strict_validate=True, convergence=False,
                  current_traversal=None, tags=None, prev_raw_template_id=None,
-                 current_deps=None, cache_data=None, resource_validate=True):
+                 current_deps=None, cache_data=None, resource_validate=True,
+                 service_check_defer=False):
 
-        """Initialisation of stack.
+        """Initialise the Stack.
 
         Initialise from a context, name, Template object and (optionally)
         Environment object. The database ID may also be initialised, if the
@@ -132,10 +135,15 @@ class Stack(collections.Mapping):
         """
 
         def _validate_stack_name(name):
-            if not re.match("[a-zA-Z][a-zA-Z0-9_.-]*$", name):
-                message = _('Invalid stack name %s must contain '
-                            'only alphanumeric or \"_-.\" characters, '
-                            'must start with alpha') % name
+            try:
+                if not re.match("[a-zA-Z][a-zA-Z0-9_.-]{0,254}$", name):
+                    message = _('Invalid stack name %s must contain '
+                                'only alphanumeric or \"_-.\" characters, '
+                                'must start with alpha and must be 255 '
+                                'characters or less.') % name
+                    raise exception.StackValidationFailed(message=message)
+            except TypeError:
+                message = _('Invalid stack name %s, must be a string') % name
                 raise exception.StackValidationFailed(message=message)
 
         if owner_id is None:
@@ -172,6 +180,7 @@ class Stack(collections.Mapping):
         self.cache_data = cache_data
         self._worker_client = None
         self._convg_deps = None
+        self.thread_group_mgr = None
 
         # strict_validate can be used to disable value validation
         # in the resource properties schema, this is useful when
@@ -186,6 +195,12 @@ class Stack(collections.Mapping):
         # commonly done in plugin validate() methods
         self.resource_validate = resource_validate
 
+        # service_check_defer can be used to defer the validation of service
+        # availability for a given resource, which helps to create the resource
+        # dependency tree completely when respective service is not available,
+        # especially during template_validate
+        self.service_check_defer = service_check_defer
+
         if use_stored_context:
             self.context = self.stored_context()
             self.context.roles = self.context.clients.client(
@@ -246,6 +261,9 @@ class Stack(collections.Mapping):
 
         return self._parent_stack[self.parent_resource_name]
 
+    def set_parent_stack(self, parent_stack):
+        self._parent_stack = parent_stack
+
     def stored_context(self):
         if self.user_creds_id:
             creds_obj = ucreds_object.UserCreds.get_by_id(self.user_creds_id)
@@ -264,21 +282,40 @@ class Stack(collections.Mapping):
 
     @property
     def resources(self):
+        return self._find_resources()
+
+    def _find_resources(self, filters=None):
         if self._resources is None:
-            self._resources = dict((name, resource.Resource(name, data, self))
-                                   for (name, data) in
-                                   self.t.resource_definitions(self).items())
+            res_defns = self.t.resource_definitions(self)
+
+            if not filters:
+                self._resources = dict((name,
+                                        resource.Resource(name, data, self))
+                                       for (name, data) in res_defns.items())
+            else:
+                self._resources = dict()
+                self._db_resources = dict()
+                for rsc in six.itervalues(
+                        resource_objects.Resource.get_all_by_stack(
+                            self.context, self.id, True, filters)):
+                    self._db_resources[rsc.name] = rsc
+                    res = resource.Resource(rsc.name,
+                                            res_defns[rsc.name],
+                                            self)
+                    self._resources[rsc.name] = res
+
             # There is no need to continue storing the db resources
             # after resource creation
             self._db_resources = None
+
         return self._resources
 
-    def iter_resources(self, nested_depth=0):
+    def iter_resources(self, nested_depth=0, filters=None):
         """Iterates over all the resources in a stack.
 
         Iterating includes nested stacks up to `nested_depth` levels below.
         """
-        for res in six.itervalues(self):
+        for res in six.itervalues(self._find_resources(filters)):
             yield res
 
             if not res.has_nested() or nested_depth == 0:
@@ -359,18 +396,18 @@ class Stack(collections.Mapping):
     def _set_param_stackid(self):
         """Update self.parameters with the current ARN.
 
-        self.parameters is then provided via the Parameters class as
-        the StackId pseudo parameter.
+        The ARN is then provided via the Parameters class as the StackId pseudo
+        parameter.
         """
         if not self.parameters.set_stack_id(self.identifier()):
-            LOG.warn(_LW("Unable to set parameters StackId identifier"))
+            LOG.warning(_LW("Unable to set parameters StackId identifier"))
 
     @staticmethod
     def get_dep_attrs(resources, outputs, resource_name):
-        """Return the set of dependent attributes for specified resource name.
+        """Return the attributes of the specified resource that are referenced.
 
-        Return the set of dependent attributes for specified resource name by
-        inspecting all resources and outputs in template.
+        Return an iterator over any attributes of the specified resource that
+        are referenced.
         """
         attr_lists = itertools.chain((res.dep_attrs(resource_name)
                                       for res in resources),
@@ -401,7 +438,8 @@ class Stack(collections.Mapping):
 
     @classmethod
     def load(cls, context, stack_id=None, stack=None, show_deleted=True,
-             use_stored_context=False, force_reload=False, cache_data=None):
+             use_stored_context=False, force_reload=False, cache_data=None,
+             resolve_data=True):
         """Retrieve a Stack from the database."""
         if stack is None:
             stack = stack_object.Stack.get_by_id(
@@ -418,7 +456,7 @@ class Stack(collections.Mapping):
 
         return cls._from_db(context, stack,
                             use_stored_context=use_stored_context,
-                            cache_data=cache_data)
+                            cache_data=cache_data, resolve_data=resolve_data)
 
     @classmethod
     def load_all(cls, context, limit=None, marker=None, sort_keys=None,
@@ -442,7 +480,12 @@ class Stack(collections.Mapping):
             not_tags,
             not_tags_any) or []
         for stack in stacks:
-            yield cls._from_db(context, stack, resolve_data=resolve_data)
+            try:
+                yield cls._from_db(context, stack, resolve_data=resolve_data)
+            except exception.NotFound:
+                # We're in a different transaction than the get_all, so a stack
+                # returned above can be deleted by the time we try to load it.
+                pass
 
     @classmethod
     def _from_db(cls, context, stack, resolve_data=True,
@@ -516,13 +559,13 @@ class Stack(collections.Mapping):
         return stack
 
     @profiler.trace('Stack.store', hide_args=False)
-    def store(self, backup=False):
+    def store(self, backup=False, exp_trvsl=None):
         """Store the stack in the database and return its ID.
 
         If self.id is set, we update the existing stack.
         """
         s = self.get_kwargs_for_cloning(keep_status=True, only_db=True)
-        s['name'] = self._backup_name() if backup else self.name
+        s['name'] = self.name
         s['backup'] = backup
         s['updated_at'] = self.updated_time
         if self.t.id is None:
@@ -532,7 +575,19 @@ class Stack(collections.Mapping):
             s['raw_template_id'] = self.t.id
 
         if self.id:
-            stack_object.Stack.update_by_id(self.context, self.id, s)
+            if exp_trvsl is None:
+                exp_trvsl = self.current_traversal
+
+            if self.convergence:
+                # do things differently for convergence
+                updated = stack_object.Stack.select_and_update(
+                    self.context, self.id, s, exp_trvsl=exp_trvsl)
+
+                if not updated:
+                    return None
+            else:
+                stack_object.Stack.update_by_id(self.context, self.id, s)
+
         else:
             if not self.user_creds_id:
                 # Create a context containing a trust_id and trustor_user_id
@@ -546,6 +601,11 @@ class Stack(collections.Mapping):
                 s['user_creds_id'] = new_creds.id
                 self.user_creds_id = new_creds.id
 
+            if self.convergence:
+                    # create a traversal ID
+                    self.current_traversal = uuidutils.generate_uuid()
+                    s['current_traversal'] = self.current_traversal
+
             new_s = stack_object.Stack.create(self.context, s)
             self.id = new_s.id
             self.created_time = new_s.created_at
@@ -623,18 +683,19 @@ class Stack(collections.Mapping):
                   not found.
         """
         for r in six.itervalues(self):
-            if r.state in (
+            if (r.state in (
                     (r.INIT, r.COMPLETE),
                     (r.CREATE, r.IN_PROGRESS),
                     (r.CREATE, r.COMPLETE),
                     (r.RESUME, r.IN_PROGRESS),
                     (r.RESUME, r.COMPLETE),
                     (r.UPDATE, r.IN_PROGRESS),
-                    (r.UPDATE, r.COMPLETE)) and r.FnGetRefId() == refid:
+                    (r.UPDATE, r.COMPLETE)) and
+                    (r.FnGetRefId() == refid or r.name == refid)):
                 return r
 
     def register_access_allowed_handler(self, credential_id, handler):
-        """Register a specific function.
+        """Register an authorization handler function.
 
         Register a function which determines whether the credentials with a
         given ID can have access to a named resource.
@@ -654,7 +715,7 @@ class Stack(collections.Mapping):
         return handler and handler(resource_name)
 
     @profiler.trace('Stack.validate', hide_args=False)
-    def validate(self):
+    def validate(self, ignorable_errors=None, validate_by_deps=True):
         """Validates the stack."""
         # TODO(sdake) Should return line number of invalid reference
 
@@ -681,7 +742,12 @@ class Stack(collections.Mapping):
             raise exception.StackValidationFailed(
                 message=_("Duplicate names %s") % dup_names)
 
-        for res in self.dependencies:
+        if validate_by_deps:
+            iter_rsc = self.dependencies
+        else:
+            iter_rsc = six.itervalues(self.resources)
+
+        for res in iter_rsc:
             try:
                 if self.resource_validate:
                     result = res.validate()
@@ -689,7 +755,10 @@ class Stack(collections.Mapping):
                     result = res.validate_template()
             except exception.HeatException as ex:
                 LOG.debug('%s', ex)
-                raise ex
+                if ignorable_errors and ex.error_code in ignorable_errors:
+                    result = None
+                else:
+                    raise ex
             except AssertionError:
                 raise
             except Exception as ex:
@@ -743,6 +812,20 @@ class Stack(collections.Mapping):
                          self.name, 'OS::Heat::Stack')
 
         ev.store()
+        self.dispatch_event(ev)
+
+    def dispatch_event(self, ev):
+        def _dispatch(ctx, sinks, ev):
+            try:
+                for sink in sinks:
+                    sink.consume(ctx, ev)
+            except Exception as e:
+                LOG.debug('Got error sending events %s' % e)
+        if self.thread_group_mgr is not None:
+            self.thread_group_mgr.start(self.id, _dispatch,
+                                        self.context,
+                                        self.env.get_event_sinks(),
+                                        ev.as_dict())
 
     @profiler.trace('Stack.state_set', hide_args=False)
     def state_set(self, action, status, reason):
@@ -757,6 +840,21 @@ class Stack(collections.Mapping):
         self.status = status
         self.status_reason = reason
 
+        if self.convergence and action in (self.UPDATE, self.DELETE,
+                                           self.CREATE, self.ADOPT):
+            # if convergence and stack operation is create/update/delete,
+            # stack lock is not used, hence persist state
+            updated = self._persist_state()
+            if not updated:
+                # Possibly failed concurrent update
+                LOG.warning(_LW("Failed to set state of stack %(name)s with"
+                                " traversal ID %(trvsl_id)s, to"
+                                " %(action)s_%(status)s"),
+                            {'name': self.name,
+                             'trvsl_id': self.current_traversal,
+                             'action': action, 'status': status})
+            return updated
+
         # Persist state to db only if status == IN_PROGRESS
         # or action == UPDATE/DELETE/ROLLBACK. Else, it would
         # be done before releasing the stack lock.
@@ -774,7 +872,14 @@ class Stack(collections.Mapping):
                       'status': self.status,
                       'status_reason': self.status_reason}
             self._send_notification_and_add_event()
-            stack.update_and_save(values)
+            if self.convergence:
+                # do things differently for convergence
+                updated = stack_object.Stack.select_and_update(
+                    self.context, self.id, values,
+                    exp_trvsl=self.current_traversal)
+                return updated
+            else:
+                stack.update_and_save(values)
 
     def _send_notification_and_add_event(self):
         notification.send(self)
@@ -851,18 +956,18 @@ class Stack(collections.Mapping):
                    aggregate_exceptions=False, pre_completion_func=None):
         """A task to perform an action on the stack.
 
-        All of the resources in forward or reverse dependency order as
-        specified by reverse.
+        All of the resources are traversed in forward or reverse dependency
+        order.
 
-        :param action action that should be executed with stack resources
-        :param reverse defines if action on the resources need to be executed
+        :param action: action that should be executed with stack resources
+        :param reverse: define if action on the resources need to be executed
          in reverse order (resources - first and then res dependencies )
-        :param post_func function that need to be executed after
+        :param post_func: function that need to be executed after
         action complete on the stack
-        :param error_wait_time time to wait before cancelling all execution
+        :param error_wait_time: time to wait before cancelling all execution
         threads when an error occurred
-        :param aggregate_exceptions defines if exceptions should be aggregated
-        :param pre_completion_func function that need to be executed right
+        :param aggregate_exceptions: define if exceptions should be aggregated
+        :param pre_completion_func: function that need to be executed right
         before action completion. Uses stack ,action, status and reason as
         input parameters
         """
@@ -881,15 +986,18 @@ class Stack(collections.Mapping):
         stack_status = self.COMPLETE
         reason = 'Stack %s completed successfully' % action
 
+        action_method = action.lower()
+        # If a local _$action_kwargs function exists, call it to get the
+        # action specific argument list, otherwise an empty arg list
+        handle_kwargs = getattr(self,
+                                '_%s_kwargs' % action_method,
+                                lambda x: {})
+
+        @functools.wraps(getattr(resource.Resource, action_method))
         def resource_action(r):
             # Find e.g resource.create and call it
-            action_l = action.lower()
-            handle = getattr(r, '%s' % action_l)
+            handle = getattr(r, action_method)
 
-            # If a local _$action_kwargs function exists, call it to get the
-            # action specific argument list, otherwise an empty arg list
-            handle_kwargs = getattr(self,
-                                    '_%s_kwargs' % action_l, lambda x: {})
             return handle(**handle_kwargs(r))
 
         action_task = scheduler.DependencyTaskGroup(
@@ -926,7 +1034,7 @@ class Stack(collections.Mapping):
     @profiler.trace('Stack.check', hide_args=False)
     @reset_state_on_error
     def check(self):
-        self.updated_time = datetime.datetime.utcnow()
+        self.updated_time = oslo_timeutils.utcnow()
         checker = scheduler.TaskRunner(
             self.stack_task, self.CHECK,
             post_func=self.supports_check_action,
@@ -969,7 +1077,8 @@ class Stack(collections.Mapping):
             kwargs = self.get_kwargs_for_cloning()
             kwargs['owner_id'] = self.id
             del(kwargs['prev_raw_template_id'])
-            prev = type(self)(self.context, self.name, copy.deepcopy(self.t),
+            prev = type(self)(self.context, self._backup_name(),
+                              copy.deepcopy(self.t),
                               **kwargs)
             prev.store(backup=True)
             LOG.debug('Created new backup stack')
@@ -980,7 +1089,7 @@ class Stack(collections.Mapping):
     @profiler.trace('Stack.adopt', hide_args=False)
     @reset_state_on_error
     def adopt(self):
-        """Adopt the stack (create stack with all the existing resources)."""
+        """Adopt existing resources into a new stack."""
         def rollback():
             if not self.disable_rollback and self.state == (self.ADOPT,
                                                             self.FAILED):
@@ -1012,14 +1121,14 @@ class Stack(collections.Mapping):
         Update will fail if it exceeds the specified timeout. The default is
         60 minutes, set in the constructor
         """
-        self.updated_time = datetime.datetime.utcnow()
+        self.updated_time = oslo_timeutils.utcnow()
         updater = scheduler.TaskRunner(self.update_task, newstack,
                                        event=event)
         updater()
 
     @profiler.trace('Stack.converge_stack', hide_args=False)
     def converge_stack(self, template, action=UPDATE, new_stack=None):
-        """Updates the stack and triggers convergence for resources."""
+        """Update the stack template and trigger convergence for resources."""
         if action not in [self.CREATE, self.ADOPT]:
             # no back-up template for create action
             self.prev_raw_template_id = getattr(self.t, 'id', None)
@@ -1029,9 +1138,9 @@ class Stack(collections.Mapping):
         self.reset_dependencies()
         self._resources = None
 
-        previous_traversal = self.current_traversal
-        self.current_traversal = uuidutils.generate_uuid()
-        self.updated_time = datetime.datetime.utcnow()
+        if action is not self.CREATE:
+            self.updated_time = oslo_timeutils.utcnow()
+
         if new_stack is not None:
             self.disable_rollback = new_stack.disable_rollback
             self.timeout_mins = new_stack.timeout_mins
@@ -1044,14 +1153,29 @@ class Stack(collections.Mapping):
             else:
                 stack_tag_object.StackTagList.delete(self.context, self.id)
 
-        self.store()
+        self.action = action
+        self.status = self.IN_PROGRESS
+        self.status_reason = 'Stack %s started' % self.action
 
-        # TODO(later): lifecycle_plugin_utils.do_pre_ops
-        self.state_set(action, self.IN_PROGRESS,
-                       'Stack %s started' % action)
+        # generate new traversal and store
+        previous_traversal = self.current_traversal
+        self.current_traversal = uuidutils.generate_uuid()
+        # we expect to update the stack having previous traversal ID
+        stack_id = self.store(exp_trvsl=previous_traversal)
+        if stack_id is None:
+            LOG.warning(_LW("Failed to store stack %(name)s with traversal "
+                            "ID %(trvsl_id)s, aborting stack %(action)s"),
+                        {'name': self.name, 'trvsl_id': previous_traversal,
+                         'action': self.action})
+            return
+        self._send_notification_and_add_event()
 
         # delete the prev traversal sync_points
-        sync_point.delete_all(self.context, self.id, previous_traversal)
+        if previous_traversal:
+            sync_point.delete_all(self.context, self.id, previous_traversal)
+
+        # TODO(later): lifecycle_plugin_utils.do_pre_ops
+
         self._converge_create_or_update()
 
     def _converge_create_or_update(self):
@@ -1062,7 +1186,14 @@ class Stack(collections.Mapping):
         self.current_deps = {
             'edges': [[rqr, rqd] for rqr, rqd in
                       self.convergence_dependencies.graph().edges()]}
-        self.store()
+        stack_id = self.store()
+        if stack_id is None:
+            # Failed concurrent update
+            LOG.warning(_LW("Failed to store stack %(name)s with traversal "
+                            "ID %(trvsl_id)s, aborting stack %(action)s"),
+                        {'name': self.name, 'trvsl_id': self.current_traversal,
+                         'action': self.action})
+            return
 
         LOG.info(_LI('convergence_dependencies: %s'),
                  self.convergence_dependencies)
@@ -1078,12 +1209,14 @@ class Stack(collections.Mapping):
 
         leaves = set(self.convergence_dependencies.leaves())
         if not any(leaves):
-            self.mark_complete(self.current_traversal)
+            self.mark_complete()
         else:
             for rsrc_id, is_update in self.convergence_dependencies.leaves():
-                LOG.info(_LI("Triggering resource %(rsrc_id)s "
-                             "for %(is_update)s update"),
-                         {'rsrc_id': rsrc_id, 'is_update': is_update})
+                if is_update:
+                    LOG.info(_LI("Triggering resource %s for update"), rsrc_id)
+                else:
+                    LOG.info(_LI("Triggering resource %s for cleanup"),
+                             rsrc_id)
                 input_data = sync_point.serialize_input_data({})
                 self.worker_client.check_resource(self.context, rsrc_id,
                                                   self.current_traversal,
@@ -1098,7 +1231,14 @@ class Stack(collections.Mapping):
         else:
             rollback_tmpl = tmpl.Template.load(self.context, old_tmpl_id)
             self.prev_raw_template_id = None
-            self.store()
+            stack_id = self.store()
+            if stack_id is None:
+                # Failed concurrent update
+                LOG.warning(_LW("Failed to store stack %(name)s with traversal"
+                                " ID %(trvsl_id)s, not trigerring rollback."),
+                            {'name': self.name,
+                             'trvsl_id': self.current_traversal})
+                return
 
         self.converge_stack(rollback_tmpl, action=self.ROLLBACK)
 
@@ -1116,6 +1256,12 @@ class Stack(collections.Mapping):
                         self.prev_raw_template_id):
                     # Current resource is otherwise a good candidate
                     candidate = ext_rsrc
+                elif candidate is None:
+                    # In multiple concurrent updates, if candidate is not
+                    # found in current/previous template, it could be found
+                    # in old tmpl.
+                    candidate = ext_rsrc
+
         return candidate
 
     def _update_or_store_resources(self):
@@ -1209,8 +1355,7 @@ class Stack(collections.Mapping):
         self.action = action
         self.status = self.IN_PROGRESS
         self.status_reason = 'Stack %s started' % action
-        notification.send(self)
-        self._add_event(self.action, self.status, self.status_reason)
+        self._send_notification_and_add_event()
         self.store()
         if prev_tmpl_id is not None:
             raw_template_object.RawTemplate.delete(self.context, prev_tmpl_id)
@@ -1225,8 +1370,8 @@ class Stack(collections.Mapping):
         backup_stack = self._backup_stack()
         existing_params = environment.Environment({env_fmt.PARAMETERS:
                                                   self.t.env.params})
-        should_rollback = False
         previous_template_id = None
+        should_rollback = False
         try:
             update_task = update.StackUpdate(
                 self, newstack, backup_stack,
@@ -1310,6 +1455,7 @@ class Stack(collections.Mapping):
             if previous_template_id is not None:
                 raw_template_object.RawTemplate.delete(self.context,
                                                        previous_template_id)
+
             lifecycle_plugin_utils.do_post_ops(self.context, self,
                                                newstack, action,
                                                (self.status == self.FAILED))
@@ -1384,7 +1530,7 @@ class Stack(collections.Mapping):
 
         stack.delete(backup=True)
 
-    def _try_get_user_creds(self, user_creds_id):
+    def _try_get_user_creds(self):
         # There are cases where the user_creds cannot be returned
         # due to credentials truncated when being saved to DB.
         # Ignore this error instead of blocking stack deletion.
@@ -1402,7 +1548,7 @@ class Stack(collections.Mapping):
         # The stack_status and reason passed in are current values, which
         # may get rewritten and returned from this method
         if self.user_creds_id:
-            user_creds = self._try_get_user_creds(self.user_creds_id)
+            user_creds = self._try_get_user_creds()
             # If we created a trust, delete it
             if user_creds is not None:
                 trust_id = user_creds.get('trust_id')
@@ -1509,11 +1655,8 @@ class Stack(collections.Mapping):
                                'Failed stack pre-ops: %s' % six.text_type(e))
                 return
 
-        def destroy_resource(stack_resource):
-            return stack_resource.destroy()
-
         action_task = scheduler.DependencyTaskGroup(self.dependencies,
-                                                    destroy_resource,
+                                                    resource.Resource.destroy,
                                                     reverse=True)
         try:
             scheduler.TaskRunner(action_task)(timeout=self.timeout_secs())
@@ -1568,7 +1711,7 @@ class Stack(collections.Mapping):
             LOG.info(_LI('%s is already suspended'), six.text_type(self))
             return
 
-        self.updated_time = datetime.datetime.utcnow()
+        self.updated_time = oslo_timeutils.utcnow()
         sus_task = scheduler.TaskRunner(
             self.stack_task,
             action=self.SUSPEND,
@@ -1594,7 +1737,7 @@ class Stack(collections.Mapping):
             LOG.info(_LI('%s is already resumed'), six.text_type(self))
             return
 
-        self.updated_time = datetime.datetime.utcnow()
+        self.updated_time = oslo_timeutils.utcnow()
         sus_task = scheduler.TaskRunner(
             self.stack_task,
             action=self.RESUME,
@@ -1606,7 +1749,7 @@ class Stack(collections.Mapping):
     @reset_state_on_error
     def snapshot(self, save_snapshot_func):
         """Snapshot the stack, invoking handle_snapshot on all resources."""
-        self.updated_time = datetime.datetime.utcnow()
+        self.updated_time = oslo_timeutils.utcnow()
         sus_task = scheduler.TaskRunner(
             self.stack_task,
             action=self.SNAPSHOT,
@@ -1631,7 +1774,7 @@ class Stack(collections.Mapping):
 
         Invokes handle_restore on all resources.
         """
-        self.updated_time = datetime.datetime.utcnow()
+        self.updated_time = oslo_timeutils.utcnow()
         env = environment.Environment(snapshot.data['environment'])
         files = snapshot.data['files']
         template = tmpl.Template(snapshot.data['template'],
@@ -1728,7 +1871,8 @@ class Stack(collections.Mapping):
             'resources': dict((res.name, res.prepare_abandon())
                               for res in six.itervalues(self.resources)),
             'project_id': self.tenant_id,
-            'stack_user_project_id': self.stack_user_project_id
+            'stack_user_project_id': self.stack_user_project_id,
+            'tags': self.tags,
         }
 
     def resolve_static_data(self, snippet):
@@ -1765,21 +1909,23 @@ class Stack(collections.Mapping):
         attrs = self.cache_data.get(resource_name, {}).get('attributes', {})
         return attrs
 
-    def mark_complete(self, traversal_id):
+    def mark_complete(self):
         """Mark the update as complete.
 
         This currently occurs when all resources have been updated; there may
         still be resources being cleaned up, but the Stack should now be in
         service.
         """
-        if traversal_id != self.current_traversal:
-            return
 
         LOG.info(_LI('[%(name)s(%(id)s)] update traversal %(tid)s complete'),
-                 {'name': self.name, 'id': self.id, 'tid': traversal_id})
+                 {'name': self.name, 'id': self.id,
+                  'tid': self.current_traversal})
 
         reason = 'Stack %s completed successfully' % self.action
-        self.state_set(self.action, self.COMPLETE, reason)
+        updated = self.state_set(self.action, self.COMPLETE, reason)
+        if not updated:
+            return
+
         self.purge_db()
 
     def purge_db(self):
@@ -1794,12 +1940,28 @@ class Stack(collections.Mapping):
                 self.status != self.FAILED):
             prev_tmpl_id = self.prev_raw_template_id
             self.prev_raw_template_id = None
-            self.store()
+            stack_id = self.store()
+            if stack_id is None:
+                # Failed concurrent update
+                LOG.warning(_LW("Failed to store stack %(name)s with traversal"
+                                " ID %(trvsl_id)s, aborting stack purge"),
+                            {'name': self.name,
+                             'trvsl_id': self.current_traversal})
+                return
             raw_template_object.RawTemplate.delete(self.context, prev_tmpl_id)
 
         sync_point.delete_all(self.context, self.id, self.current_traversal)
 
         if (self.action, self.status) == (self.DELETE, self.COMPLETE):
+            if not self.owner_id:
+                status, reason = self._delete_credentials(
+                    self.status,
+                    self.status_reason,
+                    False)
+                if status == self.FAILED:
+                    # something wrong when delete credentials, set FAILED
+                    self.state_set(self.action, status, reason)
+                    return
             try:
                 stack_object.Stack.delete(self.context, self.id)
             except exception.NotFound:
