@@ -17,7 +17,6 @@ import copy
 import functools
 import hashlib
 
-from oslo_log import log as logging
 import six
 from stevedore import extension
 
@@ -26,8 +25,6 @@ from heat.common.i18n import _
 from heat.engine import environment
 from heat.objects import raw_template as template_object
 
-LOG = logging.getLogger(__name__)
-
 __all__ = ['Template']
 
 
@@ -262,7 +259,7 @@ class Template(collections.Mapping):
     @classmethod
     def create_empty_template(cls,
                               version=('heat_template_version', '2015-04-30')):
-        """Creates an empty template.
+        """Create an empty template.
 
         Creates a new empty template with given version. If version is
         not provided, a new empty HOT template of version "2015-04-30"
