@@ -77,17 +77,20 @@ class WatchRule(object):
     def load(cls, context, watch_name=None, watch=None):
         """Load the watchrule object.
 
-        Loading object either by name or via an existing DB object.
+        The object can be loaded either from the DB by name or from an existing
+        DB object.
         """
         if watch is None:
             try:
                 watch = watch_rule_objects.WatchRule.get_by_name(context,
                                                                  watch_name)
             except Exception as ex:
-                LOG.warn(_LW('WatchRule.load (%(watch_name)s) db error '
-                             '%(ex)s'), {'watch_name': watch_name, 'ex': ex})
+                LOG.warning(_LW('WatchRule.load (%(watch_name)s) db error '
+                                '%(ex)s'), {'watch_name': watch_name,
+                                            'ex': ex})
         if watch is None:
-            raise exception.WatchRuleNotFound(watch_name=watch_name)
+            raise exception.EntityNotFound(entity='Watch Rule',
+                                           name=watch_name)
         else:
             return cls(context=context,
                        watch_name=watch.name,
@@ -270,8 +273,8 @@ class WatchRule(object):
                 for refid in self.rule[self.ACTION_MAP[new_state]]:
                     actions.append(stk.resource_by_refid(refid).signal)
             else:
-                LOG.warn(_LW("Could not process watch state %s for stack"),
-                         new_state)
+                LOG.warning(_LW("Could not process watch state %s for stack"),
+                            new_state)
         return actions
 
     def _to_ceilometer(self, data):
@@ -353,9 +356,9 @@ class WatchRule(object):
                           % {'self_state': self.state, 'name': self.name,
                              'state': state})
             else:
-                LOG.warn(_LW("Unable to override state %(state)s for "
-                             "watch %(name)s"), {'state': self.state,
-                                                 'name': self.name})
+                LOG.warning(_LW("Unable to override state %(state)s for "
+                                "watch %(name)s"), {'state': self.state,
+                                                    'name': self.name})
         return actions
 
 
