@@ -13,10 +13,7 @@
 # under the License.
 
 
-"""
-Resource object
-"""
-
+"""Resource object."""
 
 from oslo_config import cfg
 from oslo_serialization import jsonutils
@@ -29,6 +26,7 @@ from heat.common import crypt
 from heat.common import exception
 from heat.common.i18n import _
 from heat.db import api as db_api
+from heat.objects import base as heat_base
 from heat.objects import fields as heat_fields
 from heat.objects import resource_data
 from heat.objects import stack
@@ -46,7 +44,7 @@ def retry_on_conflict(func):
 
 
 class Resource(
-    base.VersionedObject,
+    heat_base.HeatObject,
     base.VersionedObjectDictCompat,
     base.ComparableVersionedObject,
 ):
@@ -85,12 +83,9 @@ class Resource(
             return None
         for field in resource.fields:
             if field == 'data':
-                resource['data'] = map(
-                    lambda resd: resource_data.ResourceData._from_db_object(
-                        resource_data.ResourceData(context), resd
-                    ),
-                    db_resource.data
-                )
+                resource['data'] = [resource_data.ResourceData._from_db_object(
+                    resource_data.ResourceData(context), resd
+                ) for resd in db_resource.data]
             else:
                 resource[field] = db_resource[field]
 
@@ -142,9 +137,10 @@ class Resource(
             resource_id2)
 
     @classmethod
-    def get_all_by_stack(cls, context, stack_id, key_id=False):
+    def get_all_by_stack(cls, context, stack_id, key_id=False, filters=None):
         resources_db = db_api.resource_get_all_by_stack(context,
-                                                        stack_id, key_id)
+                                                        stack_id, key_id,
+                                                        filters)
         resources = [
             (
                 resource_key,
@@ -169,10 +165,14 @@ class Resource(
             physical_resource_id)
         return cls._from_db_object(cls(context), context, resource_db)
 
+    @classmethod
+    def update_by_id(cls, context, resource_id, values):
+        resource_db = db_api.resource_get(context, resource_id)
+        resource_db.update_and_save(values)
+
     def update_and_save(self, values):
         resource_db = db_api.resource_get(self._context, self.id)
         resource_db.update_and_save(values)
-        return self.refresh()
 
     def select_and_update(self, values, expected_engine_id=None,
                           atomic_key=0):
