@@ -13,9 +13,9 @@
 # under the License.
 
 
-"""
-Stack object
-"""
+"""Stack object."""
+
+import six
 
 from oslo_versionedobjects import base
 from oslo_versionedobjects import fields
@@ -23,13 +23,14 @@ from oslo_versionedobjects import fields
 from heat.common import exception
 from heat.common.i18n import _
 from heat.db import api as db_api
+from heat.objects import base as heat_base
 from heat.objects import fields as heat_fields
 from heat.objects import raw_template
 from heat.objects import stack_tag
 
 
 class Stack(
-    base.VersionedObject,
+    heat_base.HeatObject,
     base.VersionedObjectDictCompat,
     base.ComparableVersionedObject,
 ):
@@ -70,11 +71,8 @@ class Stack(
                     raw_template.RawTemplate.get_by_id(
                         context, db_stack['raw_template_id']))
             elif field == 'tags':
-                if db_stack.get(field) is not None:
-                    stack['tags'] = stack_tag.StackTagList.get(
-                        context, db_stack['id'])
-                else:
-                    stack['tags'] = None
+                stack['tags'] = stack_tag.StackTagList.from_db_object(
+                    context, db_stack.get(field))
             else:
                 stack[field] = db_stack.__dict__.get(field)
         stack._context = context
@@ -97,7 +95,7 @@ class Stack(
     def get_by_name_and_owner_id(cls, context, stack_name, owner_id):
         db_stack = db_api.stack_get_by_name_and_owner_id(
             context,
-            stack_name,
+            six.text_type(stack_name),
             owner_id
         )
         if not db_stack:
@@ -107,7 +105,7 @@ class Stack(
 
     @classmethod
     def get_by_name(cls, context, stack_name):
-        db_stack = db_api.stack_get_by_name(context, stack_name)
+        db_stack = db_api.stack_get_by_name(context, six.text_type(stack_name))
         if not db_stack:
             return None
         stack = cls._from_db_object(context, cls(context), db_stack)
@@ -116,24 +114,20 @@ class Stack(
     @classmethod
     def get_all(cls, context, *args, **kwargs):
         db_stacks = db_api.stack_get_all(context, *args, **kwargs)
-        stacks = map(
-            lambda db_stack: cls._from_db_object(
-                context,
-                cls(context),
-                db_stack),
-            db_stacks)
-        return stacks
+        for db_stack in db_stacks:
+            try:
+                yield cls._from_db_object(context, cls(context), db_stack)
+            except exception.NotFound:
+                pass
 
     @classmethod
     def get_all_by_owner_id(cls, context, owner_id):
         db_stacks = db_api.stack_get_all_by_owner_id(context, owner_id)
-        stacks = map(
-            lambda db_stack: cls._from_db_object(
-                context,
-                cls(context),
-                db_stack),
-            db_stacks)
-        return stacks
+        for db_stack in db_stacks:
+            try:
+                yield cls._from_db_object(context, cls(context), db_stack)
+            except exception.NotFound:
+                pass
 
     @classmethod
     def count_all(cls, context, **kwargs):
@@ -161,6 +155,9 @@ class Stack(
     def select_and_update(cls, context, stack_id, values, exp_trvsl=None):
         """Update the stack by selecting on traversal ID.
 
+        Uses UPDATE ... WHERE (compare and swap) to catch any concurrent
+        update problem.
+
         If the stack is found with given traversal, it is updated.
 
         If there occurs a race while updating, only one will succeed and
@@ -189,8 +186,6 @@ class Stack(
                                            'traversal': self.current_traversal,
                                            'msg': 'that does not exist'})
 
-        return self.refresh()
-
     def __eq__(self, another):
         self.refresh()  # to make test object comparison work well
         return super(Stack, self).__eq__(another)
@@ -201,7 +196,6 @@ class Stack(
         if db_stack is None:
             message = _('No stack exists with id "%s"') % str(self.id)
             raise exception.NotFound(message)
-        db_stack.refresh()
         return self.__class__._from_db_object(
             self._context,
             self,
@@ -211,3 +205,8 @@ class Stack(
     @classmethod
     def encrypt_hidden_parameters(cls, tmpl):
         raw_template.RawTemplate.encrypt_hidden_parameters(tmpl)
+
+    @classmethod
+    def get_status(cls, context, stack_id):
+        """Return action and status for the given stack."""
+        return db_api.stack_get_status(context, stack_id)
