@@ -117,6 +117,26 @@ class TestAutoScalingPolicy(common.HeatTestCase):
             mock_isa.assert_called_once_with()
         group.adjust.assert_called_once_with(1, 'change_in_capacity', None)
 
+    def test_scaling_policy_refid(self):
+        t = template_format.parse(as_template)
+        stack = utils.parse_stack(t)
+        rsrc = stack['my-policy']
+        rsrc.resource_id = 'xyz'
+        self.assertEqual('xyz', rsrc.FnGetRefId())
+
+    def test_scaling_policy_refid_convg_cache_data(self):
+        t = template_format.parse(as_template)
+        cache_data = {'my-policy': {
+            'uuid': mock.ANY,
+            'id': mock.ANY,
+            'action': 'CREATE',
+            'status': 'COMPLETE',
+            'reference_id': 'convg_xyz'
+        }}
+        stack = utils.parse_stack(t, cache_data=cache_data)
+        rsrc = stack['my-policy']
+        self.assertEqual('convg_xyz', rsrc.FnGetRefId())
+
 
 class TestCooldownMixin(common.HeatTestCase):
     def setUp(self):
@@ -197,16 +217,6 @@ class TestCooldownMixin(common.HeatTestCase):
         self.patchobject(pol, 'metadata_get', return_value=previous_meta)
         self.assertTrue(pol._is_scaling_allowed())
 
-    def test_no_cooldown_no_scaling_in_progress(self):
-        t = template_format.parse(as_template)
-        stack = utils.parse_stack(t, params=as_params)
-        pol = self.create_scaling_policy(t, stack, 'my-policy')
-
-        # no cooldown entry in the metadata
-        previous_meta = {'scaling_in_progress': False}
-        self.patchobject(pol, 'metadata_get', return_value=previous_meta)
-        self.assertTrue(pol._is_scaling_allowed())
-
     def test_metadata_is_written(self):
         t = template_format.parse(as_template)
         stack = utils.parse_stack(t, params=as_params)
