@@ -15,6 +15,7 @@ from ceilometerclient import exc as ceil_exc
 from ceilometerclient.openstack.common.apiclient import exceptions as c_a_exc
 from cinderclient import exceptions as cinder_exc
 from glanceclient import exc as glance_exc
+from glanceclient.openstack.common.apiclient import exceptions as g_a_exc
 from heatclient import client as heatclient
 from heatclient import exc as heat_exc
 from keystoneclient.auth.identity import v3
@@ -35,7 +36,7 @@ from heat.engine import clients
 from heat.engine.clients import client_plugin
 from heat.tests import common
 from heat.tests import fakes
-from heat.tests.nova import fakes as fakes_nova
+from heat.tests.openstack.nova import fakes as fakes_nova
 from heat.tests import utils
 
 
@@ -73,18 +74,36 @@ class ClientsTest(common.HeatTestCase):
         obj._get_client_option.return_value = result
         self.assertEqual(result, obj.get_heat_url())
 
-    def test_clients_get_heat_cfn_url(self):
+    def _client_cfn_url(self):
         con = mock.Mock()
         c = clients.Clients(con)
         con.clients = c
-
         obj = c.client_plugin('heat')
         obj._get_client_option = mock.Mock()
         obj._get_client_option.return_value = None
         obj.url_for = mock.Mock(name="url_for")
-        heat_cfn_url = "http://0.0.0.0:8000/v1"
-        obj.url_for.return_value = heat_cfn_url
-        self.assertEqual(heat_cfn_url, obj.get_heat_cfn_url())
+        obj.url_for.return_value = "http://0.0.0.0:8000/v1/"
+        return obj
+
+    def test_clients_get_heat_cfn_url(self):
+        obj = self._client_cfn_url()
+        self.assertEqual("http://0.0.0.0:8000/v1/", obj.get_heat_cfn_url())
+
+    def test_clients_get_watch_server_url(self):
+        obj = self._client_cfn_url()
+        self.assertEqual("http://0.0.0.0:8003/v1/", obj.get_watch_server_url())
+
+    def test_clients_get_heat_cfn_metadata_url(self):
+        obj = self._client_cfn_url()
+        self.assertEqual("http://0.0.0.0:8000/v1/",
+                         obj.get_cfn_metadata_server_url())
+
+    def test_clients_get_heat_cfn_metadata_url_conf(self):
+        cfg.CONF.set_override('heat_metadata_server_url',
+                              'http://server.test:123')
+        obj = self._client_cfn_url()
+        self.assertEqual("http://server.test:123/v1/",
+                         obj.get_cfn_metadata_server_url())
 
     @mock.patch.object(heatclient, 'Client')
     def test_clients_heat(self, mock_call):
@@ -303,6 +322,33 @@ class ClientPluginTest(common.HeatTestCase):
         self.assertEqual('http://192.0.2.1/bar',
                          plugin.url_for(service_type='bar'))
 
+    @mock.patch.object(v3, "Token", name="v3_token")
+    def test_endpoint_not_found(self, mock_v3):
+        class FakeKeystone(fakes.FakeKeystoneClient):
+            def __init__(self):
+                super(FakeKeystone, self).__init__()
+                self.client = self
+                self.version = 'v3'
+
+        self.stub_keystoneclient(fake_client=FakeKeystone())
+        con = mock.MagicMock(auth_token="1234", trust_id=None)
+        c = clients.Clients(con)
+        con.clients = c
+
+        con.auth_plugin = mock.Mock(name="auth_plugin")
+        get_endpoint_side_effects = [keystone_exc.EmptyCatalog(), None]
+        con.auth_plugin.get_endpoint = mock.Mock(
+            name="get_endpoint", side_effect=get_endpoint_side_effects)
+
+        mock_token_obj = mock.Mock()
+        mock_v3.return_value = mock_token_obj
+        mock_token_obj.get_auth_ref.return_value = {'no_catalog': 'without'}
+
+        plugin = FooClientsPlugin(con)
+
+        self.assertRaises(keystone_exc.EndpointNotFound,
+                          plugin.url_for, service_type='nonexistent')
+
     def test_abstract_create(self):
         con = mock.Mock()
         c = clients.Clients(con)
@@ -367,7 +413,7 @@ class TestClientPluginsInitialise(common.HeatTestCase):
             self.assertEqual(con, plugin.context)
             self.assertIsNone(plugin._client)
             self.assertTrue(clients.has_client(plugin_name))
-            self.assertTrue(isinstance(plugin.service_types, list))
+            self.assertIsInstance(plugin.service_types, list)
             self.assertTrue(len(plugin.service_types) >= 1,
                             'service_types is not defined for plugin')
 
@@ -460,13 +506,21 @@ class TestIsNotFound(common.HeatTestCase):
             exception=lambda: cinder_exc.ClientException(
                 code=409, message='conflict'),
         )),
-        ('glance_not_found', dict(
+        ('glance_not_found_1', dict(
             is_not_found=True,
             is_over_limit=False,
             is_client_exception=True,
             is_conflict=False,
             plugin='glance',
-            exception=lambda: glance_exc.HTTPNotFound(details='gone'),
+            exception=lambda: g_a_exc.NotFound(),
+        )),
+        ('glance_not_found_2', dict(
+            is_not_found=True,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=False,
+            plugin='glance',
+            exception=lambda: glance_exc.HTTPNotFound(),
         )),
         ('glance_exception', dict(
             is_not_found=False,
@@ -484,13 +538,21 @@ class TestIsNotFound(common.HeatTestCase):
             plugin='glance',
             exception=lambda: glance_exc.HTTPOverLimit(details='over'),
         )),
-        ('glance_conflict', dict(
+        ('glance_conflict_1', dict(
             is_not_found=False,
             is_over_limit=False,
             is_client_exception=True,
             is_conflict=True,
             plugin='glance',
-            exception=lambda: glance_exc.HTTPConflict(),
+            exception=lambda: g_a_exc.Conflict(),
+        )),
+        ('glance_conflict_1', dict(
+            is_not_found=False,
+            is_over_limit=False,
+            is_client_exception=True,
+            is_conflict=True,
+            plugin='glance',
+            exception=lambda: glance_exc.Conflict(),
         )),
         ('heat_not_found', dict(
             is_not_found=True,
@@ -833,6 +895,23 @@ class TestIsNotFound(common.HeatTestCase):
                                   client_plugin.ignore_not_found,
                                   e)
 
+    def test_ignore_not_found_context_manager(self):
+        con = mock.Mock()
+        c = clients.Clients(con)
+        client_plugin = c.client_plugin(self.plugin)
+
+        exp = self.exception()
+        exp_class = exp.__class__
+
+        def try_raise():
+            with client_plugin.ignore_not_found:
+                raise exp
+
+        if self.is_not_found:
+            try_raise()
+        else:
+            self.assertRaises(exp_class, try_raise)
+
     def test_ignore_conflict_and_not_found(self):
         con = mock.Mock()
         c = clients.Clients(con)
@@ -849,6 +928,23 @@ class TestIsNotFound(common.HeatTestCase):
                                   client_plugin.ignore_conflict_and_not_found,
                                   e)
 
+    def test_ignore_conflict_and_not_found_context_manager(self):
+        con = mock.Mock()
+        c = clients.Clients(con)
+        client_plugin = c.client_plugin(self.plugin)
+
+        exp = self.exception()
+        exp_class = exp.__class__
+
+        def try_raise():
+            with client_plugin.ignore_conflict_and_not_found:
+                raise exp
+
+        if self.is_conflict or self.is_not_found:
+            try_raise()
+        else:
+            self.assertRaises(exp_class, try_raise)
+
     def test_is_over_limit(self):
         con = mock.Mock()
         c = clients.Clients(con)
