@@ -11,11 +11,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import six
+
 import mock
 from neutronclient.common import exceptions as qe
 
 from heat.common import exception
 from heat.engine.clients.os import neutron
+from heat.engine.clients.os.neutron import lbaas_constraints as lc
+from heat.engine.clients.os.neutron import neutron_constraints as nc
 from heat.tests import common
 from heat.tests import utils
 
@@ -31,29 +35,21 @@ class NeutronClientPluginTestCase(common.HeatTestCase):
         self.neutron_plugin._client = self.neutron_client
 
 
-class NeutronClientPluginTests(NeutronClientPluginTestCase):
+class NeutronClientPluginTest(NeutronClientPluginTestCase):
     def setUp(self):
-        super(NeutronClientPluginTests, self).setUp()
+        super(NeutronClientPluginTest, self).setUp()
         self.mock_find = self.patchobject(neutron.neutronV20,
                                           'find_resourceid_by_name_or_id')
         self.mock_find.return_value = 42
 
-    def test_find_neutron_resource(self):
-        props = {'net': 'test_network'}
-
-        res = self.neutron_plugin.find_neutron_resource(props, 'net',
-                                                        'network')
-        self.assertEqual(42, res)
-        self.mock_find.assert_called_once_with(self.neutron_client, 'network',
-                                               'test_network')
-
     def test_resolve_network(self):
         props = {'net': 'test_network'}
 
         res = self.neutron_plugin.resolve_network(props, 'net', 'net_id')
         self.assertEqual(42, res)
         self.mock_find.assert_called_once_with(self.neutron_client, 'network',
-                                               'test_network')
+                                               'test_network',
+                                               cmd_resource=None)
 
         # check resolve if was send id instead of name
         props = {'net_id': 77}
@@ -61,7 +57,8 @@ class NeutronClientPluginTests(NeutronClientPluginTestCase):
         self.assertEqual(77, res)
         # in this case find_resourceid_by_name_or_id is not called
         self.mock_find.assert_called_once_with(self.neutron_client, 'network',
-                                               'test_network')
+                                               'test_network',
+                                               cmd_resource=None)
 
     def test_resolve_subnet(self):
         props = {'snet': 'test_subnet'}
@@ -69,7 +66,8 @@ class NeutronClientPluginTests(NeutronClientPluginTestCase):
         res = self.neutron_plugin.resolve_subnet(props, 'snet', 'snet_id')
         self.assertEqual(42, res)
         self.mock_find.assert_called_once_with(self.neutron_client, 'subnet',
-                                               'test_subnet')
+                                               'test_subnet',
+                                               cmd_resource=None)
 
         # check resolve if was send id instead of name
         props = {'snet_id': 77}
@@ -77,7 +75,8 @@ class NeutronClientPluginTests(NeutronClientPluginTestCase):
         self.assertEqual(77, res)
         # in this case find_resourceid_by_name_or_id is not called
         self.mock_find.assert_called_once_with(self.neutron_client, 'subnet',
-                                               'test_subnet')
+                                               'test_subnet',
+                                               cmd_resource=None)
 
     def test_get_secgroup_uuids(self):
         # test get from uuids
@@ -150,35 +149,142 @@ class NeutronClientPluginTests(NeutronClientPluginTestCase):
                           self.neutron_plugin.get_secgroup_uuids,
                           sgs_non_uuid)
 
+    def test_check_lb_status(self):
+        self.neutron_client.show_loadbalancer.side_effect = [
+            {'loadbalancer': {'provisioning_status': 'ACTIVE'}},
+            {'loadbalancer': {'provisioning_status': 'PENDING_CREATE'}},
+            {'loadbalancer': {'provisioning_status': 'ERROR'}}
+        ]
+
+        self.assertTrue(self.neutron_plugin.check_lb_status('1234'))
+        self.assertFalse(self.neutron_plugin.check_lb_status('1234'))
+        self.assertRaises(exception.ResourceInError,
+                          self.neutron_plugin.check_lb_status,
+                          '1234')
+
 
 class NeutronConstraintsValidate(common.HeatTestCase):
     scenarios = [
         ('validate_network',
-            dict(constraint_class=neutron.NetworkConstraint,
-                 resource_type='network')),
+            dict(constraint_class=nc.NetworkConstraint,
+                 resource_type='network',
+                 cmd_resource=None)),
         ('validate_port',
-            dict(constraint_class=neutron.PortConstraint,
-                 resource_type='port')),
+            dict(constraint_class=nc.PortConstraint,
+                 resource_type='port',
+                 cmd_resource=None)),
         ('validate_router',
-            dict(constraint_class=neutron.RouterConstraint,
-                 resource_type='router')),
+            dict(constraint_class=nc.RouterConstraint,
+                 resource_type='router',
+                 cmd_resource=None)),
         ('validate_subnet',
-            dict(constraint_class=neutron.SubnetConstraint,
-                 resource_type='subnet'))
+            dict(constraint_class=nc.SubnetConstraint,
+                 resource_type='subnet',
+                 cmd_resource=None)),
+        ('validate_subnetpool',
+            dict(constraint_class=nc.SubnetPoolConstraint,
+                 resource_type='subnetpool',
+                 cmd_resource=None)),
+        ('validate_address_scope',
+            dict(constraint_class=nc.AddressScopeConstraint,
+                 resource_type='address_scope',
+                 cmd_resource=None)),
+        ('validate_loadbalancer',
+            dict(constraint_class=lc.LoadbalancerConstraint,
+                 resource_type='loadbalancer',
+                 cmd_resource='lbaas_loadbalancer')),
+        ('validate_listener',
+            dict(constraint_class=lc.ListenerConstraint,
+                 resource_type='listener',
+                 cmd_resource=None)),
+        ('validate_pool',
+            dict(constraint_class=lc.PoolConstraint,
+                 resource_type='pool',
+                 cmd_resource='lbaas_pool')),
+        ('validate_qos_policy',
+            dict(constraint_class=nc.QoSPolicyConstraint,
+                 resource_type='policy',
+                 cmd_resource='qos_policy'))
     ]
 
     def test_validate(self):
+        mock_extension = self.patchobject(
+            neutron.NeutronClientPlugin, 'has_extension', return_value=True)
         nc = mock.Mock()
         mock_create = self.patchobject(neutron.NeutronClientPlugin, '_create')
         mock_create.return_value = nc
-        mock_find = self.patchobject(neutron.neutronV20,
+        mock_find = self.patchobject(neutron.NeutronClientPlugin,
                                      'find_resourceid_by_name_or_id')
-        mock_find.side_effect = ['foo',
-                                 qe.NeutronClientException(status_code=404)]
+        mock_find.side_effect = [
+            'foo',
+            qe.NeutronClientException(status_code=404)
+        ]
 
         constraint = self.constraint_class()
         ctx = utils.dummy_context()
+        if hasattr(constraint, 'extension') and constraint.extension:
+            mock_extension.side_effect = [
+                False,
+                True,
+                True,
+            ]
+            ex = self.assertRaises(
+                exception.EntityNotFound,
+                constraint.validate_with_client, ctx.clients, "foo"
+            )
+            expected = ("The neutron extension (%s) could not be found." %
+                        constraint.extension)
+            self.assertEqual(expected, six.text_type(ex))
         self.assertTrue(constraint.validate("foo", ctx))
         self.assertFalse(constraint.validate("bar", ctx))
-        mock_find.assert_has_calls([mock.call(nc, self.resource_type, 'foo'),
-                                    mock.call(nc, self.resource_type, 'bar')])
+        mock_find.assert_has_calls(
+            [mock.call(self.resource_type, 'foo',
+                       cmd_resource=self.cmd_resource),
+             mock.call(self.resource_type, 'bar',
+                       cmd_resource=self.cmd_resource)])
+
+
+class NeutronProviderConstraintsValidate(common.HeatTestCase):
+    scenarios = [
+        ('validate_lbaasv1',
+            dict(constraint_class=nc.LBaasV1ProviderConstraint,
+                 service_type='LOADBALANCER')),
+        ('validate_lbaasv2',
+            dict(constraint_class=lc.LBaasV2ProviderConstraint,
+                 service_type='LOADBALANCERV2'))
+    ]
+
+    def test_provider_validate(self):
+        nc = mock.Mock()
+        mock_create = self.patchobject(neutron.NeutronClientPlugin, '_create')
+        mock_create.return_value = nc
+        providers = {
+            'service_providers': [
+                {'service_type': 'LOADBANALCERV2', 'name': 'haproxy'},
+                {'service_type': 'LOADBANALCER', 'name': 'haproxy'}
+            ]
+        }
+        nc.list_service_providers.return_value = providers
+        constraint = self.constraint_class()
+        ctx = utils.dummy_context()
+        self.assertTrue(constraint.validate('haproxy', ctx))
+        self.assertFalse(constraint.validate("bar", ctx))
+
+
+class NeutronClientPluginExtensionsTest(NeutronClientPluginTestCase):
+    """Tests for extensions in neutronclient."""
+
+    def test_has_no_extension(self):
+        mock_extensions = {'extensions': []}
+        self.neutron_client.list_extensions.return_value = mock_extensions
+        self.assertFalse(self.neutron_plugin.has_extension('lbaas'))
+
+    def test_without_service_extension(self):
+        mock_extensions = {'extensions': [{'alias': 'router'}]}
+        self.neutron_client.list_extensions.return_value = mock_extensions
+        self.assertFalse(self.neutron_plugin.has_extension('lbaas'))
+
+    def test_has_service_extension(self):
+        mock_extensions = {'extensions': [{'alias': 'router'}]}
+        self.neutron_client.list_extensions.return_value = mock_extensions
+        self.assertTrue(self.neutron_plugin.has_extension('router'))
