@@ -11,14 +11,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
-import logging
 import os
 import sys
 
 import fixtures
 import mox
 from oslo_config import cfg
+from oslo_log import log as logging
 from oslotest import mockpatch
 import testscenarios
 import testtools
@@ -26,10 +25,11 @@ import testtools
 from heat.common import context
 from heat.common import messaging
 from heat.common import policy
+from heat.engine.clients.os import barbican
 from heat.engine.clients.os import cinder
 from heat.engine.clients.os import glance
 from heat.engine.clients.os import keystone
-from heat.engine.clients.os import neutron
+from heat.engine.clients.os.neutron import neutron_constraints as neutron
 from heat.engine.clients.os import nova
 from heat.engine.clients.os import sahara
 from heat.engine.clients.os import trove
@@ -60,7 +60,8 @@ class FakeLogMixin(object):
         self.LOG = self.useFixture(
             fixtures.FakeLogger(level=default_level, format=_LOG_FORMAT))
         base_list = set([nlog.split('.')[0]
-                         for nlog in logging.Logger.manager.loggerDict])
+                         for nlog in logging.logging.Logger.manager.loggerDict]
+                        )
         for base in base_list:
             if base in TEST_DEFAULT_LOGLEVELS:
                 self.useFixture(fixtures.FakeLogger(
@@ -190,8 +191,6 @@ class HeatTestCase(testscenarios.WithScenarios,
                                  generic_rsrc.ResourceTypeUnSupportedLiberty)
         resource._register_class('ResourceTypeSupportedKilo',
                                  generic_rsrc.ResourceTypeSupportedKilo)
-        resource._register_class('ResourceTypeHidden',
-                                 generic_rsrc.ResourceTypeHidden)
 
     def patchobject(self, obj, attr, **kwargs):
         mockfixture = self.useFixture(mockpatch.PatchObject(obj, attr,
@@ -269,10 +268,22 @@ class HeatTestCase(testscenarios.WithScenarios,
         validate = self.patchobject(neutron.SubnetConstraint, 'validate')
         validate.return_value = True
 
+    def stub_AddressScopeConstraint_validate(self):
+        validate = self.patchobject(neutron.AddressScopeConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_SubnetPoolConstraint_validate(self):
+        validate = self.patchobject(neutron.SubnetPoolConstraint, 'validate')
+        validate.return_value = True
+
     def stub_RouterConstraint_validate(self):
         validate = self.patchobject(neutron.RouterConstraint, 'validate')
         validate.return_value = True
 
+    def stub_QoSPolicyConstraint_validate(self):
+        validate = self.patchobject(neutron.QoSPolicyConstraint, 'validate')
+        validate.return_value = True
+
     def stub_NovaNetworkConstraint(self):
         validate = self.patchobject(nova.NetworkConstraint, 'validate')
         validate.return_value = True
@@ -285,3 +296,11 @@ class HeatTestCase(testscenarios.WithScenarios,
     def stub_SaharaPluginConstraint(self):
         validate = self.patchobject(sahara.PluginConstraint, 'validate')
         validate.return_value = True
+
+    def stub_ProviderConstraint_validate(self):
+        validate = self.patchobject(neutron.ProviderConstraint, 'validate')
+        validate.return_value = True
+
+    def stub_SecretConstraint_validate(self):
+        validate = self.patchobject(barbican.SecretConstraint, 'validate')
+        validate.return_value = True
