@@ -20,6 +20,7 @@ from oslo_serialization import jsonutils as json
 from oslo_utils import timeutils
 import six
 
+from heat.common import crypt
 from heat.common import exception
 from heat.common import template_format
 from heat.db import api as db_api
@@ -28,6 +29,7 @@ from heat.engine.clients.os import zaqar
 from heat.engine import service
 from heat.engine import service_software_config
 from heat.objects import resource as resource_objects
+from heat.objects import software_config as software_config_object
 from heat.objects import software_deployment as software_deployment_object
 from heat.tests import common
 from heat.tests.engine import tools
@@ -85,6 +87,18 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         return self.engine.create_software_config(
             self.ctx, group, name, config, inputs, outputs, options)
 
+    def _create_dummy_config_object(self):
+        obj_config = software_config_object.SoftwareConfig()
+        obj_config['id'] = str(uuid.uuid4())
+        obj_config['name'] = 'myconfig'
+        obj_config['group'] = 'mygroup'
+        obj_config['config'] = {'config': 'hello world',
+                                'inputs': [],
+                                'outputs': [],
+                                'options': {}}
+        obj_config['created_at'] = timeutils.utcnow()
+        return obj_config
+
     def assert_status_reason(self, expected, actual):
         expected_dict = dict((i.split(' : ') for i in expected.split(', ')))
         actual_dict = dict((i.split(' : ') for i in actual.split(', ')))
@@ -274,6 +288,15 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
             ctx, server_id=server_id)
         self.assertEqual(0, len(metadata))
 
+        # assert None config is filtered out
+        obj_conf = self._create_dummy_config_object()
+        side_effect = [obj_conf, obj_conf, None]
+        self.patchobject(software_config_object.SoftwareConfig,
+                         '_from_db_object', side_effect=side_effect)
+        metadata = self.engine.metadata_software_deployments(
+            self.ctx, server_id=server_id)
+        self.assertEqual(2, len(metadata))
+
     def test_show_software_deployment(self):
         deployment_id = str(uuid.uuid4())
         ex = self.assertRaises(dispatcher.ExpectedException,
@@ -511,9 +534,7 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         self.assertEqual('DEPLOY', updated['action'])
         self.assertEqual('WAITING', updated['status'])
 
-        sd = software_deployment_object.SoftwareDeployment.get_by_id(
-            self.ctx, deployment_id)
-        mock_push.assert_called_once_with(self.ctx, server_id, sd)
+        mock_push.assert_called_once_with(self.ctx, server_id, None)
 
     def test_update_software_deployment_fields(self):
 
@@ -543,7 +564,9 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         check_software_deployment_updated(status='COMPLETE')
         check_software_deployment_updated(status_reason='Done!')
 
-    def test_delete_software_deployment(self):
+    @mock.patch.object(service_software_config.SoftwareConfigService,
+                       '_push_metadata_software_deployments')
+    def test_delete_software_deployment(self, pmsd):
         deployment_id = str(uuid.uuid4())
         ex = self.assertRaises(dispatcher.ExpectedException,
                                self.engine.delete_software_deployment,
@@ -553,11 +576,19 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         deployment = self._create_software_deployment()
         self.assertIsNotNone(deployment)
         deployment_id = deployment['id']
+
         deployments = self.engine.list_software_deployments(
             self.ctx, server_id=None)
         deployment_ids = [x['id'] for x in deployments]
         self.assertIn(deployment_id, deployment_ids)
         self.engine.delete_software_deployment(self.ctx, deployment_id)
+
+        # assert one call for the create, and one for the delete
+        pmsd.assert_has_calls([
+            mock.call(self.ctx, deployment['server_id'], None),
+            mock.call(self.ctx, deployment['server_id'], None)
+        ])
+
         deployments = self.engine.list_software_deployments(
             self.ctx, server_id=None)
         deployment_ids = [x['id'] for x in deployments]
@@ -612,7 +643,7 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         md_sd.return_value = deployments
 
         self.assertRaises(
-            exception.DeploymentConcurrentTransaction,
+            exception.ConcurrentTransaction,
             self.engine.software_config._push_metadata_software_deployments,
             self.ctx,
             '1234',
@@ -672,8 +703,6 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         rs.data = [rd]
         res_get.return_value = rs
         res_upd.return_value = 1
-        sd = mock.Mock()
-        sd.stack_user_project_id = 'project1'
         queue = mock.Mock()
         zaqar_client = mock.Mock()
         plugin.return_value = zaqar_client
@@ -688,11 +717,11 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
         }
 
         self.engine.software_config._push_metadata_software_deployments(
-            self.ctx, '1234', sd)
+            self.ctx, '1234', 'project1')
         res_upd.assert_called_once_with(
             self.ctx, '1234', {'rsrc_metadata': result_metadata}, 1)
 
-        plugin.assert_called_once_with('project1')
+        plugin.assert_called_once_with('project1', mock.ANY)
         zaqar_client.queue.assert_called_once_with('6789')
         queue.post.assert_called_once_with(
             {'body': result_metadata, 'ttl': 3600})
@@ -815,8 +844,29 @@ class SoftwareConfigServiceTest(common.HeatTestCase):
 
     @mock.patch.object(service_software_config.SoftwareConfigService,
                        'signal_software_deployment')
+    @mock.patch.object(service_software_config.SoftwareConfigService,
+                       'metadata_software_deployments')
+    @mock.patch.object(db_api, 'resource_update')
+    @mock.patch.object(db_api, 'resource_get_by_physical_resource_id')
     @mock.patch.object(zaqar.ZaqarClientPlugin, 'create_for_tenant')
-    def test_refresh_zaqar_software_deployment(self, plugin, ssd):
+    def test_refresh_zaqar_software_deployment(self, plugin, res_get, res_upd,
+                                               md_sd, ssd):
+        rs = mock.Mock()
+        rs.rsrc_metadata = {}
+        rs.id = '1234'
+        rs.atomic_key = 1
+        rd1 = mock.Mock()
+        rd1.key = 'user'
+        rd1.value = 'user1'
+        rd2 = mock.Mock()
+        rd2.key = 'password'
+        rd2.decrypt_method, rd2.value = crypt.encrypt('pass1')
+        rs.data = [rd1, rd2]
+        res_get.return_value = rs
+
+        res_upd.return_value = 1
+        deployments = {'deploy': 'this'}
+        md_sd.return_value = deployments
         config = self._create_software_config(inputs=[
             {
                 'name': 'deploy_signal_transport',
