@@ -21,7 +21,10 @@ import six
 from heat.common import exception
 from heat.common import messaging
 from heat.common import template_format
+from heat.engine.clients.os import glance
+from heat.engine.clients.os import nova
 from heat.engine import environment
+from heat.engine import resource
 from heat.engine import service
 from heat.engine import stack
 from heat.engine import stack_lock
@@ -78,7 +81,8 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         mock_env.assert_called_once_with(params)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
-            convergence=False, current_traversal=None,
+            convergence=False,
+            current_traversal=old_stack.current_traversal,
             prev_raw_template_id=None,
             current_deps=None,
             disable_rollback=True,
@@ -94,6 +98,37 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         mock_load.assert_called_once_with(self.ctx, stack=s)
         mock_validate.assert_called_once_with()
 
+    def test_stack_update_with_environment_files(self):
+        # Setup
+        stack_name = 'service_update_env_files_stack'
+        params = {}
+        template = '{ "Template": "data" }'
+        old_stack = tools.get_stack(stack_name, self.ctx)
+        sid = old_stack.store()
+        old_stack.set_stack_user_project_id('1234')
+        stack_object.Stack.get_by_id(self.ctx, sid)
+
+        stk = tools.get_stack(stack_name, self.ctx)
+
+        # prepare mocks
+        self.patchobject(stack, 'Stack', return_value=stk)
+        self.patchobject(stack.Stack, 'load', return_value=old_stack)
+        self.patchobject(templatem, 'Template', return_value=stk.t)
+        self.patchobject(environment, 'Environment', return_value=stk.env)
+        self.patchobject(stk, 'validate', return_value=None)
+        self.patchobject(grevent, 'Event', return_value=mock.Mock())
+
+        mock_merge = self.patchobject(self.man, '_merge_environments')
+
+        # Test
+        environment_files = ['env_1']
+        self.man.update_stack(self.ctx, old_stack.identifier(),
+                              template, params, None, {},
+                              environment_files=environment_files)
+
+        # Verify
+        mock_merge.assert_called_once_with(environment_files, None, params)
+
     def test_stack_update_existing_parameters(self):
         # Use a template with existing parameters, then update the stack
         # with a template containing additional parameters and ensure all
@@ -102,6 +137,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         stack_name = 'service_update_test_stack_existing_parameters'
         update_params = {'encrypted_param_names': [],
                          'parameter_defaults': {},
+                         'event_sinks': [],
                          'parameters': {'newparam': 123},
                          'resource_registry': {'resources': {}}}
         api_args = {rpc_api.PARAM_TIMEOUT: 60,
@@ -128,13 +164,16 @@ class ServiceStackUpdateTest(common.HeatTestCase):
             self.assertEqual(stk.identifier(), result)
 
     def test_stack_update_existing_parameters_remove(self):
-        '''Use a template with existing parameters, then update with a
+        """Test case for updating stack with changed parameters.
+
+        Use a template with existing parameters, then update with a
         template containing additional parameters and a list of
         parameters to be removed.
-        '''
+        """
         stack_name = 'service_update_test_stack_existing_parameters_remove'
         update_params = {'encrypted_param_names': [],
                          'parameter_defaults': {},
+                         'event_sinks': [],
                          'parameters': {'newparam': 123},
                          'resource_registry': {'resources': {}}}
         api_args = {rpc_api.PARAM_TIMEOUT: 60,
@@ -175,6 +214,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         intial_params = {'encrypted_param_names': [],
                          'parameter_defaults': {},
                          'parameters': {},
+                         'event_sinks': [],
                          'resource_registry': intital_registry}
         initial_files = {'foo.yaml': 'foo',
                          'foo2.yaml': 'foo2',
@@ -205,6 +245,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         expected_env = {'encrypted_param_names': [],
                         'parameter_defaults': {},
                         'parameters': {},
+                        'event_sinks': [],
                         'resource_registry': expected_reg}
         # FIXME(shardy): Currently we don't prune unused old files
         expected_files = {'foo.yaml': 'foo',
@@ -229,9 +270,11 @@ class ServiceStackUpdateTest(common.HeatTestCase):
             self.assertEqual(stk.identifier(), result)
 
     def test_stack_update_existing_parameter_defaults(self):
-        '''Use a template with existing flag and ensure the
-        environment parameter_defaults are preserved.
-        '''
+        """Ensure the environment parameter_defaults are preserved.
+
+        Use a template with existing flag and ensure the environment
+        parameter_defaults are preserved.
+        """
         stack_name = 'service_update_test_stack_existing_param_defaults'
         intial_params = {'encrypted_param_names': [],
                          'parameter_defaults': {'mydefault': 123},
@@ -253,6 +296,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
                             'mydefault': 123,
                             'default2': 456},
                         'parameters': {},
+                        'event_sinks': [],
                         'resource_registry': {'resources': {}}}
         with mock.patch('heat.engine.stack.Stack') as mock_stack:
             stk.update = mock.Mock()
@@ -304,7 +348,8 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         mock_load.assert_called_once_with(self.ctx, stack=s)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
-            convergence=False, current_traversal=None,
+            convergence=False,
+            current_traversal=old_stack.current_traversal,
             prev_raw_template_id=None, current_deps=None,
             disable_rollback=False, nested_depth=0,
             owner_id=None, parent_resource=None,
@@ -366,7 +411,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
 
         self.assertEqual(exception.NotSupported, ex.exc_info[0])
         self.assertIn("Cancelling update when stack is "
-                      "('UPDATE', 'COMPLETE')",
+                      "UPDATE_COMPLETE",
                       six.text_type(ex.exc_info[1]))
 
     @mock.patch.object(stack_object.Stack, 'count_total_resources')
@@ -416,7 +461,8 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         mock_env.assert_called_once_with(params)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
-            convergence=False, current_traversal=None,
+            convergence=False,
+            current_traversal=old_stack.current_traversal,
             prev_raw_template_id=None, current_deps=None,
             disable_rollback=True, nested_depth=0,
             owner_id=None, parent_resource=None,
@@ -531,7 +577,8 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         mock_env.assert_called_once_with(params)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
-            convergence=False, current_traversal=None,
+            convergence=False,
+            current_traversal=old_stack.current_traversal,
             prev_raw_template_id=None, current_deps=None,
             disable_rollback=True, nested_depth=0,
             owner_id=None, parent_resource=None,
@@ -552,7 +599,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
                                self.man.update_stack,
                                self.ctx, stk.identifier(), template,
                                params, None, {})
-        self.assertEqual(exception.StackNotFound, ex.exc_info[0])
+        self.assertEqual(exception.EntityNotFound, ex.exc_info[0])
 
     def test_stack_update_no_credentials(self):
         cfg.CONF.set_default('deferred_auth_method', 'password')
@@ -593,7 +640,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         mock_env.assert_called_once_with(params)
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t,
-            convergence=False, current_traversal=None,
+            convergence=False, current_traversal=stk.current_traversal,
             prev_raw_template_id=None, current_deps=None,
             disable_rollback=True, nested_depth=0,
             owner_id=None, parent_resource=None,
@@ -618,8 +665,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         stack = utils.parse_stack(t, stack_name=stack_name,
                                   params=params)
         stack.set_stack_user_project_id('1234')
-        self.assertEqual(stack.t.t,
-                         t)
+        self.assertEqual(t, stack.t.t)
         stack.action = stack.CREATE
         stack.status = stack.COMPLETE
 
@@ -651,8 +697,7 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         stack = utils.parse_stack(t, stack_name=stack_name,
                                   params=params)
         stack.set_stack_user_project_id('1234')
-        self.assertEqual(stack.t.t,
-                         t)
+        self.assertEqual(t, stack.t.t)
         stack.action = stack.UPDATE
         stack.status = stack.FAILED
 
@@ -665,6 +710,112 @@ class ServiceStackUpdateTest(common.HeatTestCase):
         self.assertIn("PATCH update to non-COMPLETE stack",
                       six.text_type(ex.exc_info[1]))
 
+    def test_update_immutable_parameter_disallowed(self):
+
+        template = '''
+heat_template_version: 2014-10-16
+parameters:
+  param1:
+    type: string
+    immutable: true
+    default: foo
+'''
+
+        self.ctx = utils.dummy_context(password=None)
+        stack_name = 'test_update_immutable_parameters'
+        params = {}
+        old_stack = tools.get_stack(stack_name, self.ctx,
+                                    template=template)
+        sid = old_stack.store()
+        old_stack.set_stack_user_project_id('1234')
+        s = stack_object.Stack.get_by_id(self.ctx, sid)
+
+        # prepare mocks
+        self.patchobject(self.man, '_get_stack', return_value=s)
+        self.patchobject(stack, 'Stack', return_value=old_stack)
+        self.patchobject(stack.Stack, 'load', return_value=old_stack)
+        self.patchobject(templatem, 'Template', return_value=old_stack.t)
+        self.patchobject(environment, 'Environment',
+                         return_value=old_stack.env)
+
+        params = {'param1': 'bar'}
+        exc = self.assertRaises(dispatcher.ExpectedException,
+                                self.man.update_stack,
+                                self.ctx, old_stack.identifier(),
+                                templatem.Template(template), params,
+                                None, {})
+        self.assertEqual(exception.ImmutableParameterModified, exc.exc_info[0])
+        self.assertEqual('The following parameters are immutable and may not '
+                         'be updated: param1', exc.exc_info[1].message)
+
+    def test_update_mutable_parameter_allowed(self):
+
+        template = '''
+heat_template_version: 2014-10-16
+parameters:
+  param1:
+    type: string
+    immutable: false
+    default: foo
+'''
+
+        self.ctx = utils.dummy_context(password=None)
+        stack_name = 'test_update_immutable_parameters'
+        params = {}
+        old_stack = tools.get_stack(stack_name, self.ctx,
+                                    template=template)
+        sid = old_stack.store()
+        old_stack.set_stack_user_project_id('1234')
+        s = stack_object.Stack.get_by_id(self.ctx, sid)
+
+        # prepare mocks
+        self.patchobject(self.man, '_get_stack', return_value=s)
+        self.patchobject(stack, 'Stack', return_value=old_stack)
+        self.patchobject(stack.Stack, 'load', return_value=old_stack)
+        self.patchobject(templatem, 'Template', return_value=old_stack.t)
+        self.patchobject(environment, 'Environment',
+                         return_value=old_stack.env)
+
+        params = {'param1': 'bar'}
+        result = self.man.update_stack(self.ctx, old_stack.identifier(),
+                                       templatem.Template(template), params,
+                                       None, {})
+        self.assertEqual(s.id, result['stack_id'])
+
+    def test_update_immutable_parameter_same_value(self):
+
+        template = '''
+heat_template_version: 2014-10-16
+parameters:
+  param1:
+    type: string
+    immutable: true
+    default: foo
+'''
+
+        self.ctx = utils.dummy_context(password=None)
+        stack_name = 'test_update_immutable_parameters'
+        params = {}
+        old_stack = tools.get_stack(stack_name, self.ctx,
+                                    template=template)
+        sid = old_stack.store()
+        old_stack.set_stack_user_project_id('1234')
+        s = stack_object.Stack.get_by_id(self.ctx, sid)
+
+        # prepare mocks
+        self.patchobject(self.man, '_get_stack', return_value=s)
+        self.patchobject(stack, 'Stack', return_value=old_stack)
+        self.patchobject(stack.Stack, 'load', return_value=old_stack)
+        self.patchobject(templatem, 'Template', return_value=old_stack.t)
+        self.patchobject(environment, 'Environment',
+                         return_value=old_stack.env)
+
+        params = {'param1': 'foo'}
+        result = self.man.update_stack(self.ctx, old_stack.identifier(),
+                                       templatem.Template(template), params,
+                                       None, {})
+        self.assertEqual(s.id, result['stack_id'])
+
 
 class ServiceStackUpdatePreviewTest(common.HeatTestCase):
 
@@ -702,9 +853,18 @@ resources:
         self.man = service.EngineService('a-host', 'a-topic')
         self.man.thread_group_mgr = tools.DummyThreadGroupManager()
 
-    def _test_stack_update_preview(self, orig_template, new_template):
+    def _test_stack_update_preview(self, orig_template, new_template,
+                                   environment_files=None):
         stack_name = 'service_update_test_stack_preview'
         params = {'foo': 'bar'}
+
+        def side_effect(*args):
+            return 2 if args[0] == 'm1.small' else 1
+
+        self.patchobject(nova.NovaClientPlugin, 'find_flavor_by_name_or_id',
+                         side_effect=side_effect)
+        self.patchobject(glance.GlanceClientPlugin, 'find_image_by_name_or_id',
+                         return_value=1)
         old_stack = tools.get_stack(stack_name, self.ctx,
                                     template=orig_template)
         sid = old_stack.store()
@@ -721,21 +881,29 @@ resources:
         mock_env = self.patchobject(environment, 'Environment',
                                     return_value=stk.env)
         mock_validate = self.patchobject(stk, 'validate', return_value=None)
+        mock_merge = self.patchobject(self.man, '_merge_environments')
+
+        # Patch _resolve_all_attributes or it tries to call novaclient
+        self.patchobject(resource.Resource, '_resolve_all_attributes',
+                         return_value=None)
 
         # do preview_update_stack
         api_args = {'timeout_mins': 60}
-        result = self.man.preview_update_stack(self.ctx,
-                                               old_stack.identifier(),
-                                               new_template, params, None,
-                                               api_args)
+        result = self.man.preview_update_stack(
+            self.ctx,
+            old_stack.identifier(),
+            new_template, params, None,
+            api_args,
+            environment_files=environment_files)
+
         # assertions
         mock_stack.assert_called_once_with(
             self.ctx, stk.name, stk.t, convergence=False,
-            current_traversal=None, prev_raw_template_id=None,
-            current_deps=None, disable_rollback=True,
-            nested_depth=0, owner_id=None, parent_resource=None,
-            stack_user_project_id='1234', strict_validate=True,
-            tenant_id='test_tenant_id', timeout_mins=60,
+            current_traversal=old_stack.current_traversal,
+            prev_raw_template_id=None, current_deps=None,
+            disable_rollback=True, nested_depth=0, owner_id=None,
+            parent_resource=None, stack_user_project_id='1234',
+            strict_validate=True, tenant_id='test_tenant_id', timeout_mins=60,
             user_creds_id=u'1', username='test_username')
         mock_load.assert_called_once_with(self.ctx, stack=s)
         mock_tmpl.assert_called_once_with(new_template, files=None,
@@ -743,20 +911,24 @@ resources:
         mock_env.assert_called_once_with(params)
         mock_validate.assert_called_once_with()
 
+        if environment_files:
+            mock_merge.assert_called_once_with(environment_files, None, params)
+
         return result
 
     def test_stack_update_preview_added_unchanged(self):
         result = self._test_stack_update_preview(self.old_tmpl, self.new_tmpl)
 
         added = [x for x in result['added']][0]
-        self.assertEqual(added['resource_name'], 'password')
+        self.assertEqual('password', added['resource_name'])
         unchanged = [x for x in result['unchanged']][0]
-        self.assertEqual(unchanged['resource_name'], 'web_server')
+        self.assertEqual('web_server', unchanged['resource_name'])
+        self.assertNotEqual('None', unchanged['resource_identity']['stack_id'])
 
         empty_sections = ('deleted', 'replaced', 'updated')
         for section in empty_sections:
             section_contents = [x for x in result[section]]
-            self.assertEqual(section_contents, [])
+            self.assertEqual([], section_contents)
 
     def test_stack_update_preview_replaced(self):
         # new template with a different key_name
@@ -765,34 +937,56 @@ resources:
         result = self._test_stack_update_preview(self.old_tmpl, new_tmpl)
 
         replaced = [x for x in result['replaced']][0]
-        self.assertEqual(replaced['resource_name'], 'web_server')
+        self.assertEqual('web_server', replaced['resource_name'])
         empty_sections = ('added', 'deleted', 'unchanged', 'updated')
         for section in empty_sections:
             section_contents = [x for x in result[section]]
-            self.assertEqual(section_contents, [])
+            self.assertEqual([], section_contents)
+
+    def test_stack_update_preview_replaced_type(self):
+        # new template with a different type for web_server
+        new_tmpl = self.old_tmpl.replace('OS::Nova::Server', 'OS::Heat::None')
+
+        result = self._test_stack_update_preview(self.old_tmpl, new_tmpl)
+
+        replaced = [x for x in result['replaced']][0]
+        self.assertEqual('web_server', replaced['resource_name'])
+        empty_sections = ('added', 'deleted', 'unchanged', 'updated')
+        for section in empty_sections:
+            section_contents = [x for x in result[section]]
+            self.assertEqual([], section_contents)
 
     def test_stack_update_preview_updated(self):
         # new template changes to flavor of server
         new_tmpl = self.old_tmpl.replace('m1.large', 'm1.small')
-
         result = self._test_stack_update_preview(self.old_tmpl, new_tmpl)
 
         updated = [x for x in result['updated']][0]
-        self.assertEqual(updated['resource_name'], 'web_server')
+        self.assertEqual('web_server', updated['resource_name'])
         empty_sections = ('added', 'deleted', 'unchanged', 'replaced')
         for section in empty_sections:
             section_contents = [x for x in result[section]]
-            self.assertEqual(section_contents, [])
+            self.assertEqual([], section_contents)
 
     def test_stack_update_preview_deleted(self):
         # do the reverse direction, i.e. delete resources
         result = self._test_stack_update_preview(self.new_tmpl, self.old_tmpl)
 
         deleted = [x for x in result['deleted']][0]
-        self.assertEqual(deleted['resource_name'], 'password')
+        self.assertEqual('password', deleted['resource_name'])
         unchanged = [x for x in result['unchanged']][0]
-        self.assertEqual(unchanged['resource_name'], 'web_server')
+        self.assertEqual('web_server', unchanged['resource_name'])
         empty_sections = ('added', 'updated', 'replaced')
         for section in empty_sections:
             section_contents = [x for x in result[section]]
-            self.assertEqual(section_contents, [])
+            self.assertEqual([], section_contents)
+
+    def test_stack_update_preview_with_environment_files(self):
+        # Setup
+        environment_files = ['env_1']
+
+        # Test
+        self._test_stack_update_preview(self.old_tmpl, self.new_tmpl,
+                                        environment_files=environment_files)
+
+        # Assertions done in _test_stack_update_preview
