@@ -62,7 +62,7 @@ class ResourceTypeTest(common.HeatTestCase):
             mock_is_service_available):
         mock_is_service_available.return_value = False
         resources = self.eng.list_resource_types(self.ctx)
-        # Check for an known resource, not listed
+        # Check for a known resource, not listed
         self.assertNotIn('OS::Nova::Server', resources)
 
     def test_resource_schema(self):
@@ -101,16 +101,6 @@ class ResourceTypeTest(common.HeatTestCase):
         schema = self.eng.resource_schema(self.ctx, type_name=type_name)
         self.assertEqual(expected, schema)
 
-    def test_resource_schema_for_hidden_type(self):
-        type_name = 'ResourceTypeHidden'
-        self.assertRaises(exception.NotSupported, self.eng.resource_schema,
-                          self.ctx, type_name)
-
-    def test_generate_template_for_hidden_type(self):
-        type_name = 'ResourceTypeHidden'
-        self.assertRaises(exception.NotSupported, self.eng.generate_template,
-                          self.ctx, type_name)
-
     def test_resource_schema_with_attr_type(self):
 
         type_name = 'ResourceWithAttributeType'
@@ -159,7 +149,7 @@ class ResourceTypeTest(common.HeatTestCase):
         self._no_template_file(self.eng.generate_template)
 
     def test_resource_schema_nonexist(self):
-        ex = self.assertRaises(exception.ResourceTypeNotFound,
+        ex = self.assertRaises(exception.EntityNotFound,
                                self.eng.resource_schema,
                                self.ctx, type_name='Bogus')
         msg = 'The Resource Type (Bogus) could not be found.'
@@ -176,10 +166,9 @@ class ResourceTypeTest(common.HeatTestCase):
                                    self.eng.resource_schema,
                                    self.ctx,
                                    type_name)
-
-            msg = ('Service sample does not have required endpoint in service'
-                   ' catalog for the resource type'
-                   ' ResourceWithDefaultClientName')
+            msg = ('HEAT-E99001 Service sample is not available for resource '
+                   'type ResourceWithDefaultClientName, reason: '
+                   'Service endpoint not in service catalog.')
             self.assertEqual(msg,
                              six.text_type(ex),
                              'invalid exception message')
