@@ -24,7 +24,7 @@ from heat.engine.resources.aws.ec2 import instance as instances
 from heat.engine import stack as parser
 from heat.engine import template as templatem
 from heat.tests import fakes as test_fakes
-from heat.tests.nova import fakes as fakes_nova
+from heat.tests.openstack.nova import fakes as fakes_nova
 from heat.tests import utils
 
 wp_template = u'''
@@ -170,13 +170,14 @@ def setup_keystone_mocks(mocks, stack):
     fkc = test_fakes.FakeKeystoneClient()
 
     mocks.StubOutWithMock(keystone.KeystoneClientPlugin, '_create')
-    keystone.KeystoneClientPlugin._create().MultipleTimes().AndReturn(fkc)
+    keystone.KeystoneClientPlugin._create().AndReturn(fkc)
 
 
 def setup_mock_for_image_constraint(mocks, imageId_input,
                                     imageId_output=744):
-    mocks.StubOutWithMock(glance.GlanceClientPlugin, 'get_image_id')
-    glance.GlanceClientPlugin.get_image_id(
+    mocks.StubOutWithMock(glance.GlanceClientPlugin,
+                          'find_image_by_name_or_id')
+    glance.GlanceClientPlugin.find_image_by_name_or_id(
         imageId_input).MultipleTimes().AndReturn(imageId_output)
 
 
@@ -221,8 +222,8 @@ def setup_mocks(mocks, stack, mock_image_constraint=True,
     return fc
 
 
-def setup_stack(stack_name, ctx, create_res=True):
-    stack = get_stack(stack_name, ctx)
+def setup_stack(stack_name, ctx, create_res=True, convergence=False):
+    stack = get_stack(stack_name, ctx, convergence=convergence)
     stack.store()
     if create_res:
         m = mox.Mox()
@@ -249,11 +250,12 @@ def clean_up_stack(stack, delete_res=True):
         m.UnsetStubs()
 
 
-def stack_context(stack_name, create_res=True):
-    """
+def stack_context(stack_name, create_res=True, convergence=False):
+    """Decorator for creating and deleting stack.
+
     Decorator which creates a stack by using the test case's context and
     deletes it afterwards to ensure tests clean up their stacks regardless
-    of test success/failure
+    of test success/failure.
     """
     def stack_delete(test_fn):
         @six.wraps(test_fn)
@@ -261,7 +263,8 @@ def stack_context(stack_name, create_res=True):
             def create_stack():
                 ctx = getattr(test_case, 'ctx', None)
                 if ctx is not None:
-                    stack = setup_stack(stack_name, ctx, create_res)
+                    stack = setup_stack(stack_name, ctx,
+                                        create_res, convergence)
                     setattr(test_case, 'stack', stack)
 
             def delete_stack():
