@@ -11,8 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-A fake server that "responds" to API methods with pre-canned responses.
+"""A fake server that "responds" to API methods with pre-canned responses.
 
 All of these responses come from the spec, so if for some reason the spec's
 wrong the tests might raise AssertionError. I've indicated in comments the
@@ -28,9 +27,7 @@ from heat.common import context
 class FakeClient(object):
 
     def assert_called(self, method, url, body=None, pos=-1):
-        """
-        Assert than an API method was just called.
-        """
+        """Assert that an API method was just called."""
         expected = (method, url)
         called = self.client.callstack[pos][0:2]
 
@@ -44,9 +41,7 @@ class FakeClient(object):
             assert self.client.callstack[pos][2] == body
 
     def assert_called_anytime(self, method, url, body=None):
-        """
-        Assert than an API method was called anytime in the test.
-        """
+        """Assert that an API method was called anytime in the test."""
         expected = (method, url)
 
         assert self.client.callstack, ("Expected %s %s but no calls "
